@@ -317,8 +317,7 @@ def AddNode(opts, args):
              "Performing this operation is going to perform the following\n"
              "changes to the target machine (%s) and the current cluster\n"
              "nodes:\n"
-             "* A new SSH daemon key pair is generated is generated on\n"
-             "  the target machine.\n"
+             "* A new SSH daemon key pair is generated on the target machine.\n"
              "* The public SSH keys of all master candidates of the cluster\n"
              "  are added to the target machine's 'authorized_keys' file.\n"
              "* In case the target machine is a master candidate, its newly\n"
