@@ -39,13 +39,12 @@ from ganeti import errors
 from ganeti import locking
 from ganeti import masterd
 from ganeti import utils
-from ganeti.utils import retry
 
 from ganeti.cmdlib.base import NoHooksLU, LogicalUnit
-from ganeti.cmdlib.common import CheckNodeOnline, ExpandNodeUuidAndName, \
-  IsInstanceRunning, DetermineImageSize
+from ganeti.cmdlib.common import CheckNodeOnline, ExpandNodeUuidAndName
+from ganeti.cmdlib.instance_helpervm import RunWithHelperVM
 from ganeti.cmdlib.instance_storage import StartInstanceDisks, \
-  ShutdownInstanceDisks, TemporaryDisk, ImageDisks
+  ShutdownInstanceDisks
 from ganeti.cmdlib.instance_utils import GetClusterDomainSecret, \
   BuildInstanceHookEnvByObject, CheckNodeNotDrained, RemoveInstance, \
   CheckCompressionTool
@@ -157,11 +156,6 @@ class LUBackupExport(LogicalUnit):
       #  - removing the removal operation altogether
       self.needed_locks[locking.LEVEL_NODE] = locking.ALL_SET
 
-      # Allocations should be stopped while this LU runs with node locks, but
-      # it doesn't have to be exclusive
-      self.share_locks[locking.LEVEL_NODE_ALLOC] = 1
-      self.needed_locks[locking.LEVEL_NODE_ALLOC] = locking.ALL_SET
-
   def DeclareLocks(self, level):
     """Last minute lock declaration."""
     # All nodes are locked anyway, so nothing to do here.
@@ -283,13 +277,6 @@ class LUBackupExport(LogicalUnit):
       raise errors.ProgrammerError("Unhandled export mode %r" %
                                    self.op.mode)
 
-    # instance disk type verification
-    # TODO: Implement export support for file-based disks
-    for disk in self.cfg.GetInstanceDisks(self.instance.uuid):
-      if disk.dev_type in constants.DTS_FILEBASED:
-        raise errors.OpPrereqError("Export not supported for instances with"
-                                   " file-based disks", errors.ECODE_INVAL)
-
     # Check prerequisites for zeroing
     if self.op.zero_free_space:
       # Check that user shutdown detection has been enabled
@@ -297,18 +284,19 @@ class LUBackupExport(LogicalUnit):
       if self.instance.hypervisor == constants.HT_KVM and \
          not hvparams.get(constants.HV_KVM_USER_SHUTDOWN, False):
         raise errors.OpPrereqError("Instance shutdown detection must be "
-                                   "enabled for zeroing to work")
+                                   "enabled for zeroing to work",
+                                   errors.ECODE_INVAL)
 
       # Check that the instance is set to boot from the disk
       if constants.HV_BOOT_ORDER in hvparams and \
          hvparams[constants.HV_BOOT_ORDER] != constants.HT_BO_DISK:
         raise errors.OpPrereqError("Booting from disk must be set for zeroing "
-                                   "to work")
+                                   "to work", errors.ECODE_INVAL)
 
       # Check that the zeroing image is set
       if not self.cfg.GetZeroingImage():
         raise errors.OpPrereqError("A zeroing image must be set for zeroing to"
-                                   " work")
+                                   " work", errors.ECODE_INVAL)
 
       if self.op.zeroing_timeout_fixed is None:
         self.op.zeroing_timeout_fixed = constants.HELPER_VM_STARTUP
@@ -320,7 +308,13 @@ class LUBackupExport(LogicalUnit):
       if (self.op.zeroing_timeout_fixed is not None or
           self.op.zeroing_timeout_per_mib is not None):
         raise errors.OpPrereqError("Zeroing timeout options can only be used"
-                                   " only with the --zero-free-space option")
+                                   " only with the --zero-free-space option",
+                                   errors.ECODE_INVAL)
+
+    if self.op.long_sleep and not self.op.shutdown:
+      raise errors.OpPrereqError("The long sleep option only makes sense when"
+                                 " the instance can be shut down.",
+                                 errors.ECODE_INVAL)
 
     self.secondary_nodes = \
       self.cfg.GetInstanceSecondaryNodes(self.instance.uuid)
@@ -379,54 +373,44 @@ class LUBackupExport(LogicalUnit):
     assert self.op.zeroing_timeout_per_mib is not None
 
     zeroing_image = self.cfg.GetZeroingImage()
-    src_node_uuid = self.instance.primary_node
-
-    try:
-      disk_size = DetermineImageSize(self, zeroing_image, src_node_uuid)
-    except errors.OpExecError, err:
-      raise errors.OpExecError("Could not create temporary disk for zeroing:"
-                               " %s", err)
 
     # Calculate the sum prior to adding the temporary disk
     instance_disks_size_sum = self._InstanceDiskSizeSum()
+    timeout = self.op.zeroing_timeout_fixed + \
+              self.op.zeroing_timeout_per_mib * instance_disks_size_sum
 
-    with TemporaryDisk(self,
-                       self.instance,
-                       [(constants.DT_PLAIN, constants.DISK_RDWR, disk_size)],
-                       feedback_fn):
-      feedback_fn("Activating instance disks")
-      StartInstanceDisks(self, self.instance, False)
+    RunWithHelperVM(self, self.instance, zeroing_image,
+                    self.op.shutdown_timeout, timeout,
+                    log_prefix="Zeroing free disk space",
+                    feedback_fn=feedback_fn)
 
-      feedback_fn("Imaging disk with zeroing image")
-      ImageDisks(self, self.instance, zeroing_image)
+  def StartInstance(self, feedback_fn, src_node_uuid):
+    """Send the node instructions to start the instance.
 
-      feedback_fn("Starting instance with zeroing image")
-      result = self.rpc.call_instance_start(src_node_uuid,
-                                            (self.instance, [], []),
-                                            False, self.op.reason)
-      result.Raise("Could not start instance %s when using the zeroing image "
-                   "%s" % (self.instance.name, zeroing_image))
+    @raise errors.OpExecError: If the instance didn't start up.
 
-      # First wait for the instance to start up
-      running_check = lambda: IsInstanceRunning(self, self.instance,
-                                                prereq=False)
-      instance_up = retry.SimpleRetry(True, running_check, 5.0,
-                                      self.op.shutdown_timeout)
-      if not instance_up:
-        raise errors.OpExecError("Could not boot instance when using the "
-                                 "zeroing image %s" % zeroing_image)
+    """
+    assert self.instance.disks_active
+    feedback_fn("Starting instance %s" % self.instance.name)
+    result = self.rpc.call_instance_start(src_node_uuid,
+                                          (self.instance, None, None),
+                                          False, self.op.reason)
+    msg = result.fail_msg
+    if msg:
+      feedback_fn("Failed to start instance: %s" % msg)
+      ShutdownInstanceDisks(self, self.instance)
+      raise errors.OpExecError("Could not start instance: %s" % msg)
 
-      feedback_fn("Instance is up, now awaiting shutdown")
+  def TrySnapshot(self):
+    """Returns true if there is a reason to prefer a snapshot."""
+    return (not self.op.remove_instance and
+            self.instance.admin_state == constants.ADMINST_UP)
 
-      # Then for it to be finished, detected by its shutdown
-      timeout = self.op.zeroing_timeout_fixed + \
-                self.op.zeroing_timeout_per_mib * instance_disks_size_sum
-      instance_up = retry.SimpleRetry(False, running_check, 20.0, timeout)
-      if instance_up:
-        self.LogWarning("Zeroing not completed prior to timeout; instance will"
-                        "be shut down forcibly")
-
-    feedback_fn("Zeroing completed!")
+  def DoReboot(self):
+    """Returns true iff the instance needs to be started after transfer."""
+    return (self.op.shutdown and
+            self.instance.admin_state == constants.ADMINST_UP and
+            not self.op.remove_instance)
 
   def Exec(self, feedback_fn):
     """Export an instance to an image in the cluster.
@@ -462,22 +446,25 @@ class LUBackupExport(LogicalUnit):
       helper = masterd.instance.ExportInstanceHelper(self, feedback_fn,
                                                      self.instance)
 
-      helper.CreateSnapshots()
-      try:
-        if (self.op.shutdown and
-            self.instance.admin_state == constants.ADMINST_UP and
-            not self.op.remove_instance):
-          assert self.instance.disks_active
-          feedback_fn("Starting instance %s" % self.instance.name)
-          result = self.rpc.call_instance_start(src_node_uuid,
-                                                (self.instance, None, None),
-                                                False, self.op.reason)
-          msg = result.fail_msg
-          if msg:
-            feedback_fn("Failed to start instance: %s" % msg)
-            ShutdownInstanceDisks(self, self.instance)
-            raise errors.OpExecError("Could not start instance: %s" % msg)
+      snapshots_available = False
+      if self.TrySnapshot():
+        snapshots_available = helper.CreateSnapshots()
+        if not snapshots_available:
+          if not self.op.shutdown:
+            raise errors.OpExecError(
+              "Not all disks could be snapshotted, and you requested a live "
+              "export; aborting"
+            )
+          if not self.op.long_sleep:
+            raise errors.OpExecError(
+              "Not all disks could be snapshotted, and you did not allow the "
+              "instance to remain offline for a longer time through the "
+              "--long-sleep option; aborting"
+            )
 
+      try:
+        if self.DoReboot() and snapshots_available:
+          self.StartInstance(feedback_fn, src_node_uuid)
         if self.op.mode == constants.EXPORT_MODE_LOCAL:
           (fin_resu, dresults) = helper.LocalExport(self.dst_node,
                                                     self.op.compress)
@@ -495,6 +482,9 @@ class LUBackupExport(LogicalUnit):
                                                      key_name, dest_ca_pem,
                                                      self.op.compress,
                                                      timeouts)
+
+        if self.DoReboot() and not snapshots_available:
+          self.StartInstance(feedback_fn, src_node_uuid)
       finally:
         helper.Cleanup()
 
@@ -546,15 +536,8 @@ class LUBackupRemove(NoHooksLU):
       # we don't need to lock the instance itself, as nothing will happen to it
       # (and we can remove exports also for a removed instance)
       locking.LEVEL_NODE: locking.ALL_SET,
-
-      # Removing backups is quick, so blocking allocations is justified
-      locking.LEVEL_NODE_ALLOC: locking.ALL_SET,
       }
 
-    # Allocations should be stopped while this LU runs with node locks, but it
-    # doesn't have to be exclusive
-    self.share_locks[locking.LEVEL_NODE_ALLOC] = 1
-
   def Exec(self, feedback_fn):
     """Remove any export.
 
