@@ -89,7 +89,7 @@ class LUWConfdClient(object):
       self.lu.wconfd.Client().ListLocks(self.lu.wconfdcontext)
 
 
-class LogicalUnit(object):
+class LogicalUnit(object): # pylint: disable=R0902
   """Logical Unit base class.
 
   Subclasses must follow these rules:
@@ -108,6 +108,9 @@ class LogicalUnit(object):
       in dry-run mode (signalled by opcode dry_run parameter)
 
   """
+  # This class has more than 20 instance variables, but at most have sensible
+  # defaults and are used in a declartive way, this is not a problem.
+
   HPATH = None
   HTYPE = None
   REQ_BGL = True
@@ -141,6 +144,7 @@ class LogicalUnit(object):
     self.share_locks = dict.fromkeys(locking.LEVELS, 0)
     self.opportunistic_locks = dict.fromkeys(locking.LEVELS, False)
     self.opportunistic_locks_count = dict.fromkeys(locking.LEVELS, 1)
+    self.dont_collate_locks = dict.fromkeys(locking.LEVELS, False)
 
     self.add_locks = {}
 
@@ -185,7 +189,6 @@ class LogicalUnit(object):
       locking.LEVEL_CLUSTER: (lambda: [locking.BGL]),
       locking.LEVEL_INSTANCE:
         lambda: self.cfg.GetInstanceNames(self.cfg.GetInstanceList()),
-      locking.LEVEL_NODE_ALLOC: (lambda: [locking.NAL]),
       locking.LEVEL_NODEGROUP: self.cfg.GetNodeGroupList,
       locking.LEVEL_NODE: self.cfg.GetNodeList,
       locking.LEVEL_NODE_RES: self.cfg.GetNodeList,
@@ -350,6 +353,18 @@ class LogicalUnit(object):
     else:
       raise NotImplementedError
 
+  def PrepareRetry(self, _feedback_fn):
+    """Prepare the LU to run again.
+
+    This method is called if the Exec failed for temporarily lacking resources.
+    It is expected to change the state of the LU so that it can be tried again,
+    and also change its locking policy to acquire more resources to have a
+    better chance of suceeding in the retry.
+
+    """
+    # pylint: disable=R0201
+    raise errors.OpRetryNotSupportedError()
+
   def BuildHooksEnv(self):
     """Build hooks environment for this LU.
 
@@ -424,7 +439,7 @@ class LogicalUnit(object):
     # pylint: disable=W0613,R0201
     return lu_result
 
-  def _ExpandAndLockInstance(self):
+  def _ExpandAndLockInstance(self, allow_forthcoming=False):
     """Helper function to expand and lock an instance.
 
     Many LUs that work on an instance take its name in self.op.instance_name
@@ -433,6 +448,10 @@ class LogicalUnit(object):
     name. It also initializes needed_locks as a dict, if this hasn't been done
     before.
 
+    @param allow_forthcoming: if True, do not insist that the intsance be real;
+        the default behaviour is to raise a prerequisite error if the specified
+        instance is forthcoming.
+
     """
     if self.needed_locks is None:
       self.needed_locks = {}
@@ -443,6 +462,10 @@ class LogicalUnit(object):
       ExpandInstanceUuidAndName(self.cfg, self.op.instance_uuid,
                                 self.op.instance_name)
     self.needed_locks[locking.LEVEL_INSTANCE] = self.op.instance_name
+    if not allow_forthcoming:
+      if self.cfg.GetInstanceInfo(self.op.instance_uuid).forthcoming:
+        raise errors.OpPrereqError(
+          "forthcoming instances not supported for this operation")
 
   def _LockInstancesNodes(self, primary_only=False,
                           level=locking.LEVEL_NODE):
@@ -494,6 +517,12 @@ class LogicalUnit(object):
 
     del self.recalculate_locks[level]
 
+  def AssertReleasedLocks(self, level):
+    """Raise AssertionError if the LU holds some locks of the given level.
+
+    """
+    assert not self.owned_locks(level)
+
 
 class NoHooksLU(LogicalUnit): # pylint: disable=W0223
   """Simple LU which runs no hooks.
