@@ -295,7 +295,7 @@ class LUGroupAssignNodes(NoHooksLU):
     In particular, it returns information about newly split instances, and
     instances that were already split, and remain so after the change.
 
-    Only instances whose disk template is listed in constants.DTS_INT_MIRROR are
+    Only disks whose template is listed in constants.DTS_INT_MIRROR are
     considered.
 
     @type changes: list of (node_uuid, new_group_uuid) pairs.
@@ -315,7 +315,8 @@ class LUGroupAssignNodes(NoHooksLU):
     previously_split_instances = set()
 
     for inst in instance_data.values():
-      if inst.disk_template not in constants.DTS_INT_MIRROR:
+      inst_disks = self.cfg.GetInstanceDisks(inst.uuid)
+      if not utils.AnyDiskOfType(inst_disks, constants.DTS_INT_MIRROR):
         continue
 
       inst_nodes = self.cfg.GetInstanceNodes(inst.uuid)
@@ -846,13 +847,9 @@ class LUGroupVerifyDisks(NoHooksLU):
       locking.LEVEL_INSTANCE: [],
       locking.LEVEL_NODEGROUP: [],
       locking.LEVEL_NODE: [],
-
-      # This opcode is acquires all node locks in a group. LUClusterVerifyDisks
-      # starts one instance of this opcode for every group, which means all
-      # nodes will be locked for a short amount of time, so it's better to
-      # acquire the node allocation lock as well.
-      locking.LEVEL_NODE_ALLOC: locking.ALL_SET,
       }
+    self.dont_collate_locks[locking.LEVEL_NODEGROUP] = True
+    self.dont_collate_locks[locking.LEVEL_NODE] = True
 
   def DeclareLocks(self, level):
     if level == locking.LEVEL_INSTANCE:
@@ -941,7 +938,9 @@ class LUGroupVerifyDisks(NoHooksLU):
   def _VerifyDrbdStates(self, node_errors, offline_disk_instance_names):
     node_to_inst = {}
     for inst in self.instances.values():
-      if not inst.disks_active or inst.disk_template != constants.DT_DRBD8:
+      disks = self.cfg.GetInstanceDisks(inst.uuid)
+      if not (inst.disks_active and
+              utils.AnyDiskOfType(disks, [constants.DT_DRBD8])):
         continue
 
       secondary_nodes = self.cfg.GetInstanceSecondaryNodes(inst.uuid)
