@@ -36,6 +36,7 @@ import time
 from ganeti import constants
 from ganeti import errors
 from ganeti import locking
+from ganeti import hypervisor
 from ganeti.masterd import iallocator
 from ganeti import utils
 from ganeti.cmdlib.base import LogicalUnit, Tasklet
@@ -62,13 +63,11 @@ def _ExpandNamesForMigration(lu):
 
   lu.needed_locks[locking.LEVEL_NODE] = []
   lu.recalculate_locks[locking.LEVEL_NODE] = constants.LOCKS_REPLACE
+  lu.dont_collate_locks[locking.LEVEL_NODE] = True
 
   lu.needed_locks[locking.LEVEL_NODE_RES] = []
   lu.recalculate_locks[locking.LEVEL_NODE_RES] = constants.LOCKS_REPLACE
-
-  # The node allocation lock is actually only needed for externally replicated
-  # instances (e.g. sharedfile or RBD) and if an iallocator is used.
-  lu.needed_locks[locking.LEVEL_NODE_ALLOC] = []
+  lu.dont_collate_locks[locking.LEVEL_NODE_RES] = True
 
 
 def _DeclareLocksForMigration(lu, level):
@@ -78,26 +77,21 @@ def _DeclareLocksForMigration(lu, level):
   @param level: Lock level
 
   """
-  if level == locking.LEVEL_NODE_ALLOC:
+  if level == locking.LEVEL_NODE:
     assert lu.op.instance_name in lu.owned_locks(locking.LEVEL_INSTANCE)
 
     instance = lu.cfg.GetInstanceInfo(lu.op.instance_uuid)
 
-    # Node locks are already declared here rather than at LEVEL_NODE as we need
-    # the instance object anyway to declare the node allocation lock.
-    if instance.disk_template in constants.DTS_EXT_MIRROR:
+    disks = lu.cfg.GetInstanceDisks(instance.uuid)
+    if utils.AnyDiskOfType(disks, constants.DTS_EXT_MIRROR):
       if lu.op.target_node is None:
         lu.needed_locks[locking.LEVEL_NODE] = locking.ALL_SET
-        lu.needed_locks[locking.LEVEL_NODE_ALLOC] = locking.ALL_SET
       else:
         lu.needed_locks[locking.LEVEL_NODE] = [instance.primary_node,
                                                lu.op.target_node_uuid]
-      del lu.recalculate_locks[locking.LEVEL_NODE]
     else:
       lu._LockInstancesNodes() # pylint: disable=W0212
 
-  elif level == locking.LEVEL_NODE:
-    # Node locks are declared together with the node allocation lock
     assert (lu.needed_locks[locking.LEVEL_NODE] or
             lu.needed_locks[locking.LEVEL_NODE] is locking.ALL_SET)
 
@@ -110,6 +104,14 @@ def _DeclareLocksForMigration(lu, level):
 class LUInstanceFailover(LogicalUnit):
   """Failover an instance.
 
+  This is migration by shutting the instance down, but with the disks
+  of the instance already available on the new node.
+
+  See also:
+  L{LUInstanceMove} for moving an instance by copying the data.
+
+  L{LUInstanceMigrate} for the live migration of an instance (no shutdown
+  required).
   """
   HPATH = "instance-failover"
   HTYPE = constants.HTYPE_INSTANCE
@@ -123,14 +125,14 @@ class LUInstanceFailover(LogicalUnit):
     self.target_node = getattr(self.op, "target_node", None)
 
   def ExpandNames(self):
-    self._ExpandAndLockInstance()
+    self._ExpandAndLockInstance(allow_forthcoming=True)
     _ExpandNamesForMigration(self)
 
     self._migrater = \
       TLMigrateInstance(self, self.op.instance_uuid, self.op.instance_name,
                         self.op.cleanup, True, False,
                         self.op.ignore_consistency, True,
-                        self.op.shutdown_timeout, self.op.ignore_ipolicy)
+                        self.op.shutdown_timeout, self.op.ignore_ipolicy, True)
 
     self.tasklets = [self._migrater]
 
@@ -154,7 +156,8 @@ class LUInstanceFailover(LogicalUnit):
       "FAILOVER_CLEANUP": self.op.cleanup,
       }
 
-    if instance.disk_template in constants.DTS_INT_MIRROR:
+    disks = self.cfg.GetInstanceDisks(instance.uuid)
+    if utils.AnyDiskOfType(disks, constants.DTS_INT_MIRROR):
       secondary_nodes = self.cfg.GetInstanceSecondaryNodes(instance.uuid)
       env["OLD_SECONDARY"] = self.cfg.GetNodeName(secondary_nodes[0])
       env["NEW_SECONDARY"] = self.cfg.GetNodeName(source_node_uuid)
@@ -179,9 +182,14 @@ class LUInstanceFailover(LogicalUnit):
 class LUInstanceMigrate(LogicalUnit):
   """Migrate an instance.
 
-  This is migration without shutting down, compared to the failover,
-  which is done with shutdown.
+  This is migration without shutting down (live migration) and the disks are
+  already available on the new node.
 
+  See also:
+  L{LUInstanceMove} for moving an instance by copying the data.
+
+  L{LUInstanceFailover} for the migration of an instance where a shutdown is
+  required.
   """
   HPATH = "instance-migrate"
   HTYPE = constants.HTYPE_INSTANCE
@@ -196,7 +204,7 @@ class LUInstanceMigrate(LogicalUnit):
                         self.op.cleanup, False, self.op.allow_failover, False,
                         self.op.allow_runtime_changes,
                         constants.DEFAULT_SHUTDOWN_TIMEOUT,
-                        self.op.ignore_ipolicy)
+                        self.op.ignore_ipolicy, self.op.ignore_hvversions)
 
     self.tasklets = [self._migrater]
 
@@ -221,7 +229,8 @@ class LUInstanceMigrate(LogicalUnit):
       "ALLOW_RUNTIME_CHANGES": self.op.allow_runtime_changes,
       })
 
-    if instance.disk_template in constants.DTS_INT_MIRROR:
+    disks = self.cfg.GetInstanceDisks(instance.uuid)
+    if utils.AnyDiskOfType(disks, constants.DTS_INT_MIRROR):
       secondary_nodes = self.cfg.GetInstanceSecondaryNodes(instance.uuid)
       env["OLD_SECONDARY"] = self.cfg.GetNodeName(secondary_nodes[0])
       env["NEW_SECONDARY"] = self.cfg.GetNodeName(source_node_uuid)
@@ -267,6 +276,8 @@ class TLMigrateInstance(Tasklet):
   @ivar shutdown_timeout: In case of failover timeout of the shutdown
   @type ignore_ipolicy: bool
   @ivar ignore_ipolicy: If true, we can ignore instance policy when migrating
+  @type ignore_hvversions: bool
+  @ivar ignore_hvversions: If true, accept incompatible hypervisor versions
 
   """
 
@@ -276,7 +287,7 @@ class TLMigrateInstance(Tasklet):
 
   def __init__(self, lu, instance_uuid, instance_name, cleanup, failover,
                fallback, ignore_consistency, allow_runtime_changes,
-               shutdown_timeout, ignore_ipolicy):
+               shutdown_timeout, ignore_ipolicy, ignore_hvversions):
     """Initializes this class.
 
     """
@@ -293,6 +304,7 @@ class TLMigrateInstance(Tasklet):
     self.shutdown_timeout = shutdown_timeout
     self.ignore_ipolicy = ignore_ipolicy
     self.allow_runtime_changes = allow_runtime_changes
+    self.ignore_hvversions = ignore_hvversions
 
   def CheckPrereq(self):
     """Check prerequisites.
@@ -314,20 +326,24 @@ class TLMigrateInstance(Tasklet):
                       " switching to failover")
       self.failover = True
 
-    if self.instance.disk_template not in constants.DTS_MIRRORED:
+    disks = self.cfg.GetInstanceDisks(self.instance.uuid)
+
+    if not utils.AllDiskOfType(disks, constants.DTS_MIRRORED):
       if self.failover:
         text = "failovers"
       else:
         text = "migrations"
+      invalid_disks = set(d.dev_type for d in disks
+                             if d.dev_type not in constants.DTS_MIRRORED)
       raise errors.OpPrereqError("Instance's disk layout '%s' does not allow"
-                                 " %s" % (self.instance.disk_template, text),
+                                 " %s" % (utils.CommaJoin(invalid_disks), text),
                                  errors.ECODE_STATE)
 
-    if self.instance.disk_template in constants.DTS_EXT_MIRROR:
+    # TODO allow heterogeneous disk types if all are mirrored in some way.
+    if utils.AllDiskOfType(disks, constants.DTS_EXT_MIRROR):
       CheckIAllocatorOrNode(self.lu, "iallocator", "target_node")
 
       if self.lu.op.iallocator:
-        assert locking.NAL in self.lu.owned_locks(locking.LEVEL_NODE_ALLOC)
         self._RunAllocator()
       else:
         # We set set self.target_node_uuid as it is required by
@@ -357,15 +373,15 @@ class TLMigrateInstance(Tasklet):
         # in the LU
         ReleaseLocks(self.lu, locking.LEVEL_NODE,
                      keep=[self.instance.primary_node, self.target_node_uuid])
-        ReleaseLocks(self.lu, locking.LEVEL_NODE_ALLOC)
 
-    else:
+    elif utils.AllDiskOfType(disks, constants.DTS_INT_MIRROR):
+      templates = [d.dev_type for d in disks]
       secondary_node_uuids = \
         self.cfg.GetInstanceSecondaryNodes(self.instance.uuid)
       if not secondary_node_uuids:
         raise errors.ConfigurationError("No secondary node but using"
-                                        " %s disk template" %
-                                        self.instance.disk_template)
+                                        " %s disk types" %
+                                        utils.CommaJoin(set(templates)))
       self.target_node_uuid = target_node_uuid = secondary_node_uuids[0]
       if self.lu.op.iallocator or \
         (self.lu.op.target_node_uuid and
@@ -374,11 +390,11 @@ class TLMigrateInstance(Tasklet):
           text = "failed over"
         else:
           text = "migrated"
-        raise errors.OpPrereqError("Instances with disk template %s cannot"
+        raise errors.OpPrereqError("Instances with disk types %s cannot"
                                    " be %s to arbitrary nodes"
                                    " (neither an iallocator nor a target"
                                    " node can be passed)" %
-                                   (self.instance.disk_template, text),
+                                   (utils.CommaJoin(set(templates)), text),
                                    errors.ECODE_INVAL)
       nodeinfo = self.cfg.GetNodeInfo(target_node_uuid)
       group_info = self.cfg.GetNodeGroup(nodeinfo.group)
@@ -387,6 +403,10 @@ class TLMigrateInstance(Tasklet):
       CheckTargetNodeIPolicy(self.lu, ipolicy, self.instance, nodeinfo,
                              self.cfg, ignore=self.ignore_ipolicy)
 
+    else:
+      raise errors.OpPrereqError("Instance mixes internal and external "
+                                 "mirroring. This is not currently supported.")
+
     i_be = cluster.FillBE(self.instance)
 
     # check memory requirements on the secondary node
@@ -466,8 +486,6 @@ class TLMigrateInstance(Tasklet):
     """Run the allocator based on input opcode.
 
     """
-    assert locking.NAL in self.lu.owned_locks(locking.LEVEL_NODE_ALLOC)
-
     # FIXME: add a self.ignore_ipolicy option
     req = iallocator.IAReqRelocate(
           inst_uuid=self.instance_uuid,
@@ -512,17 +530,35 @@ class TLMigrateInstance(Tasklet):
           self.feedback_fn("   - progress: %.1f%%" % min_percent)
         time.sleep(2)
 
-  def _EnsureSecondary(self, node_uuid):
-    """Demote a node to secondary.
+  def _OpenInstanceDisks(self, node_uuid, exclusive):
+    """Open instance disks.
 
     """
-    self.feedback_fn("* switching node %s to secondary mode" %
+    if exclusive:
+      mode = "in exclusive mode"
+    else:
+      mode = "in shared mode"
+
+    self.feedback_fn("* opening instance disks on node %s %s" %
+                     (self.cfg.GetNodeName(node_uuid), mode))
+
+    disks = self.cfg.GetInstanceDisks(self.instance.uuid)
+    result = self.rpc.call_blockdev_open(node_uuid, self.instance.name,
+                                         (disks, self.instance), exclusive)
+    result.Raise("Cannot open disks on node %s" %
+                 self.cfg.GetNodeName(node_uuid))
+
+  def _CloseInstanceDisks(self, node_uuid):
+    """Close instance disks.
+
+    """
+    self.feedback_fn("* closing instance disks on node %s" %
                      self.cfg.GetNodeName(node_uuid))
 
     disks = self.cfg.GetInstanceDisks(self.instance.uuid)
     result = self.rpc.call_blockdev_close(node_uuid, self.instance.name,
                                           (disks, self.instance))
-    result.Raise("Cannot change disk to secondary on node %s" %
+    result.Raise("Cannot close instance disks on node %s" %
                  self.cfg.GetNodeName(node_uuid))
 
   def _GoStandalone(self):
@@ -549,7 +585,7 @@ class TLMigrateInstance(Tasklet):
     disks = self.cfg.GetInstanceDisks(self.instance.uuid)
     result = self.rpc.call_drbd_attach_net(self.all_node_uuids,
                                            (disks, self.instance),
-                                           self.instance.name, multimaster)
+                                           multimaster)
     for node_uuid, nres in result.items():
       nres.Raise("Cannot change disks config on node %s" %
                  self.cfg.GetNodeName(node_uuid))
@@ -609,8 +645,11 @@ class TLMigrateInstance(Tasklet):
                        self.cfg.GetNodeName(self.source_node_uuid))
       demoted_node_uuid = self.target_node_uuid
 
-    if self.instance.disk_template in constants.DTS_INT_MIRROR:
-      self._EnsureSecondary(demoted_node_uuid)
+    disks = self.cfg.GetInstanceDisks(self.instance.uuid)
+
+    self._CloseInstanceDisks(demoted_node_uuid)
+
+    if utils.AnyDiskOfType(disks, constants.DTS_INT_MIRROR):
       try:
         self._WaitUntilSync()
       except errors.OpExecError:
@@ -620,6 +659,8 @@ class TLMigrateInstance(Tasklet):
       self._GoStandalone()
       self._GoReconnect(False)
       self._WaitUntilSync()
+    elif utils.AnyDiskOfType(disks, constants.DTS_EXT_MIRROR):
+      self._OpenInstanceDisks(self.instance.primary_node, True)
 
     self.feedback_fn("* done")
 
@@ -627,11 +668,16 @@ class TLMigrateInstance(Tasklet):
     """Try to revert the disk status after a failed migration.
 
     """
-    if self.instance.disk_template in constants.DTS_EXT_MIRROR:
+
+    disks = self.cfg.GetInstanceDisks(self.instance.uuid)
+
+    self._CloseInstanceDisks(self.target_node_uuid)
+
+    if utils.AllDiskOfType(disks, constants.DTS_EXT_MIRROR):
+      self._OpenInstanceDisks(self.source_node_uuid, True)
       return
 
     try:
-      self._EnsureSecondary(self.target_node_uuid)
       self._GoStandalone()
       self._GoReconnect(False)
       self._WaitUntilSync()
@@ -692,6 +738,20 @@ class TLMigrateInstance(Tasklet):
         self.feedback_fn("* warning: hypervisor version mismatch between"
                          " source (%s) and target (%s) node" %
                          (src_version, dst_version))
+        hv = hypervisor.GetHypervisor(self.instance.hypervisor)
+        if hv.VersionsSafeForMigration(src_version, dst_version):
+          self.feedback_fn("  migrating from hypervisor version %s to %s should"
+                           " be safe" % (src_version, dst_version))
+        else:
+          self.feedback_fn("  migrating from hypervisor version %s to %s is"
+                           " likely unsupported" % (src_version, dst_version))
+          if self.ignore_hvversions:
+            self.feedback_fn("  continuing anyway (told to ignore version"
+                             " mismatch)")
+          else:
+            raise errors.OpExecError("Unsupported migration between hypervisor"
+                                     " versions (%s to %s)" %
+                                     (src_version, dst_version))
 
     self.feedback_fn("* checking disk consistency between source and target")
     for (idx, dev) in enumerate(self.cfg.GetInstanceDisks(self.instance.uuid)):
@@ -727,13 +787,19 @@ class TLMigrateInstance(Tasklet):
 
     self.migration_info = migration_info = result.payload
 
-    if self.instance.disk_template not in constants.DTS_EXT_MIRROR:
+    disks = self.cfg.GetInstanceDisks(self.instance.uuid)
+
+    self._CloseInstanceDisks(self.target_node_uuid)
+
+    if utils.AnyDiskOfType(disks, constants.DTS_INT_MIRROR):
       # Then switch the disks to master/master mode
-      self._EnsureSecondary(self.target_node_uuid)
       self._GoStandalone()
       self._GoReconnect(True)
       self._WaitUntilSync()
 
+    self._OpenInstanceDisks(self.source_node_uuid, False)
+    self._OpenInstanceDisks(self.target_node_uuid, False)
+
     self.feedback_fn("* preparing %s to accept the instance" %
                      self.cfg.GetNodeName(self.target_node_uuid))
     result = self.rpc.call_accept_instance(self.target_node_uuid,
@@ -809,6 +875,7 @@ class TLMigrateInstance(Tasklet):
 
     self.cfg.SetInstancePrimaryNode(self.instance.uuid, self.target_node_uuid)
     self.instance = self.cfg.GetInstanceInfo(self.instance_uuid)
+    disks = self.cfg.GetInstanceDisks(self.instance_uuid)
 
     result = self.rpc.call_instance_finalize_migration_dst(
                self.target_node_uuid, self.instance, migration_info, True)
@@ -819,20 +886,26 @@ class TLMigrateInstance(Tasklet):
       raise errors.OpExecError("Could not finalize instance migration: %s" %
                                msg)
 
-    if self.instance.disk_template not in constants.DTS_EXT_MIRROR:
-      self._EnsureSecondary(self.source_node_uuid)
+    self._CloseInstanceDisks(self.source_node_uuid)
+
+    if utils.AnyDiskOfType(disks, constants.DTS_INT_MIRROR):
       self._WaitUntilSync()
       self._GoStandalone()
       self._GoReconnect(False)
       self._WaitUntilSync()
+    elif utils.AnyDiskOfType(disks, constants.DTS_EXT_MIRROR):
+      self._OpenInstanceDisks(self.target_node_uuid, True)
 
     # If the instance's disk template is `rbd' or `ext' and there was a
     # successful migration, unmap the device from the source node.
-    if self.instance.disk_template in (constants.DT_RBD, constants.DT_EXT):
-      inst_disks = self.cfg.GetInstanceDisks(self.instance.uuid)
-      disks = ExpandCheckDisks(inst_disks, inst_disks)
-      self.feedback_fn("* unmapping instance's disks from %s" %
-                       self.cfg.GetNodeName(self.source_node_uuid))
+    unmap_types = (constants.DT_RBD, constants.DT_EXT)
+
+    if utils.AnyDiskOfType(disks, unmap_types):
+      unmap_disks = [d for d in disks if d.dev_type in unmap_types]
+      disks = ExpandCheckDisks(unmap_disks, unmap_disks)
+      self.feedback_fn("* unmapping instance's disks %s from %s" %
+                       (utils.CommaJoin(d.name for d in unmap_disks),
+                        self.cfg.GetNodeName(self.source_node_uuid)))
       for disk in disks:
         result = self.rpc.call_blockdev_shutdown(self.source_node_uuid,
                                                  (disk, self.instance))
@@ -855,6 +928,13 @@ class TLMigrateInstance(Tasklet):
     starting it on the secondary.
 
     """
+    if self.instance.forthcoming:
+      self.feedback_fn("Instance is forthcoming, just updating the"
+                       "  configuration")
+      self.cfg.SetInstancePrimaryNode(self.instance.uuid,
+                                      self.target_node_uuid)
+      return
+
     primary_node = self.cfg.GetNodeInfo(self.instance.primary_node)
 
     source_node_uuid = self.instance.primary_node
@@ -900,6 +980,10 @@ class TLMigrateInstance(Tasklet):
                                  (self.instance.name,
                                   self.cfg.GetNodeName(source_node_uuid), msg))
 
+    disk_template = self.cfg.GetInstanceDiskTemplate(self.instance.uuid)
+    if disk_template in constants.DTS_EXT_MIRROR:
+      self._CloseInstanceDisks(source_node_uuid)
+
     self.feedback_fn("* deactivating the instance's disks on source node")
     if not ShutdownInstanceDisks(self.lu, self.instance, ignore_primary=True):
       raise errors.OpExecError("Can't shut down the instance's disks")
@@ -914,8 +998,8 @@ class TLMigrateInstance(Tasklet):
       logging.info("Starting instance %s on node %s", self.instance.name,
                    self.cfg.GetNodeName(self.target_node_uuid))
 
-      disks_ok, _ = AssembleInstanceDisks(self.lu, self.instance,
-                                          ignore_secondaries=True)
+      disks_ok, _, _ = AssembleInstanceDisks(self.lu, self.instance,
+                                             ignore_secondaries=True)
       if not disks_ok:
         ShutdownInstanceDisks(self.lu, self.instance)
         raise errors.OpExecError("Can't activate the instance's disks")
@@ -941,7 +1025,10 @@ class TLMigrateInstance(Tasklet):
     self.source_node_uuid = self.instance.primary_node
 
     # FIXME: if we implement migrate-to-any in DRBD, this needs fixing
-    if self.instance.disk_template in constants.DTS_INT_MIRROR:
+    disks = self.cfg.GetInstanceDisks(self.instance.uuid)
+
+    # TODO allow mixed disks
+    if utils.AllDiskOfType(disks, constants.DTS_INT_MIRROR):
       secondary_nodes = self.cfg.GetInstanceSecondaryNodes(self.instance.uuid)
       self.target_node_uuid = secondary_nodes[0]
       # Otherwise self.target_node has been populated either
