@@ -65,12 +65,6 @@ class LUOobCommand(NoHooksLU):
       locking.LEVEL_NODE: lock_node_uuids,
       }
 
-    self.share_locks[locking.LEVEL_NODE_ALLOC] = 1
-
-    if not self.op.node_names:
-      # Acquire node allocation lock only if all nodes are affected
-      self.needed_locks[locking.LEVEL_NODE_ALLOC] = locking.ALL_SET
-
   def CheckPrereq(self):
     """Check prerequisites.
 
