@@ -53,7 +53,7 @@ from ganeti.cmdlib.common import CheckParamsNotGlobal, \
   GetWantedNodes, MapInstanceLvsToNodes, RunPostHook, \
   FindFaultyInstanceDisks, CheckStorageTypeEnabled, GetClientCertDigest, \
   AddNodeCertToCandidateCerts, RemoveNodeCertFromCandidateCerts, \
-  EnsureKvmdOnNodes, WarnAboutFailedSshUpdates
+  EnsureKvmdOnNodes, WarnAboutFailedSshUpdates, AddMasterCandidateSshKey
 
 
 def _DecideSelfPromotion(lu, exceptions=None):
@@ -359,7 +359,8 @@ class LUNodeAdd(LogicalUnit):
         True, # from authorized keys
         True, # from public keys
         False, # clear authorized keys
-        True) # clear public keys
+        True, # clear public keys
+        True) # it's a readd
       remove_result[master_node].Raise(
         "Could not remove SSH keys of node %s before readding,"
         " (UUID: %s)." % (new_node_name, new_node_uuid))
@@ -828,6 +829,9 @@ class LUNodeSetParams(LogicalUnit):
 
     # this will trigger configuration file update, if needed
     self.cfg.Update(node, feedback_fn)
+    master_node = self.cfg.GetMasterNode()
+    potential_master_candidates = self.cfg.GetPotentialMasterCandidates()
+    modify_ssh_setup = self.cfg.GetClusterInfo().modify_ssh_setup
 
     if self.new_role != self.old_role:
       new_flags = self._R2F[self.new_role]
@@ -848,7 +852,9 @@ class LUNodeSetParams(LogicalUnit):
 
       # we locked all nodes, we adjust the CP before updating this node
       if self.lock_all:
-        AdjustCandidatePool(self, [node.uuid])
+        AdjustCandidatePool(
+            self, [node.uuid], master_node, potential_master_candidates,
+            feedback_fn, modify_ssh_setup)
 
       # if node gets promoted, grant RPC priviledges
       if self.new_role == self._ROLE_CANDIDATE:
@@ -864,9 +870,7 @@ class LUNodeSetParams(LogicalUnit):
     if [self.old_role, self.new_role].count(self._ROLE_CANDIDATE) == 1:
       self.context.ReaddNode(node)
 
-      if self.cfg.GetClusterInfo().modify_ssh_setup:
-        potential_master_candidates = self.cfg.GetPotentialMasterCandidates()
-        master_node = self.cfg.GetMasterNode()
+      if modify_ssh_setup:
         if self.old_role == self._ROLE_CANDIDATE:
           master_candidate_uuids = self.cfg.GetMasterCandidateUuids()
           ssh_result = self.rpc.call_node_ssh_key_remove(
@@ -876,23 +880,16 @@ class LUNodeSetParams(LogicalUnit):
             True, # remove node's key from all nodes' authorized_keys file
             False, # currently, all nodes are potential master candidates
             False, # do not clear node's 'authorized_keys'
-            False) # do not clear node's 'ganeti_pub_keys'
+            False, # do not clear node's 'ganeti_pub_keys'
+            False) # no readd
           ssh_result[master_node].Raise(
             "Could not adjust the SSH setup after demoting node '%s'"
             " (UUID: %s)." % (node.name, node.uuid))
           WarnAboutFailedSshUpdates(ssh_result, master_node, feedback_fn)
 
         if self.new_role == self._ROLE_CANDIDATE:
-          ssh_result = self.rpc.call_node_ssh_key_add(
-            [master_node], node.uuid, node.name,
-            potential_master_candidates,
-            True, # add node's key to all node's 'authorized_keys'
-            True, # all nodes are potential master candidates
-            False) # do not update the node's public keys
-          ssh_result[master_node].Raise(
-            "Could not update the SSH setup of node '%s' after promotion"
-            " (UUID: %s)." % (node.name, node.uuid))
-          WarnAboutFailedSshUpdates(ssh_result, master_node, feedback_fn)
+          AddMasterCandidateSshKey(
+              self, master_node, node, potential_master_candidates, feedback_fn)
 
     return result
 
@@ -1568,14 +1565,14 @@ class LUNodeRemove(LogicalUnit):
     assert locking.BGL in self.owned_locks(locking.LEVEL_CLUSTER), \
       "Not owning BGL"
 
+    master_node = self.cfg.GetMasterNode()
+    potential_master_candidates = self.cfg.GetPotentialMasterCandidates()
     if modify_ssh_setup:
       # retrieve the list of potential master candidates before the node is
       # removed
-      potential_master_candidates = self.cfg.GetPotentialMasterCandidates()
       potential_master_candidate = \
         self.op.node_name in potential_master_candidates
       master_candidate_uuids = self.cfg.GetMasterCandidateUuids()
-      master_node = self.cfg.GetMasterNode()
       result = self.rpc.call_node_ssh_key_remove(
         [master_node],
         self.node.uuid, self.op.node_name,
@@ -1583,14 +1580,17 @@ class LUNodeRemove(LogicalUnit):
         self.node.master_candidate, # from_authorized_keys
         potential_master_candidate, # from_public_keys
         True, # clear node's 'authorized_keys'
-        True) # clear node's 'ganeti_public_keys'
+        True, # clear node's 'ganeti_public_keys'
+        False) # no readd
       result[master_node].Raise(
         "Could not remove the SSH key of node '%s' (UUID: %s)." %
         (self.op.node_name, self.node.uuid))
       WarnAboutFailedSshUpdates(result, master_node, feedback_fn)
 
     # Promote nodes to master candidate as needed
-    AdjustCandidatePool(self, [self.node.uuid])
+    AdjustCandidatePool(
+        self, [self.node.uuid], master_node, potential_master_candidates,
+        feedback_fn, modify_ssh_setup)
     self.context.RemoveNode(self.cfg, self.node)
 
     # Run post hooks on the node before it's removed
