@@ -409,6 +409,7 @@ class LUTestAllocator(NoHooksLU):
                                           memory=self.op.memory,
                                           disks=self.op.disks,
                                           disk_template=self.op.disk_template,
+                                          group_name=self.op.group_name,
                                           os=self.op.os,
                                           tags=self.op.tags,
                                           nics=self.op.nics,
@@ -425,13 +426,15 @@ class LUTestAllocator(NoHooksLU):
                                         target_groups=self.op.target_groups)
     elif self.op.mode == constants.IALLOCATOR_MODE_NODE_EVAC:
       req = iallocator.IAReqNodeEvac(instances=self.op.instances,
-                                     evac_mode=self.op.evac_mode)
+                                     evac_mode=self.op.evac_mode,
+                                     ignore_soft_errors=False)
     elif self.op.mode == constants.IALLOCATOR_MODE_MULTI_ALLOC:
       disk_template = self.op.disk_template
       insts = [iallocator.IAReqInstanceAlloc(name="%s%s" % (self.op.name, idx),
                                              memory=self.op.memory,
                                              disks=self.op.disks,
                                              disk_template=disk_template,
+                                             group_name=self.op.group_name,
                                              os=self.op.os,
                                              tags=self.op.tags,
                                              nics=self.op.nics,
