@@ -202,6 +202,12 @@ class OpResultError(GenericError):
   """
 
 
+class OpRetryNotSupportedError(GenericError):
+  """This opcode does not support retries
+
+  """
+
+
 class DeviceCreationError(GenericError):
   """Error during the creation of a device.
 
@@ -448,6 +454,12 @@ class FileStoragePathError(GenericError):
   """
 
 
+class SshUpdateError(GenericError):
+  """Error from updating the SSH setup.
+
+  """
+
+
 # errors should be added above
 
 
