@@ -216,11 +216,25 @@ def _IsInstanceRunning(instance_info):
   @return: Whether an instance is running.
 
   """
-  return instance_info == "r-----" \
-      or instance_info == "rb----" \
-      or instance_info == "-b----" \
-      or instance_info == "-----d" \
-      or instance_info == "------"
+  allowable_running_prefixes = [
+    "r--",
+    "rb-",
+    "-b-",
+    "---",
+  ]
+
+  def _RunningWithSuffix(suffix):
+    return map(lambda x: x + suffix, allowable_running_prefixes)
+
+  # The shutdown suspend ("ss") state is encountered during migration, where
+  # the instance is still considered to be running.
+  # The shutdown restart ("sr") is probably encountered during restarts - still
+  # running.
+  # See Xen commit e1475a6693aac8cddc4bdd456548aa05a625556b
+  return instance_info in _RunningWithSuffix("---") \
+      or instance_info in _RunningWithSuffix("ss-") \
+      or instance_info in _RunningWithSuffix("sr-") \
+      or instance_info == "-----d"
 
 
 def _IsInstanceShutdown(instance_info):
@@ -1179,8 +1193,8 @@ class XenHypervisor(hv_base.BaseHypervisor):
       # And try and kill a previous daemon
       XenHypervisor._KillMigrationDaemon(instance)
 
-      listening_arg = "TCP-LISTEN:%d,bind=%s" % (port, target)
-      socat_pid = utils.StartDaemon(["socat", listening_arg,
+      listening_arg = "TCP-LISTEN:%d,bind=%s,reuseaddr" % (port, target)
+      socat_pid = utils.StartDaemon(["socat", "-b524288", listening_arg,
                                      "SYSTEM:'xl migrate-receive'"],
                                      pidfile=pidfile)
 
