@@ -300,23 +300,36 @@ class CommandBuilder(object):
 
     compr = self._opts.compress
 
-    assert compr in constants.IEC_ALL
-
     parts = []
 
     if self._mode == constants.IEM_IMPORT:
       parts.append(socat_cmd)
 
-      if compr == constants.IEC_GZIP:
-        parts.append("gunzip -c")
+      if compr in [constants.IEC_GZIP, constants.IEC_GZIP_FAST,
+                   constants.IEC_GZIP_SLOW, constants.IEC_LZOP]:
+        utility_name = constants.IEC_COMPRESSION_UTILITIES.get(compr, compr)
+        parts.append("%s -d -c" % utility_name)
+      elif compr != constants.IEC_NONE:
+        parts.append("%s -d" % compr)
+      else:
+        # No compression
+        pass
 
       parts.append(dd_cmd)
 
     elif self._mode == constants.IEM_EXPORT:
       parts.append(dd_cmd)
 
-      if compr == constants.IEC_GZIP:
-        parts.append("gzip -c")
+      if compr in [constants.IEC_GZIP_SLOW, constants.IEC_LZOP]:
+        utility_name = constants.IEC_COMPRESSION_UTILITIES.get(compr, compr)
+        parts.append("%s -c" % utility_name)
+      elif compr in [constants.IEC_GZIP_FAST, constants.IEC_GZIP]:
+        parts.append("gzip -1 -c")
+      elif compr != constants.IEC_NONE:
+        parts.append(compr)
+      else:
+        # No compression
+        pass
 
       parts.append(socat_cmd)
 
