@@ -1162,28 +1162,47 @@ class ExportInstanceHelper(object):
     self._feedback_fn = feedback_fn
     self._instance = instance
 
-    self._snap_disks = []
-    self._removed_snaps = [False] * len(instance.disks)
+    self._snapshots = [None] * len(instance.disks)
+    self._snapshots_removed = [False] * len(instance.disks)
 
-  def CreateSnapshots(self):
-    """Creates an LVM snapshot for every disk of the instance.
+  def _SnapshotsReady(self):
+    """Returns true if snapshots are ready to be used in exports.
 
     """
-    assert not self._snap_disks
+    return all(self._snapshots)
+
+  def CreateSnapshots(self):
+    """Attempts to create a snapshot for every disk of the instance.
+
+    Currently support drbd, plain and ext disk templates.
+
+    @rtype: bool
+    @return: Whether following transfers can use snapshots
+
+    """
+    if any(self._snapshots):
+      raise errors.ProgrammerError("Snapshot creation was invoked more than "
+                                   "once")
 
     instance = self._instance
+    inst_disks = self._lu.cfg.GetInstanceDisks(instance.uuid)
+
+    # A quick check whether we can support snapshots at all
+    if not all([d.SupportsSnapshots() for d in inst_disks]):
+      return False
+
     src_node = instance.primary_node
     src_node_name = self._lu.cfg.GetNodeName(src_node)
 
-    inst_disks = self._lu.cfg.GetInstanceDisks(instance.uuid)
     for idx, disk in enumerate(inst_disks):
       self._feedback_fn("Creating a snapshot of disk/%s on node %s" %
                         (idx, src_node_name))
 
       # result.payload will be a snapshot of an lvm leaf of the one we
       # passed
-      result = self._lu.rpc.call_blockdev_snapshot(src_node, (disk, instance))
-      new_dev = False
+      result = self._lu.rpc.call_blockdev_snapshot(src_node,
+                                                   (disk, instance),
+                                                   None, None)
       msg = result.fail_msg
       if msg:
         self._lu.LogWarning("Could not snapshot disk/%s on node %s: %s",
@@ -1194,15 +1213,29 @@ class ExportInstanceHelper(object):
                             " result '%s'", idx, src_node_name, result.payload)
       else:
         disk_id = tuple(result.payload)
-        disk_params = constants.DISK_LD_DEFAULTS[constants.DT_PLAIN].copy()
-        new_dev = objects.Disk(dev_type=constants.DT_PLAIN, size=disk.size,
+        # Snapshot is currently supported for ExtStorage and LogicalVolume.
+        # In case disk is of type drbd the snapshot will be of type plain.
+        if disk.dev_type == constants.DT_EXT:
+          dev_type = constants.DT_EXT
+        else:
+          dev_type = constants.DT_PLAIN
+        disk_params = constants.DISK_LD_DEFAULTS[dev_type].copy()
+        new_dev = objects.Disk(dev_type=dev_type, size=disk.size,
                                logical_id=disk_id, iv_name=disk.iv_name,
                                params=disk_params)
+        new_dev.uuid = self._lu.cfg.GenerateUniqueID(self._lu.proc.GetECId())
 
-      self._snap_disks.append(new_dev)
+        self._snapshots[idx] = new_dev
+        self._snapshots_removed[idx] = False
 
-    assert len(self._snap_disks) == len(instance.disks)
-    assert len(self._removed_snaps) == len(instance.disks)
+    # One final check to see if we have managed to snapshot everything
+    if self._SnapshotsReady():
+      return True
+    else:
+      # If we failed to do so, the existing snapshots are of little value to us
+      # so we can remove them straight away.
+      self.Cleanup()
+      return False
 
   def _RemoveSnapshot(self, disk_index):
     """Removes an LVM snapshot.
@@ -1211,22 +1244,47 @@ class ExportInstanceHelper(object):
     @param disk_index: Index of the snapshot to be removed
 
     """
-    disk = self._snap_disks[disk_index]
-    if disk and not self._removed_snaps[disk_index]:
-      src_node = self._instance.primary_node
-      src_node_name = self._lu.cfg.GetNodeName(src_node)
+    snapshot = self._snapshots[disk_index]
+    if snapshot is not None and not self._snapshots_removed[disk_index]:
+      src_node_uuid = self._instance.primary_node
+      src_node_name = self._lu.cfg.GetNodeName(src_node_uuid)
 
       self._feedback_fn("Removing snapshot of disk/%s on node %s" %
                         (disk_index, src_node_name))
 
-      result = self._lu.rpc.call_blockdev_remove(src_node,
-                                                 (disk, self._instance))
+      result = self._lu.rpc.call_blockdev_remove(src_node_uuid,
+                                                 (snapshot, self._instance))
       if result.fail_msg:
         self._lu.LogWarning("Could not remove snapshot for disk/%d from node"
                             " %s: %s", disk_index, src_node_name,
                             result.fail_msg)
       else:
-        self._removed_snaps[disk_index] = True
+        self._snapshots_removed[disk_index] = True
+
+  def _GetDisksToTransfer(self):
+    """Returns disks to be transferred, whether snapshots or instance disks.
+
+    @rtype: list of L{objects.Disk}
+    @return: The disks to transfer
+
+    """
+    if self._SnapshotsReady():
+      return self._snapshots
+    else:
+      return self._lu.cfg.GetInstanceDisks(self._instance.uuid)
+
+  def _GetDiskLabel(self, idx):
+    """Returns a label which should be used to represent a disk to transfer.
+
+    @type idx: int
+    @param idx: The disk index
+    @rtype: string
+
+    """
+    if self._SnapshotsReady():
+      return "snapshot/%d" % idx
+    else:
+      return "disk/%d" % idx
 
   def LocalExport(self, dest_node, compress):
     """Intra-cluster instance export.
@@ -1237,20 +1295,16 @@ class ExportInstanceHelper(object):
     @param compress: Compression tool to use
 
     """
+    disks_to_transfer = self._GetDisksToTransfer()
+
     instance = self._instance
     src_node_uuid = instance.primary_node
 
-    assert len(self._snap_disks) == len(instance.disks)
-
     transfers = []
 
-    for idx, dev in enumerate(self._snap_disks):
-      if not dev:
-        transfers.append(None)
-        continue
-
+    for idx, dev in enumerate(disks_to_transfer):
       path = utils.PathJoin(pathutils.EXPORT_DIR, "%s.new" % instance.name,
-                            dev.logical_id[1])
+                            dev.uuid)
 
       finished_fn = compat.partial(self._TransferFinished, idx)
 
@@ -1262,10 +1316,8 @@ class ExportInstanceHelper(object):
         src_ioargs = (dev, instance)
 
       # FIXME: pass debug option from opcode to backend
-      dt = DiskTransfer("snapshot/%s" % idx,
-                        src_io, src_ioargs,
-                        constants.IEIO_FILE, (path, ),
-                        finished_fn)
+      dt = DiskTransfer(self._GetDiskLabel(idx), src_io, src_ioargs,
+                        constants.IEIO_FILE, (path, ), finished_fn)
       transfers.append(dt)
 
     # Actually export data
@@ -1281,7 +1333,7 @@ class ExportInstanceHelper(object):
     if all(dresults):
       self._feedback_fn("Finalizing export on %s" % dest_node.name)
       result = self._lu.rpc.call_finalize_export(dest_node.uuid, instance,
-                                                 self._snap_disks)
+                                                 disks_to_transfer)
       msg = result.fail_msg
       fin_resu = not msg
       if msg:
@@ -1312,15 +1364,15 @@ class ExportInstanceHelper(object):
 
     """
     instance = self._instance
-    inst_disks = self._lu.cfg.GetInstanceDisks(instance.uuid)
+    disks_to_transfer = self._GetDisksToTransfer()
 
-    assert len(disk_info) == len(instance.disks)
+    assert len(disk_info) == len(disks_to_transfer)
 
-    cbs = _RemoteExportCb(self._feedback_fn, len(instance.disks))
+    cbs = _RemoteExportCb(self._feedback_fn, len(disks_to_transfer))
 
     ieloop = ImportExportLoop(self._lu)
     try:
-      for idx, (dev, (host, port, magic)) in enumerate(zip(inst_disks,
+      for idx, (dev, (host, port, magic)) in enumerate(zip(disks_to_transfer,
                                                            disk_info)):
         # Decide whether to use IPv6
         ipv6 = netutils.IP6Address.IsValid(host)
@@ -1365,8 +1417,7 @@ class ExportInstanceHelper(object):
     """Remove all snapshots.
 
     """
-    assert len(self._removed_snaps) == len(self._instance.disks)
-    for idx in range(len(self._instance.disks)):
+    for idx in range(len(self._snapshots)):
       self._RemoveSnapshot(idx)
 
 
@@ -1676,27 +1727,29 @@ def CalculateGroupIPolicy(cluster, group):
   return cluster.SimpleFillIPolicy(group.ipolicy)
 
 
-def ComputeDiskSize(disk_template, disks):
+def ComputeDiskSize(disks):
   """Compute disk size requirements according to disk template
 
   """
   # Required free disk space as a function of disk and swap space
-  req_size_dict = {
-    constants.DT_DISKLESS: 0,
-    constants.DT_PLAIN: sum(d[constants.IDISK_SIZE] for d in disks),
-    # 128 MB are added for drbd metadata for each disk
-    constants.DT_DRBD8:
-      sum(d[constants.IDISK_SIZE] + constants.DRBD_META_SIZE for d in disks),
-    constants.DT_FILE: sum(d[constants.IDISK_SIZE] for d in disks),
-    constants.DT_SHARED_FILE: sum(d[constants.IDISK_SIZE] for d in disks),
-    constants.DT_GLUSTER: sum(d[constants.IDISK_SIZE] for d in disks),
-    constants.DT_BLOCK: 0,
-    constants.DT_RBD: sum(d[constants.IDISK_SIZE] for d in disks),
-    constants.DT_EXT: sum(d[constants.IDISK_SIZE] for d in disks),
-  }
+  def size_f(d):
+    dev_type = d[constants.IDISK_TYPE]
+    req_size_dict = {
+      constants.DT_DISKLESS: 0,
+      constants.DT_PLAIN: d[constants.IDISK_SIZE],
+      # Extra space for drbd metadata is added to each disk
+      constants.DT_DRBD8:
+        d[constants.IDISK_SIZE] + constants.DRBD_META_SIZE,
+      constants.DT_FILE: d[constants.IDISK_SIZE],
+      constants.DT_SHARED_FILE: d[constants.IDISK_SIZE],
+      constants.DT_GLUSTER: d[constants.IDISK_SIZE],
+      constants.DT_BLOCK: 0,
+      constants.DT_RBD: d[constants.IDISK_SIZE],
+      constants.DT_EXT: d[constants.IDISK_SIZE],
+    }
+    if dev_type not in req_size_dict:
+      raise errors.ProgrammerError("Disk template '%s' size requirement"
+                                   " is unknown" % dev_type)
+    return req_size_dict[dev_type]
 
-  if disk_template not in req_size_dict:
-    raise errors.ProgrammerError("Disk template '%s' size requirement"
-                                 " is unknown" % disk_template)
-
-  return req_size_dict[disk_template]
+  return sum(map(size_f, disks))
