@@ -323,7 +323,8 @@ class _FilterHints(object):
 
     # If any operator other than equality was used, all names need to be
     # retrieved
-    if op == qlang.OP_EQUAL and name == self._namefield:
+    EQ_OPS = [qlang.OP_EQUAL, qlang.OP_EQUAL_LEGACY]
+    if op in EQ_OPS and name == self._namefield:
       if self._names is None:
         self._names = []
       self._names.append(value)
@@ -456,6 +457,7 @@ class _FilterCompilerHelper(object):
 
     # Binary operators
     qlang.OP_EQUAL: (_OPTYPE_BINARY, _EQUALITY_CHECKS),
+    qlang.OP_EQUAL_LEGACY: (_OPTYPE_BINARY, _EQUALITY_CHECKS),
     qlang.OP_NOT_EQUAL:
       (_OPTYPE_BINARY, [(flags, compat.partial(_WrapNot, fn), valprepfn)
                         for (flags, fn, valprepfn) in _EQUALITY_CHECKS]),
@@ -2226,6 +2228,9 @@ def _BuildInstanceFields():
     (_MakeField("console", "Console", QFT_OTHER,
                 "Instance console information"), IQ_CONSOLE, 0,
      _GetInstanceConsole),
+    (_MakeField("forthcoming", "Forthcoming", QFT_BOOL,
+                "Whether the Instance is forthcoming"), IQ_CONFIG, 0,
+     lambda _, inst: bool(inst.forthcoming)),
     ]
 
   # Add simple fields
@@ -2909,6 +2914,28 @@ def _BuildNetworkFields():
 
   return _PrepareFieldList(fields, [])
 
+
+_FILTER_SIMPLE_FIELDS = {
+  "watermark": ("Watermark", QFT_NUMBER, 0, "Watermark"),
+  "priority": ("Priority", QFT_NUMBER, 0, "Priority"),
+  "predicates": ("Predicates", QFT_OTHER, 0, "Predicates"),
+  "action": ("Action", QFT_OTHER, 0, "Action"),
+  "reason_trail": ("ReasonTrail", QFT_OTHER, 0, "Reason trail"),
+  "uuid": ("UUID", QFT_TEXT, 0, "Network UUID"),
+  }
+
+
+def _BuildFilterFields():
+  """Builds list of fields for job filter queries.
+
+  """
+  fields = [
+    (_MakeField(name, title, kind, doc), None, 0, _GetItemMaybeAttr(name))
+    for (name, (title, kind, _, doc)) in _FILTER_SIMPLE_FIELDS.items()
+    ]
+
+  return _PrepareFieldList(fields, [])
+
 #: Fields for cluster information
 CLUSTER_FIELDS = _BuildClusterFields()
 
@@ -2939,6 +2966,9 @@ EXPORT_FIELDS = _BuildExportFields()
 #: Fields available for network queries
 NETWORK_FIELDS = _BuildNetworkFields()
 
+#: Fields available for job filter queries
+FILTER_FIELDS = _BuildFilterFields()
+
 #: All available resources
 ALL_FIELDS = {
   constants.QR_CLUSTER: CLUSTER_FIELDS,
@@ -2951,6 +2981,7 @@ ALL_FIELDS = {
   constants.QR_JOB: JOB_FIELDS,
   constants.QR_EXPORT: EXPORT_FIELDS,
   constants.QR_NETWORK: NETWORK_FIELDS,
+  constants.QR_FILTER: FILTER_FIELDS,
   }
 
 #: All available field lists
