@@ -69,6 +69,7 @@ from ganeti.utils.text import *
 from ganeti.utils.wrapper import *
 from ganeti.utils.version import *
 from ganeti.utils.x509 import *
+from ganeti.utils.bitarrays import *
 
 
 _VALID_SERVICE_NAME_RE = re.compile("^[-_.a-zA-Z0-9]{1,128}$")
@@ -884,3 +885,82 @@ def ValidateDeviceNames(kind, container):
                                    errors.ECODE_NOTUNIQUE)
       else:
         valid.append(name)
+
+
+def AllDiskOfType(disks_info, dev_types):
+  """Checks if the instance has only disks of any of the dev_types.
+
+  @type disks_info: list of L{Disk}
+  @param disks_info: all the disks of the instance.
+  @type dev_types: list of disk templates
+  @param dev_types: the disk type required.
+
+  @rtype: bool
+  @return: True iff the instance only has disks of type dev_type.
+  """
+
+  assert not isinstance(dev_types, str)
+
+  if not disks_info and constants.DT_DISKLESS not in dev_types:
+    return False
+
+  for disk in disks_info:
+    if disk.dev_type not in dev_types:
+      return False
+
+  return True
+
+
+def AnyDiskOfType(disks_info, dev_types):
+  """Checks if the instance has some disks of any types in dev_types.
+
+  @type disks_info: list of L{Disk}
+  @param disks_info: all the disks of the instance.
+  @type dev_types: list of disk template
+  @param dev_types: the disk type required.
+
+  @rtype: bool
+  @return: True if the instance has disks of type dev_types or the instance has
+    no disks and the dev_types allow DT_DISKLESS.
+  """
+
+  assert not isinstance(dev_types, str)
+
+  if not disks_info and constants.DT_DISKLESS in dev_types:
+    return True
+
+  for disk in disks_info:
+    if disk.dev_type in dev_types:
+      return True
+
+  return False
+
+
+def GetDiskTemplateString(disk_types):
+  """Gives a summary disk template from disk devtypes.
+
+  @type disk_types: list of string
+  @param disk_types: all the dev_types of the instance.
+  @rtype disk template
+  @returns the summarized disk template of the disk types.
+
+  """
+  disk_types = set(dev_type for dev_type in disk_types)
+  if not disk_types:
+    return constants.DT_DISKLESS
+  elif len(disk_types) > 1:
+    return constants.DT_MIXED
+  else:
+    return disk_types.pop()
+
+
+def GetDiskTemplate(disks_info):
+  """Gives a summary disk template from disks.
+
+  @type disks_info: list of L{Disk}
+  @param disks_info: all the disks of the instance.
+  @rtype disk template
+  @returns the summarized disk template of the disk types.
+
+  """
+  return GetDiskTemplateString(d.dev_type for d in disks_info)
