@@ -104,6 +104,32 @@ def FindDuplicates(seq):
   return list(dup)
 
 
+#pylint: disable=W0142
+# (use of *-magic in argument list)
+def GetRepeatedKeys(*dicts):
+  """Return the set of keys defined multiple times in the given dicts.
+
+  >>> GetRepeatedKeys({"foo": 1, "bar": 2},
+  ...                 {"foo": 5, "baz": 7}
+  ...                )
+  set("foo")
+
+  @type dicts: dict
+  @param dicts: The dictionaries to check for duplicate keys.
+  @rtype: set
+  @return: Keys used more than once across all dicts
+
+  """
+  if len(dicts) < 2:
+    return set()
+
+  keys = []
+  for dictionary in dicts:
+    keys.extend(dictionary)
+
+  return set(FindDuplicates(keys))
+
+
 def _NiceSortTryInt(val):
   """Attempts to convert a string to an integer.
 
