@@ -87,11 +87,14 @@ class RunResult(object):
   @type failed: boolean
   @ivar failed: True in case the program was
       terminated by a signal or exited with a non-zero exit code
+  @type failed_by_timeout: boolean
+  @ivar failed_by_timeout: True in case the program was
+      terminated by timeout
   @ivar fail_reason: a string detailing the termination reason
 
   """
   __slots__ = ["exit_code", "signal", "stdout", "stderr",
-               "failed", "fail_reason", "cmd"]
+               "failed", "failed_by_timeout", "fail_reason", "cmd"]
 
   def __init__(self, exit_code, signal_, stdout, stderr, cmd, timeout_action,
                timeout):
@@ -101,6 +104,7 @@ class RunResult(object):
     self.stdout = stdout
     self.stderr = stderr
     self.failed = (signal_ is not None or exit_code != 0)
+    self.failed_by_timeout = timeout_action != _TIMEOUT_NONE
 
     fail_msgs = []
     if self.signal is not None:
@@ -756,6 +760,24 @@ def _GetProcStatusPath(pid):
   return "/proc/%d/status" % pid
 
 
+def GetProcCmdline(pid):
+  """Returns the command line of a pid as a list of arguments.
+
+  @type pid: int
+  @param pid: Process ID
+  @rtype: list of string
+
+  @raise EnvironmentError: If the process does not exist
+
+  """
+  proc_path = "/proc/%d/cmdline" % pid
+  with open(proc_path, 'r') as f:
+    nulled_cmdline = f.read()
+  # Individual arguments are separated by nul chars in the contents of the proc
+  # file
+  return nulled_cmdline.split('\x00')
+
+
 def IsProcessAlive(pid):
   """Check if a given pid exists on the system.
 
