@@ -32,6 +32,7 @@
 """
 
 
+import logging
 import time
 
 from ganeti import errors
@@ -230,3 +231,50 @@ def SimpleRetry(expected, fn, delay, timeout, args=None, wait_fn=time.sleep,
     assert "result" in rdict
     result = rdict["result"]
   return result
+
+
+def CountRetry(expected, fn, count, args=None):
+  """A wrapper over L{SimpleRetry} implementing a count down.
+
+  Where L{Retry} fixes the time, after which the command is assumed to be
+  failing, this function assumes the total number of tries.
+
+  @see: L{Retry}
+  """
+
+  rdict = {"tries": 0}
+
+  get_tries = lambda: rdict["tries"]
+
+  def inc_tries(t):
+    rdict["tries"] += t
+
+  return SimpleRetry(expected, fn, 1, count, args=args,
+                     wait_fn=inc_tries, _time_fn=get_tries)
+
+
+def RetryByNumberOfTimes(max_retries, exception_class, fn, *args, **kwargs):
+  """Retries calling a function up to the specified number of times.
+
+  @type max_retries: integer
+  @param max_retries: Maximum number of retries.
+  @type exception_class: class
+  @param exception_class: Exception class which is used for throwing the
+                          final exception.
+  @type fn: callable
+  @param fn: Function to be called (up to the specified maximum number of
+             retries.
+
+  """
+  last_exception = None
+  for i in range(max_retries):
+    try:
+      fn(*args, **kwargs)
+      break
+    except errors.OpExecError as e:
+      logging.error("Error after retry no. %s: %s.", i, e)
+      last_exception = e
+  else:
+    if last_exception:
+      raise exception_class("Error after %s retries. Last exception: %s."
+                            % (max_retries, last_exception))
