@@ -52,12 +52,10 @@ from ganeti import compat
 from ganeti import errors
 from ganeti import opcodes
 from ganeti import cli
-import ganeti.rpc.node as rpc
 import ganeti.rpc.errors as rpcerr
 from ganeti import rapi
 from ganeti import netutils
 from ganeti import qlang
-from ganeti import objects
 from ganeti import ssconf
 from ganeti import ht
 from ganeti import pathutils
@@ -105,8 +103,7 @@ def StartNodeDaemons():
   # on master or not, try to start the node daemon
   utils.EnsureDaemon(constants.NODED)
   # start confd as well. On non candidates it will be in disabled mode.
-  if constants.ENABLE_CONFD:
-    utils.EnsureDaemon(constants.CONFD)
+  utils.EnsureDaemon(constants.CONFD)
   # start mond as well: all nodes need monitoring
   if constants.ENABLE_MOND:
     utils.EnsureDaemon(constants.MOND)
@@ -477,6 +474,8 @@ def ParseOptions():
   parser.add_option("--no-wait-children", dest="wait_children",
                     action="store_false",
                     help="Don't wait for child processes")
+  parser.add_option("--no-verify-disks", dest="no_verify_disks", default=False,
+                    action="store_true", help="Do not verify disk status")
   parser.add_option("--rapi-ip", dest="rapi_ip",
                     default=constants.IP4_ADDRESS_LOCALHOST,
                     help="Use this IP to talk to RAPI.")
@@ -607,7 +606,7 @@ def GetLuxiClient(try_restart):
     # this is, from cli.GetClient, a not-master case
     raise NotMasterError("Not on master node (%s)" % err)
 
-  except rpcerr.NoMasterError, err:
+  except (rpcerr.NoMasterError, rpcerr.TimeoutError), err:
     if not try_restart:
       raise
 
@@ -866,7 +865,8 @@ def _GroupWatcher(opts):
 
     started = _CheckInstances(client, notepad, instances, locks)
     _CheckDisks(client, notepad, nodes, instances, started)
-    _VerifyDisks(client, group_uuid, nodes, instances)
+    if not opts.no_verify_disks:
+      _VerifyDisks(client, group_uuid, nodes, instances)
   except Exception, err:
     logging.info("Not updating status file due to failure: %s", err)
     raise
