@@ -97,16 +97,13 @@ def GetInstanceInfo(instance):
         else:
           nodes.append(nodestr)
 
-  disk_template = info["Disk template"]
-  if not disk_template:
-    raise qa_error.Error("Can't get instance disk template")
-  storage_type = constants.MAP_DISK_TEMPLATE_STORAGE_TYPE[disk_template]
-
   re_drbdnode = re.compile(r"^([^\s,]+),\s+minor=([0-9]+)$")
   vols = []
   drbd_min = {}
+  dtypes = []
   for (count, diskinfo) in enumerate(info["Disks"]):
     (dtype, _) = diskinfo["disk/%s" % count].split(",", 1)
+    dtypes.append(dtype)
     if dtype == constants.DT_DRBD8:
       for child in diskinfo["child devices"]:
         vols.append(child["logical_id"])
@@ -121,6 +118,10 @@ def GetInstanceInfo(instance):
     elif dtype == constants.DT_PLAIN:
       vols.append(diskinfo["logical_id"])
 
+  # TODO remove and modify calling sites
+  disk_template = utils.GetDiskTemplateString(dtypes)
+  storage_type = constants.MAP_DISK_TEMPLATE_STORAGE_TYPE[disk_template]
+
   assert nodes
   assert len(nodes) < 2 or vols
   return {
@@ -323,6 +324,22 @@ def TestInstanceAddDiskless(nodes):
     return CreateInstanceByDiskTemplateOneNode(nodes, constants.DT_DISKLESS)
 
 
+@InstanceCheck(None, INST_UP, RETURN_VALUE)
+def TestInstanceAddRADOSBlockDevice(nodes):
+  """gnt-instance add -t rbd"""
+  assert len(nodes) == 1
+  if constants.DT_RBD in qa_config.GetEnabledDiskTemplates():
+    return CreateInstanceByDiskTemplateOneNode(nodes, constants.DT_RBD)
+
+
+@InstanceCheck(None, INST_UP, RETURN_VALUE)
+def TestInstanceAddGluster(nodes):
+  """gnt-instance add -t gluster"""
+  assert len(nodes) == 1
+  if constants.DT_GLUSTER in qa_config.GetEnabledDiskTemplates():
+    return CreateInstanceByDiskTemplateOneNode(nodes, constants.DT_GLUSTER)
+
+
 @InstanceCheck(None, INST_DOWN, FIRST_ARG)
 def TestInstanceRemove(instance):
   """gnt-instance remove"""
@@ -609,14 +626,20 @@ def TestInstanceModify(instance):
     ["-B", "%s=%s" % (constants.BE_ALWAYS_FAILOVER, constants.VALUE_TRUE)],
     ["-B", "%s=%s" % (constants.BE_ALWAYS_FAILOVER, constants.VALUE_DEFAULT)],
 
-    ["-H", "%s=%s" % (constants.HV_KERNEL_PATH, test_kernel)],
-    ["-H", "%s=%s" % (constants.HV_KERNEL_PATH, constants.VALUE_DEFAULT)],
-
     # TODO: bridge tests
     #["--bridge", "xen-br1"],
     #["--bridge", orig_bridge],
     ]
 
+  # Not all hypervisors support kernel_path(e.g, LXC)
+  if default_hv in (constants.HT_XEN_PVM,
+                    constants.HT_XEN_HVM,
+                    constants.HT_KVM):
+    args.extend([
+      ["-H", "%s=%s" % (constants.HV_KERNEL_PATH, test_kernel)],
+      ["-H", "%s=%s" % (constants.HV_KERNEL_PATH, constants.VALUE_DEFAULT)],
+      ])
+
   if default_hv == constants.HT_XEN_PVM:
     args.extend([
       ["-H", "%s=%s" % (constants.HV_INITRD_PATH, test_initrd)],
@@ -631,6 +654,13 @@ def TestInstanceModify(instance):
   elif default_hv == constants.HT_KVM and \
     qa_config.TestEnabled("instance-device-hotplug"):
     _TestKVMHotplug(instance)
+  elif default_hv == constants.HT_LXC:
+    args.extend([
+      ["-H", "%s=0" % constants.HV_CPU_MASK],
+      ["-H", "%s=%s" % (constants.HV_CPU_MASK, constants.VALUE_DEFAULT)],
+      ["-H", "%s=0" % constants.HV_LXC_NUM_TTYS],
+      ["-H", "%s=%s" % (constants.HV_LXC_NUM_TTYS, constants.VALUE_DEFAULT)],
+      ])
 
   url = "http://example.com/busybox.img"
   args.extend([
@@ -718,20 +748,47 @@ def TestInstanceStoppedModify(instance):
 
 
 @InstanceCheck(INST_DOWN, INST_DOWN, FIRST_ARG)
-def TestInstanceConvertDiskToPlain(instance, inodes):
+def TestInstanceConvertDiskTemplate(instance, requested_conversions):
   """gnt-instance modify -t"""
+  def _BuildConvertCommand(disk_template, node):
+    cmd = ["gnt-instance", "modify", "-t", disk_template]
+    if disk_template == constants.DT_DRBD8:
+      cmd.extend(["-n", node])
+    cmd.append(name)
+    return cmd
+
+  if len(requested_conversions) < 2:
+    print qa_utils.FormatInfo("You must specify more than one convertible"
+                              " disk templates in order to test the conversion"
+                              " feature")
+    return
+
   name = instance.name
 
   template = instance.disk_template
-  if template != constants.DT_DRBD8:
+  if template in constants.DTS_NOT_CONVERTIBLE_FROM:
     print qa_utils.FormatInfo("Unsupported template %s, skipping conversion"
                               " test" % template)
     return
 
-  assert len(inodes) == 2
-  AssertCommand(["gnt-instance", "modify", "-t", constants.DT_PLAIN, name])
-  AssertCommand(["gnt-instance", "modify", "-t", constants.DT_DRBD8,
-                 "-n", inodes[1].primary, name])
+  inodes = qa_config.AcquireManyNodes(2)
+  master = qa_config.GetMasterNode()
+
+  snode = inodes[0].primary
+  if master.primary == snode:
+    snode = inodes[1].primary
+
+  enabled_disk_templates = qa_config.GetEnabledDiskTemplates()
+
+  for templ in requested_conversions:
+    if (templ == template or
+        templ not in enabled_disk_templates or
+        templ in constants.DTS_NOT_CONVERTIBLE_TO):
+      continue
+    AssertCommand(_BuildConvertCommand(templ, snode))
+
+  # Before we return, convert to the original template
+  AssertCommand(_BuildConvertCommand(template, snode))
 
 
 @InstanceCheck(INST_UP, INST_UP, FIRST_ARG)
@@ -1027,10 +1084,13 @@ def TestRecreateDisks(instance, inodes, othernodes):
 def TestInstanceExport(instance, node):
   """gnt-backup export -n ..."""
   name = instance.name
-  # Export does not work for file-based templates, thus we skip the test
+  options = ["gnt-backup", "export", "-n", node.primary]
+
+  # For files and shared files, the --long-sleep option should be used
   if instance.disk_template in [constants.DT_FILE, constants.DT_SHARED_FILE]:
-    return
-  AssertCommand(["gnt-backup", "export", "-n", node.primary, name])
+    options.append("--long-sleep")
+
+  AssertCommand(options + [name])
   return qa_utils.ResolveInstanceName(name)
 
 
@@ -1306,7 +1366,10 @@ def _TestInstanceUserDownXen(instance):
   primary = _GetInstanceField(instance.name, "pnode")
   fn = lambda: AssertCommand(["xm", "shutdown", "-w", instance.name],
                              node=primary)
+
+  AssertCommand(["gnt-cluster", "modify", "--user-shutdown=true"])
   _TestInstanceUserDown(instance, fn)
+  AssertCommand(["gnt-cluster", "modify", "--user-shutdown=false"])
 
 
 @InstanceCheck(INST_UP, INST_UP, FIRST_ARG)
@@ -1467,4 +1530,8 @@ available_instance_tests = [
    TestInstanceAddFile, 1),
   ("instance-add-shared-file", constants.DT_SHARED_FILE,
    TestInstanceAddSharedFile, 1),
+  ("instance-add-rbd", constants.DT_RBD,
+   TestInstanceAddRADOSBlockDevice, 1),
+  ("instance-add-gluster", constants.DT_GLUSTER,
+   TestInstanceAddGluster, 1),
   ]
