@@ -73,6 +73,9 @@ _RUN_UUID = utils.NewUUID()
 _QA_OUTPUT = pathutils.GetLogFilename("qa-output")
 
 
+_RETRIES = 3
+
+
 (INST_DOWN,
  INST_UP) = range(500, 502)
 
@@ -225,6 +228,19 @@ def AssertCommand(cmd, fail=False, node=None, log_cmd=True, max_seconds=None):
   return rcode, stdout, stderr
 
 
+def stdout_of(cmd):
+  """Small helper to run a stdout_of.
+  Makes sure the stdout_of returns exit code 0.
+
+  @type cmd: list of strings
+  @param cmd: the stdout_of to run
+
+  @return: Captured, stripped stdout.
+  """
+  _, out, _ = AssertCommand(cmd)
+  return out.strip()
+
+
 def AssertRedirectedCommand(cmd, fail=False, node=None, log_cmd=True):
   """Executes a command with redirected output.
 
@@ -246,7 +262,7 @@ def AssertRedirectedCommand(cmd, fail=False, node=None, log_cmd=True):
                        fail=fail, node=node, log_cmd=log_cmd)
 
 
-def GetSSHCommand(node, cmd, strict=True, opts=None, tty=None,
+def GetSSHCommand(node, cmd, strict=True, opts=None, tty=False,
                   use_multiplexer=True):
   """Builds SSH command to be executed.
 
@@ -273,11 +289,7 @@ def GetSSHCommand(node, cmd, strict=True, opts=None, tty=None,
   if tty:
     args.append("-t")
 
-  if strict:
-    tmp = "yes"
-  else:
-    tmp = "no"
-  args.append("-oStrictHostKeyChecking=%s" % tmp)
+  args.append("-oStrictHostKeyChecking=%s" % ("yes" if strict else "no", ))
   args.append("-oClearAllForwardings=yes")
   args.append("-oForwardAgent=yes")
   if opts:
@@ -369,7 +381,14 @@ def _GetCommandStdout(proc):
   return out
 
 
-def GetCommandOutput(node, cmd, tty=None, use_multiplexer=True, log_cmd=True,
+def _NoTimeout(state):
+  """False iff the command timed out."""
+  rcode, out = state
+
+  return rcode == 0 or not ('TimeoutError' in out or 'timed out' in out)
+
+
+def GetCommandOutput(node, cmd, tty=False, use_multiplexer=True, log_cmd=True,
                      fail=False):
   """Returns the output of a command executed on the given node.
 
@@ -388,11 +407,17 @@ def GetCommandOutput(node, cmd, tty=None, use_multiplexer=True, log_cmd=True,
   @param fail: whether the command is expected to fail
   """
   assert cmd
-  p = StartLocalCommand(GetSSHCommand(node, cmd, tty=tty,
-                                      use_multiplexer=use_multiplexer),
-                        stdout=subprocess.PIPE, log_cmd=log_cmd)
-  rcode = p.wait()
-  out = _GetCommandStdout(p)
+
+  def CallCommand():
+    command = GetSSHCommand(node, cmd, tty=tty,
+                            use_multiplexer=use_multiplexer)
+    p = StartLocalCommand(command, stdout=subprocess.PIPE, log_cmd=log_cmd)
+    rcode = p.wait()
+    out = _GetCommandStdout(p)
+    return rcode, out
+
+  # TODO: make retries configurable
+  rcode, out = utils.CountRetry(_NoTimeout, CallCommand, _RETRIES)
   _AssertRetCode(rcode, fail, cmd, node)
   return out
 
