@@ -138,7 +138,8 @@ queryInstancesMsg =
      ["name", "disk_usage", "be/memory", "be/vcpus",
       "status", "pnode", "snodes", "tags", "oper_ram",
       "be/auto_balance", "disk_template",
-      "be/spindle_use", "disk.sizes", "disk.spindles"] Qlang.EmptyFilter
+      "be/spindle_use", "disk.sizes", "disk.spindles",
+      "forthcoming"] Qlang.EmptyFilter
 
 -- | The input data for cluster query.
 queryClusterInfoMsg :: L.LuxiOp
@@ -180,7 +181,7 @@ parseInstance :: NameAssoc
 parseInstance ktn [ name, disk, mem, vcpus
                   , status, pnode, snodes, tags, oram
                   , auto_balance, disk_template, su
-                  , dsizes, dspindles ] = do
+                  , dsizes, dspindles, forthcoming ] = do
   xname <- annotateResult "Parsing new instance" (fromJValWithStatus name)
   let convert a = genericConvert "Instance" xname a
   xdisk <- convert "disk_usage" disk
@@ -200,9 +201,11 @@ parseInstance ktn [ name, disk, mem, vcpus
   xsu <- convert "be/spindle_use" su
   xdsizes <- convert "disk.sizes" dsizes
   xdspindles <- convertArrayMaybe "Instance" xname "disk.spindles" dspindles
+  xforthcoming <- convert "forthcoming" forthcoming
   let disks = zipWith Instance.Disk xdsizes xdspindles
       inst = Instance.create xname xmem xdisk disks
              xvcpus xrunning xtags xauto_balance xpnode snode xdt xsu []
+             xforthcoming
   return (xname, inst)
 
 parseInstance _ v = fail ("Invalid instance query result: " ++ show v)
