@@ -60,7 +60,10 @@ module Ganeti.HTools.CLI
   , oDynuFile
   , oMonD
   , oMonDDataFile
+  , oMonDXen
   , oEvacMode
+  , oMonDExitMissing
+  , oFirstJobGroup
   , oRestrictedMigrate
   , oExInst
   , oExTags
@@ -71,6 +74,7 @@ module Ganeti.HTools.CLI
   , oIAllocSrc
   , oIgnoreDyn
   , oIgnoreNonRedundant
+  , oIgnoreSoftErrors
   , oIndependentGroups
   , oAcceptExisting
   , oInstMoves
@@ -99,6 +103,7 @@ module Ganeti.HTools.CLI
   , oPrintNodes
   , oQuiet
   , oRapiMaster
+  , oReason
   , oSaveCluster
   , oSelInst
   , oShowHelp
@@ -110,6 +115,7 @@ module Ganeti.HTools.CLI
   , oTieredSpec
   , oVerbose
   , oPriority
+  , oNoCapacityChecks
   , genericOpts
   ) where
 
@@ -140,16 +146,22 @@ data Options = Options
   , optSpindleUse  :: Maybe Int      -- ^ Override for the spindle usage
   , optDynuFile    :: Maybe FilePath -- ^ Optional file with dynamic use data
   , optIgnoreDynu  :: Bool           -- ^ Do not use dynamic use data
+  , optIgnoreSoftErrors :: Bool      -- ^ Ignore soft errors in balancing moves
   , optIndependentGroups :: Bool     -- ^ consider groups independently
   , optAcceptExisting :: Bool        -- ^ accept existing N+1 violations
   , optMonD        :: Bool           -- ^ Query MonDs
   , optMonDFile    :: Maybe FilePath -- ^ Optional file with data provided
                                      -- by MonDs
+  , optMonDXen     :: Bool           -- ^ Should Xen-specific collectors be
+                                     -- considered (only if MonD is queried)
+  , optMonDExitMissing :: Bool       -- ^ If the program should exit on missing
+                                     -- MonD data
   , optEvacMode    :: Bool           -- ^ Enable evacuation mode
   , optRestrictedMigrate :: Bool     -- ^ Disallow replace-primary moves
   , optExInst      :: [String]       -- ^ Instances to be excluded
   , optExTags      :: Maybe [String] -- ^ Tags to use for exclusion
   , optExecJobs    :: Bool           -- ^ Execute the commands via Luxi
+  , optFirstJobGroup :: Bool         -- ^ Only execute the first group of jobs
   , optForce       :: Bool           -- ^ Force the execution
   , optFullEvacuation :: Bool        -- ^ Fully evacuate nodes to be rebooted
   , optGroup       :: Maybe GroupID  -- ^ The UUID of the group to process
@@ -176,6 +188,8 @@ data Options = Options
   , optOneStepOnly :: Bool           -- ^ Only do the first step
   , optOutPath     :: FilePath       -- ^ Path to the output directory
   , optPrintMoves  :: Bool           -- ^ Whether to show the instance moves
+  , optReason      :: Maybe String   -- ^ The reason to be passed when
+                                     -- submitting jobs
   , optSaveCluster :: Maybe FilePath -- ^ Save cluster state to this file
   , optShowCmds    :: Maybe FilePath -- ^ Whether to show the command list
   , optShowHelp    :: Bool           -- ^ Just show the help
@@ -191,6 +205,7 @@ data Options = Options
   , optReplay      :: Maybe String   -- ^ Unittests: RNG state
   , optVerbose     :: Int            -- ^ Verbosity level
   , optPriority    :: Maybe OpSubmitPriority -- ^ OpCode submit priority
+  , optCapacity    :: Bool           -- ^ Also do capacity-related checks
   } deriving Show
 
 -- | Default values for the command line options.
@@ -204,14 +219,18 @@ defaultOptions  = Options
   , optDiskTemplate = Nothing
   , optSpindleUse  = Nothing
   , optIgnoreDynu  = False
+  , optIgnoreSoftErrors = False
   , optDynuFile    = Nothing
   , optMonD        = False
   , optMonDFile = Nothing
+  , optMonDXen     = False
+  , optMonDExitMissing = False
   , optEvacMode    = False
   , optRestrictedMigrate = False
   , optExInst      = []
   , optExTags      = Nothing
   , optExecJobs    = False
+  , optFirstJobGroup = False
   , optForce       = False
   , optFullEvacuation = False
   , optGroup       = Nothing
@@ -239,6 +258,7 @@ defaultOptions  = Options
   , optOneStepOnly = False
   , optOutPath     = "."
   , optPrintMoves  = False
+  , optReason      = Nothing
   , optSaveCluster = Nothing
   , optShowCmds    = Nothing
   , optShowHelp    = False
@@ -253,6 +273,7 @@ defaultOptions  = Options
   , optReplay      = Nothing
   , optVerbose     = 1
   , optPriority    = Nothing
+  , optCapacity    = True
   }
 
 -- | Abbreviation for the option type.
@@ -328,6 +349,20 @@ oMonDDataFile =
    "Import data provided by MonDs from the given FILE",
    OptComplFile)
 
+oMonDXen :: OptType
+oMonDXen =
+  (Option "" ["mond-xen"]
+    (NoArg (\ opts -> Ok opts { optMonDXen = True }))
+    "also consider xen-specific collectors in MonD queries",
+    OptComplNone)
+
+oMonDExitMissing :: OptType
+oMonDExitMissing =
+  (Option "" ["exit-on-missing-mond-data"]
+    (NoArg (\ opts -> Ok opts { optMonDExitMissing = True }))
+    "abort if the data available from the monitoring daemons is incomplete",
+    OptComplNone)
+
 oDiskTemplate :: OptType
 oDiskTemplate =
   (Option "" ["disk-template"]
@@ -378,6 +413,13 @@ oIgnoreDyn =
    "Ignore any dynamic utilisation information",
    OptComplNone)
 
+oIgnoreSoftErrors :: OptType
+oIgnoreSoftErrors =
+  (Option "" ["ignore-soft-errors"]
+   (NoArg (\ opts -> Ok opts {optIgnoreSoftErrors = True}))
+   "Ignore any soft restrictions in balancing",
+   OptComplNone)
+
 oIndependentGroups :: OptType
 oIndependentGroups =
   (Option "" ["independent-groups"]
@@ -430,6 +472,20 @@ oExecJobs =
    \ it for data gathering)",
    OptComplNone)
 
+oReason :: OptType
+oReason =
+  (Option "" ["reason"]
+   (ReqArg (\ f opts -> Ok opts { optReason = Just f }) "REASON")
+   "The reason to pass to the submitted jobs",
+   OptComplNone)
+
+oFirstJobGroup :: OptType
+oFirstJobGroup =
+  (Option "" ["first-job-group"]
+   (NoArg (\ opts -> Ok opts {optFirstJobGroup = True}))
+   "only execute the first group of jobs",
+   OptComplNone)
+
 oForce :: OptType
 oForce =
   (Option "f" ["force"]
@@ -558,7 +614,7 @@ oMinScore =
   (Option "e" ["min-score"]
    (reqWithConversion (tryRead "min score")
     (\e opts -> Ok opts { optMinScore = e }) "EPSILON")
-   "mininum score to aim for",
+   "mininum excess to the N+1 limit to aim for",
    OptComplFloat)
 
 oNoHeaders :: OptType
@@ -730,6 +786,13 @@ oPriority =
    "set the priority of submitted jobs",
     OptComplChoices (map fmtSubmitPriority [minBound..maxBound]))
 
+oNoCapacityChecks :: OptType
+oNoCapacityChecks =
+  (Option "" ["no-capacity-checks"]
+   (NoArg (\ opts -> Ok opts { optCapacity = False}))
+   "disable capacity checks (like global N+1 redundancy)",
+   OptComplNone)
+
 -- | Generic options.
 genericOpts :: [GenericOptType Options]
 genericOpts =  [ oShowVer
