@@ -49,14 +49,17 @@ import System.IO
 
 import Text.Printf (printf, hPrintf)
 
+import qualified Ganeti.HTools.AlgorithmParams as Alg
 import qualified Ganeti.HTools.Container as Container
 import qualified Ganeti.HTools.Cluster as Cluster
+import qualified Ganeti.HTools.Cluster.Metrics as Metrics
 import qualified Ganeti.HTools.Group as Group
 import qualified Ganeti.HTools.Node as Node
 import qualified Ganeti.HTools.Instance as Instance
 
 import Ganeti.BasicTypes
 import Ganeti.Common
+import Ganeti.HTools.AlgorithmParams (AlgorithmOptions)
 import Ganeti.HTools.Types
 import Ganeti.HTools.CLI
 import Ganeti.HTools.ExtLoader
@@ -377,8 +380,8 @@ printTiered False spec_map ini_nl fin_nl sreason = do
 -- | Displays the initial/final cluster scores.
 printClusterScores :: Node.List -> Node.List -> IO ()
 printClusterScores ini_nl fin_nl = do
-  printf "  - initial cluster score: %.8f\n" $ Cluster.compCV ini_nl::IO ()
-  printf "  -   final cluster score: %.8f\n" $ Cluster.compCV fin_nl
+  printf "  - initial cluster score: %.8f\n" $ Metrics.compCV ini_nl::IO ()
+  printf "  -   final cluster score: %.8f\n" $ Metrics.compCV fin_nl
 
 -- | Displays the cluster efficiency.
 printClusterEff :: Cluster.CStats -> Bool -> IO ()
@@ -439,11 +442,15 @@ instFromSpec :: RSpec -> DiskTemplate -> Int -> Instance.Instance
 instFromSpec spx dt su =
   Instance.create "new" (rspecMem spx) (rspecDsk spx)
     [Instance.Disk (rspecDsk spx) (Just $ rspecSpn spx)]
-    (rspecCpu spx) Running [] True (-1) (-1) dt su []
+    (rspecCpu spx) Running [] True (-1) (-1) dt su [] False
 
-combineTiered :: Maybe Int -> Cluster.AllocNodes -> Cluster.AllocResult ->
-           Instance.Instance -> Result Cluster.AllocResult
-combineTiered limit allocnodes result inst = do
+combineTiered :: AlgorithmOptions
+              -> Maybe Int
+              -> Cluster.AllocNodes
+              -> Cluster.AllocResult
+              -> Instance.Instance
+              -> Result Cluster.AllocResult
+combineTiered algOpts limit allocnodes result inst = do
   let (_, nl, il, ixes, cstats) = result
       ixes_cnt = length ixes
       (stop, newlimit) = case limit of
@@ -451,7 +458,7 @@ combineTiered limit allocnodes result inst = do
         Just n -> (n <= ixes_cnt, Just (n - ixes_cnt))
   if stop
     then return result
-    else Cluster.tieredAlloc nl il newlimit inst allocnodes ixes cstats
+    else Cluster.tieredAlloc algOpts nl il newlimit inst allocnodes ixes cstats
 
 -- | Main function.
 main :: Options -> [String] -> IO ()
@@ -487,13 +494,17 @@ main opts args = do
 
   when (verbose > 2) $
          hPrintf stderr "Initial coefficients: overall %.8f\n%s"
-                 (Cluster.compCV nl) (Cluster.printStats "  " nl)
+                 (Metrics.compCV nl) (Metrics.printStats "  " nl)
 
   printCluster machine_r (Cluster.totalResources nl) (length all_nodes)
     (Node.haveExclStorage nl)
 
   let (bad_nodes, _)  = Cluster.computeBadItems nl il
-      gl' = if accept_existing
+
+  when ((verbose > 3) && (not . null $ bad_nodes))
+    . hPrintf stderr "Bad nodes: %s\n" . show . map Node.name $ bad_nodes
+
+  let gl' = if accept_existing
               then gl
               else foldl (flip $ IntMap.adjust Group.setUnallocable) gl
                      (map Node.group bad_nodes)
@@ -521,13 +532,19 @@ main opts args = do
                             minmaxes
                  Just t -> [t]
       tinsts = map (\ts -> instFromSpec ts disk_template su) tspecs
+
+      algOpts = Alg.fromCLIOptions opts
+
   tspec <- case tspecs of
     [] -> exitErr "Empty list of specs received from the cluster"
     t:_ -> return t
 
   (treason, trl_nl, _, spec_map) <-
     runAllocation cdata stop_allocation
-       (foldM (combineTiered alloclimit allocnodes) ([], nl, il, [], []) tinsts)
+       (foldM (combineTiered algOpts alloclimit allocnodes)
+              ([], nl, il, [], [])
+              tinsts
+       )
        tspec disk_template SpecTiered opts
 
   printTiered machine_r spec_map nl trl_nl treason
@@ -539,7 +556,7 @@ main opts args = do
 
   (sreason, fin_nl, allocs, _) <-
       runAllocation cdata stop_allocation
-            (Cluster.iterateAlloc nl il alloclimit
+            (Cluster.iterateAlloc algOpts nl il alloclimit
              (instFromSpec ispec disk_template su) allocnodes [] [])
             ispec disk_template SpecNormal opts
 
