@@ -39,6 +39,7 @@ module Ganeti.HTools.Program.Hsqueeze
   ) where
 
 import Control.Applicative
+import Control.Lens (over)
 import Control.Monad
 import Data.Function
 import Data.List
@@ -48,16 +49,26 @@ import Text.Printf (printf)
 
 import Ganeti.BasicTypes
 import Ganeti.Common
+import qualified Ganeti.HTools.AlgorithmParams as Alg
 import Ganeti.HTools.CLI
 import qualified Ganeti.HTools.Container as Container
 import qualified Ganeti.HTools.Cluster as Cluster
+import qualified Ganeti.HTools.Cluster.Metrics as Metrics
 import Ganeti.HTools.ExtLoader
 import qualified Ganeti.HTools.Instance as Instance
 import Ganeti.HTools.Loader
 import qualified Ganeti.HTools.Node as Node
+import Ganeti.HTools.Tags (hasStandbyTag, standbyAuto)
 import Ganeti.HTools.Types
+import Ganeti.JQueue (currentTimestamp, reasonTrailTimestamp)
+import Ganeti.JQueue.Objects (Timestamp)
+import qualified Ganeti.Jobs as Jobs
+import Ganeti.OpCodes
+import Ganeti.OpCodes.Lens (metaParamsL, opReasonL)
 import Ganeti.Utils
 
+import Ganeti.Version (version)
+
 -- | Options list and functions.
 options :: IO [OptType]
 options = do
@@ -65,6 +76,7 @@ options = do
   return
     [ luxi
     , oDataFile
+    , oExecJobs
     , oMinResources
     , oTargetResources
     , oSaveCluster
@@ -77,14 +89,17 @@ options = do
 arguments :: [ArgCompletion]
 arguments = []
 
--- | The tag-prefix indicating that hsqueeze should consider a node
--- as being standby.
-standbyPrefix :: String
-standbyPrefix = "htools:standby:"
-
--- | Predicate of having a standby tag.
-hasStandbyTag :: Node.Node -> Bool
-hasStandbyTag = any (standbyPrefix `isPrefixOf`) . Node.nTags
+-- | Wraps an 'OpCode' in a 'MetaOpCode' while also adding a comment
+-- about what generated the opcode.
+annotateOpCode :: Timestamp -> String -> Jobs.Annotator
+annotateOpCode ts comment =
+  over (metaParamsL . opReasonL)
+      (++ [("hsqueeze"
+           , "hsqueeze " ++ version ++ " called"
+           , reasonTrailTimestamp ts
+           )])
+  . setOpComment (comment ++ " " ++ version)
+  . wrapOpCode
 
 -- | Within a cluster configuration, decide if the node hosts only
 -- externally-mirrored instances.
@@ -117,9 +132,11 @@ allNodesCapacityFor inst (nl, _) =
 balance :: (Node.List, Instance.List) 
            -> ((Node.List, Instance.List), [MoveJob])
 balance (nl, il) =
-  let ini_cv = Cluster.compCV nl
+  let ini_cv = Metrics.compCV nl
       ini_tbl = Cluster.Table nl il ini_cv []
-      balanceStep tbl = Cluster.tryBalance tbl True True False False 0.0 0.0
+      balanceStep = Cluster.tryBalance
+                      (Alg.defaultOptions { Alg.algMinGain = 0.0
+                                          , Alg.algMinGainLimit = 0.0})
       bTables = map fromJust . takeWhile isJust
                   $ iterate (>>= balanceStep) (Just ini_tbl)
       (Cluster.Table nl' il' _ _) = last bTables
@@ -203,6 +220,55 @@ instanceFromSpecAndFactor name f spec =
     Running [] False Node.noSecondary Node.noSecondary DTExt
     (floor (f * fromIntegral (iSpecSpindleUse spec)))
     []
+    False
+
+-- | Get opcodes for the given move job.
+getMoveOpCodes :: Node.List
+               -> Instance.List
+               -> [JobSet]
+               -> Result [([[OpCode]], String)]
+getMoveOpCodes nl il js = return $ zip (map opcodes js) (map descr js)
+  where opcodes = map (\(_, idx, move, _) ->
+                      Cluster.iMoveToJob nl il idx move)
+        descr job = "Moving instances " ++ commaJoin
+                       (map (\(_, idx, _, _) -> Container.nameOf il idx) job)
+
+-- | Get opcodes for tagging nodes with standby.
+getTagOpCodes ::  [Node.Node] -> Result [([[OpCode]], String)]
+getTagOpCodes nl = return $ zip (map opCode nl) (map descr nl)
+  where
+    opCode node = [[Node.genAddTagsOpCode node [standbyAuto]]]
+    descr node = "Tagging node " ++ Node.name node ++ " with standby"
+
+-- | Get opcodes for powering off nodes
+getPowerOffOpCodes :: [Node.Node] -> Result [([[OpCode]], String)]
+getPowerOffOpCodes nl = do
+  opcodes <- Node.genPowerOffOpCodes nl
+  return [([opcodes], "Powering off nodes")]
+
+-- | Get opcodes for powering on nodes
+getPowerOnOpCodes :: [Node.Node] -> Result [([[OpCode]], String)]
+getPowerOnOpCodes nl = do
+  opcodes <- Node.genPowerOnOpCodes nl
+  return [([opcodes], "Powering on nodes")]
+
+maybeExecJobs :: Options
+              -> String
+              -> Result [([[OpCode]], String)]
+              -> IO (Result ())
+maybeExecJobs opts comment opcodes =
+  if optExecJobs opts
+    then (case optLuxi opts of
+            Nothing ->
+              return $ Bad "Execution of commands possible only on LUXI"
+            Just master -> do
+              ts <- currentTimestamp
+              let annotator = maybe id setOpPriority (optPriority opts) .
+                              annotateOpCode ts comment
+              case opcodes of
+                Bad msg -> error msg
+                Ok codes -> Jobs.execWithCancel annotator master codes)
+    else return $ Ok ()
 
 -- | Main function.
 main :: Options -> [String] -> IO ()
@@ -239,10 +305,16 @@ main opts args = do
       final_off_cdata =
         ini_cdata { cdNodes = fin_off_nl, cdInstances = fin_off_il }
       off_jobs = Cluster.splitJobs off_mvs
+      off_opcodes = liftM concat $ sequence
+                    [ getMoveOpCodes nlf ilf off_jobs
+                    , getTagOpCodes toOffline
+                    , getPowerOffOpCodes toOffline
+                    ]
       off_cmd =
         Cluster.formatCmds off_jobs
         ++ "\necho Tagging Commands\n"
-        ++ (toOffline >>= printf "  gnt-node add-tags %s htools:standby:auto\n" 
+        ++ (toOffline >>= (printf "  gnt-node add-tags %s %s\n"
+                             `flip` standbyAuto)
                           . Node.alias)
         ++ "\necho Power Commands\n"
         ++ (toOffline >>= printf "  gnt-node power -f off %s\n" . Node.alias)
@@ -253,6 +325,8 @@ main opts args = do
       final_on_cdata =
         ini_cdata { cdNodes = fin_on_nl, cdInstances = fin_on_il }
       on_jobs = Cluster.splitJobs on_mvs
+      on_opcodes = liftM2 (++) (getPowerOnOpCodes nodesToOnline)
+                               (getMoveOpCodes nlf ilf on_jobs)
       on_cmd =
         "echo Power Commands\n"
         ++ (nodesToOnline >>= printf "  gnt-node power -f on %s\n" . Node.alias)
@@ -272,6 +346,8 @@ main opts args = do
       when (verbose > 1 && isNothing toOnline) . putStrLn $
         "Onlining all nodes will not yield enough capacity"
       maybeSaveCommands "Commands to run:" opts on_cmd
+      let comment = printf "expanding by %d nodes" (length nodesToOnline)
+      exitIfBad "hsqueeze" =<< maybeExecJobs opts comment on_opcodes
       maybeSaveData (optSaveCluster opts)
          "squeezed" "after hsqueeze expansion" final_on_cdata
     else
@@ -287,5 +363,7 @@ main opts args = do
             putStrLn "'Nodes to offline'"
           mapM_ (putStrLn . Node.name) toOffline
           maybeSaveCommands "Commands to run:" opts off_cmd
+          let comment = printf "condensing by %d nodes" (length toOffline)
+          exitIfBad "hsqueeze" =<< maybeExecJobs opts comment off_opcodes
           maybeSaveData (optSaveCluster opts)
             "squeezed" "after hsqueeze run" final_off_cdata
