@@ -101,7 +101,7 @@ import System.Directory
 import System.FilePath
 import System.IO.Error (isDoesNotExistError)
 import System.Posix.Files
-import System.Posix.Signals (sigHUP, sigTERM, sigUSR1, signalProcess)
+import System.Posix.Signals (sigHUP, sigTERM, sigUSR1, sigKILL, signalProcess)
 import System.Posix.Types (ProcessID)
 import System.Time
 import qualified Text.JSON
@@ -503,7 +503,10 @@ readSerialFromDisk = do
 allocateJobIds :: [Node] -> Lock -> Int -> IO (Result [JobId])
 allocateJobIds mastercandidates lock n =
   if n <= 0
-    then return . Bad $ "Can only allocate positive number of job ids"
+    then if n == 0
+           then return $ Ok []
+           else return . Bad
+                  $ "Can only allocate non-negative number of job ids"
     else withLock lock $ do
       rjobid <- readSerialFromDisk
       case rjobid of
@@ -536,15 +539,15 @@ isQueueOpen :: IO Bool
 isQueueOpen = liftM not (jobQueueDrainFile >>= doesFileExist)
 
 -- | Start enqueued jobs by executing the Python code.
-startJobs :: ConfigData
-          -> Livelock -- ^ Luxi's livelock path
+startJobs :: Livelock -- ^ Luxi's livelock path
           -> Lock -- ^ lock for forking new processes
           -> [QueuedJob] -- ^ the list of jobs to start
           -> IO [ErrorResult QueuedJob]
-startJobs cfg luxiLivelock forkLock jobs = do
+startJobs luxiLivelock forkLock jobs = do
   qdir <- queueDir
   let updateJob job llfile =
-        void . writeAndReplicateJob cfg qdir $ job { qjLivelock = Just llfile }
+        void . mkResultT . writeJobToDisk qdir
+          $ job { qjLivelock = Just llfile }
   let runJob job = withLock forkLock $ do
         (llfile, _) <- Exec.forkJobProcess (qjId job) luxiLivelock
                                            (updateJob job)
@@ -581,10 +584,11 @@ waitForJob jid tmout = do
 
 -- | Try to cancel a job that has already been handed over to execution,
 -- by terminating the process.
-cancelJob :: Livelock -- ^ Luxi's livelock path
+cancelJob :: Bool -- ^ if True, use sigKILL instead of sigTERM
+          -> Livelock -- ^ Luxi's livelock path
           -> JobId -- ^ the job to cancel
           -> IO (ErrorResult (Bool, String))
-cancelJob luxiLivelock jid = runResultT $ do
+cancelJob kill luxiLivelock jid = runResultT $ do
   -- we can't terminate the job if it's just being started, so
   -- retry several times in such a case
   result <- runMaybeT . msum . flip map [0..5 :: Int] $ \tryNo -> do
@@ -601,11 +605,13 @@ cancelJob luxiLivelock jid = runResultT $ do
       _ | dead ->
         return (True, jName ++ " has been already dead")
       Just pid -> do
-        liftIO $ signalProcess sigTERM pid
-        if calcJobStatus job > JOB_STATUS_WAITING
-          then return (False, "Job no longer waiting, can't cancel\
-                              \ (informed it anyway)")
-          else lift $ waitForJob jid C.luxiCancelJobTimeout
+        liftIO $ signalProcess (if kill then sigKILL else sigTERM) pid
+        if not kill then
+          if calcJobStatus job > JOB_STATUS_WAITING
+            then return (False, "Job no longer waiting, can't cancel\
+                                \ (informed it anyway)")
+            else lift $ waitForJob jid C.luxiCancelJobTimeout
+          else return (True, "SIGKILL send to the process")
       _ -> do
         logDebug $ jName ++ " in its startup phase, retrying"
         mzero
