@@ -108,6 +108,7 @@ module Ganeti.Objects
 import Control.Applicative
 import Control.Arrow (first)
 import Control.Monad.State
+import qualified Data.ByteString.UTF8 as UTF8
 import Data.List (foldl', intercalate)
 import Data.Maybe
 import qualified Data.Map as Map
@@ -268,7 +269,7 @@ instance TagsObject Network where
   tagsOf = networkTags
 
 instance UuidObject Network where
-  uuidOf = networkUuid
+  uuidOf = UTF8.toString . networkUuid
 
 instance TimeStampObject Network where
   cTimeOf = networkCtime
@@ -292,7 +293,6 @@ instance Monoid DataCollectorConfig where
     }
   mappend _ a = a
 
-
 -- * IPolicy definitions
 
 $(buildParam "ISpec" "ispec"
@@ -389,6 +389,18 @@ $(buildParam "ND" "ndp"
   , simpleField "cpu_speed"     [t| Double |]
   ])
 
+-- | Disk state parameters.
+--
+-- As according to the documentation this option is unused by Ganeti,
+-- the content is just a 'JSValue'.
+type DiskState = Container JSValue
+
+-- | Hypervisor state parameters.
+--
+-- As according to the documentation this option is unused by Ganeti,
+-- the content is just a 'JSValue'.
+type HypervisorState = Container JSValue
+
 $(buildObject "Node" "node" $
   [ simpleField "name"             [t| String |]
   , simpleField "primary_ip"       [t| String |]
@@ -401,6 +413,10 @@ $(buildObject "Node" "node" $
   , simpleField "vm_capable"       [t| Bool   |]
   , simpleField "ndparams"         [t| PartialNDParams |]
   , simpleField "powered"          [t| Bool   |]
+  , notSerializeDefaultField [| emptyContainer |] $
+    simpleField "hv_state_static"   [t| HypervisorState |]
+  , notSerializeDefaultField [| emptyContainer |] $
+    simpleField "disk_state_static" [t| DiskState       |]
   ]
   ++ timeStampFields
   ++ uuidFields
@@ -412,7 +428,7 @@ instance TimeStampObject Node where
   mTimeOf = nodeMtime
 
 instance UuidObject Node where
-  uuidOf = nodeUuid
+  uuidOf = UTF8.toString . nodeUuid
 
 instance SerialNoObject Node where
   serialOf = nodeSerial
@@ -436,6 +452,10 @@ $(buildObject "NodeGroup" "group" $
   , simpleField "ipolicy"      [t| PartialIPolicy  |]
   , simpleField "diskparams"   [t| GroupDiskParams |]
   , simpleField "networks"     [t| Networks        |]
+  , notSerializeDefaultField [| emptyContainer |] $
+    simpleField "hv_state_static"   [t| HypervisorState |]
+  , notSerializeDefaultField [| emptyContainer |] $
+    simpleField "disk_state_static" [t| DiskState       |]
   ]
   ++ timeStampFields
   ++ uuidFields
@@ -447,7 +467,7 @@ instance TimeStampObject NodeGroup where
   mTimeOf = groupMtime
 
 instance UuidObject NodeGroup where
-  uuidOf = groupUuid
+  uuidOf = UTF8.toString . groupUuid
 
 instance SerialNoObject NodeGroup where
   serialOf = groupSerial
@@ -528,7 +548,7 @@ $(buildObject "FilterRule" "fr" $
   ++ uuidFields)
 
 instance UuidObject FilterRule where
-  uuidOf = frUuid
+  uuidOf = UTF8.toString . frUuid
 
 
 -- | Order in which filter rules are evaluated, according to
@@ -601,18 +621,6 @@ type IAllocatorParams = Container JSValue
 -- | The master candidate client certificate digests
 type CandidateCertificates = Container String
 
--- | Disk state parameters.
---
--- As according to the documentation this option is unused by Ganeti,
--- the content is just a 'JSValue'.
-type DiskState = Container JSValue
-
--- | Hypervisor state parameters.
---
--- As according to the documentation this option is unused by Ganeti,
--- the content is just a 'JSValue'.
-type HypervisorState = Container JSValue
-
 -- * Cluster definitions
 $(buildObject "Cluster" "cluster" $
   [ simpleField "rsahostkeypub"                  [t| String                  |]
@@ -681,7 +689,7 @@ instance TimeStampObject Cluster where
   mTimeOf = clusterMtime
 
 instance UuidObject Cluster where
-  uuidOf = clusterUuid
+  uuidOf = UTF8.toString . clusterUuid
 
 instance SerialNoObject Cluster where
   serialOf = clusterSerial
