@@ -36,12 +36,20 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 module Ganeti.Objects.Lens where
 
+import qualified Data.ByteString as BS
+import qualified Data.ByteString.UTF8 as UTF8
+import Control.Lens (Simple)
+import Control.Lens.Iso (Iso, iso)
 import qualified Data.Set as Set
 import System.Time (ClockTime(..))
 
 import Ganeti.Lens (makeCustomLenses, Lens')
 import Ganeti.Objects
 
+-- | Isomorphism between Strings and bytestrings
+stringL :: Simple Iso BS.ByteString String
+stringL = iso UTF8.toString UTF8.fromString
+
 -- | Class of objects that have timestamps.
 class TimeStampObject a => TimeStampObjectL a where
   mTimeL :: Lens' a ClockTime
@@ -69,7 +77,7 @@ instance TagsObjectL Network where
   tagsL = networkTagsL
 
 instance UuidObjectL Network where
-  uuidL = networkUuidL
+  uuidL = networkUuidL . stringL
 
 instance TimeStampObjectL Network where
   mTimeL = networkMtimeL
@@ -84,7 +92,7 @@ instance TimeStampObjectL Instance where
   mTimeL = instMtimeL
 
 instance UuidObjectL Instance where
-  uuidL = instUuidL
+  uuidL = instUuidL . stringL
 
 instance SerialNoObjectL Instance where
   serialL = instSerialL
@@ -104,7 +112,7 @@ instance TimeStampObjectL Node where
   mTimeL = nodeMtimeL
 
 instance UuidObjectL Node where
-  uuidL = nodeUuidL
+  uuidL = nodeUuidL . stringL
 
 instance SerialNoObjectL Node where
   serialL = nodeSerialL
@@ -118,7 +126,7 @@ instance TimeStampObjectL NodeGroup where
   mTimeL = groupMtimeL
 
 instance UuidObjectL NodeGroup where
-  uuidL = groupUuidL
+  uuidL = groupUuidL . stringL
 
 instance SerialNoObjectL NodeGroup where
   serialL = groupSerialL
@@ -132,7 +140,7 @@ instance TimeStampObjectL Cluster where
   mTimeL = clusterMtimeL
 
 instance UuidObjectL Cluster where
-  uuidL = clusterUuidL
+  uuidL = clusterUuidL . stringL
 
 instance SerialNoObjectL Cluster where
   serialL = clusterSerialL
