@@ -1,4 +1,4 @@
-{-# LANGUAGE Rank2Types #-}
+{-# LANGUAGE RankNTypes, GADTs, StandaloneDeriving #-}
 
 {-| Implementation of the Ganeti Query2 filterning.
 
@@ -55,14 +55,23 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 module Ganeti.Query.Filter
   ( compileFilter
-  , evaluateFilter
+  , evaluateQueryFilter
+  , evaluateFilterM
+  , evaluateFilterJSON
   , requestedNames
   , makeSimpleFilter
+  , Comparator
+  , Comparison(..)
+  , toCompFun
+  , FilterOp(..)
   ) where
 
 import Control.Applicative
-import Control.Monad (liftM)
+import Control.Monad (liftM, mzero)
+import Control.Monad.Trans.Maybe (MaybeT, runMaybeT)
+import Control.Monad.Trans.Class (lift)
 import qualified Data.Map as Map
+import Data.Maybe
 import Data.Traversable (traverse)
 import Text.JSON (JSValue(..), fromJSString)
 import Text.JSON.Pretty (pp_value)
@@ -73,6 +82,7 @@ import Ganeti.Errors
 import Ganeti.Objects
 import Ganeti.Query.Language
 import Ganeti.Query.Types
+import Ganeti.Utils.Monad (anyM, allM)
 import Ganeti.JSON
 
 -- | Compiles a filter based on field names to one based on getters.
@@ -97,28 +107,24 @@ qffField QffTimestamp v =
 
 -- | Wraps a getter, filter pair. If the getter is 'FieldRuntime' but
 -- we don't have a runtime context, we skip the filtering, returning
--- \"pass\". Otherwise, we pass the actual value to the filter.
+-- `Nothing` in the MaybeT. Otherwise, we pass the actual value to the filter.
 wrapGetter :: ConfigData
            -> Maybe b
            -> a
            -> (FieldGetter a b, QffMode)
-           -> (QffMode -> JSValue -> ErrorResult Bool)
-           -> ErrorResult Bool
+           -> (JSValue -> ErrorResult Bool)
+           -> MaybeT ErrorResult Bool
 wrapGetter cfg b a (getter, qff) faction =
   case tryGetter cfg b a getter of
-    Nothing -> Ok True -- runtime missing, accepting the value
-    Just v ->
+    Nothing -> mzero -- runtime missing, signalling that with MaybeT Nothing
+    Just v -> lift $
       case v of
-        ResultEntry RSNormal (Just fval) -> qffField qff fval >>= faction qff
+        ResultEntry RSNormal (Just fval) -> qffField qff fval >>= faction
         ResultEntry RSNormal Nothing ->
           Bad $ ProgrammerError
                 "Internal error: Getter returned RSNormal/Nothing"
         _ -> Ok True -- filter has no data to work, accepting it
 
--- | Wrapper alias over field functions to ignore their first Qff argument.
-ignoreMode :: a -> QffMode -> a
-ignoreMode = const
-
 -- | Helper to evaluate a filter getter (and the value it generates) in
 -- a boolean context.
 trueFilter :: JSValue -> ErrorResult Bool
@@ -136,20 +142,20 @@ type Comparator = (Eq a, Ord a) => a -> a -> Bool
 --
 -- This will handle hostnames correctly, if the mode is set to
 -- 'QffHostname'.
-eqFilter :: FilterValue -> QffMode -> JSValue -> ErrorResult Bool
+eqFilter :: QffMode -> FilterValue -> JSValue -> ErrorResult Bool
 -- send 'QffNormal' queries to 'binOpFilter'
-eqFilter flv QffNormal    jsv = binOpFilter (==) flv jsv
+eqFilter QffNormal    flv jsv = binOpFilter (==) flv jsv
 -- and 'QffTimestamp' as well
-eqFilter flv QffTimestamp jsv = binOpFilter (==) flv jsv
+eqFilter QffTimestamp flv jsv = binOpFilter (==) flv jsv
 -- error out if we set 'QffHostname' on a non-string field
-eqFilter _ QffHostname (JSRational _ _) =
+eqFilter QffHostname _ (JSRational _ _) =
   Bad . ProgrammerError $ "QffHostname field returned a numeric value"
 -- test strings via 'compareNameComponent'
-eqFilter (QuotedString y) QffHostname (JSString x) =
+eqFilter QffHostname (QuotedString y) (JSString x) =
   Ok $ goodLookupResult (fromJSString x `compareNameComponent` y)
 -- send all other combinations (all errors) to 'binOpFilter', which
 -- has good error messages
-eqFilter flv _ jsv = binOpFilter (==) flv jsv
+eqFilter _ flv jsv = binOpFilter (==) flv jsv
 
 -- | Helper to evaluate a filder getter (and the value it generates)
 -- in a boolean context. Note the order of arguments is reversed from
@@ -183,49 +189,97 @@ containsFilter (NumericValue val) lst = do
   lst' <- fromJVal lst
   return $! val `elem` lst'
 
+
+-- | Ways we can compare things in the filter language.
+data Comparison = Eq | Lt | Le | Gt | Ge
+  deriving (Eq, Ord, Show)
+
+
+-- | Turns a comparison into the corresponding Haskell function.
+toCompFun :: Comparison -> Comparator
+toCompFun cmp = case cmp of
+  Eq -> (==)
+  Lt -> (<)
+  Le -> (<=)
+  Gt -> (>)
+  Ge -> (>=)
+
+
+-- | Operations in the leaves of the Ganeti filter language.
+data FilterOp field val where
+  Truth    ::               FilterOp field ()
+  Comp     :: Comparison -> FilterOp field FilterValue
+  Regex    ::               FilterOp field FilterRegex
+  Contains ::               FilterOp field FilterValue
+
+deriving instance Eq (FilterOp field val)
+deriving instance Show (FilterOp field val)
+
+
+-- | Checks if a filter matches.
+--
+-- The leaves of the filter are evaluated against an object using the passed
+-- `opFun`; that is why the object need not be passed in.
+--
+-- The `field` type describes the "accessors" that are used to query
+-- values from the object; those values are to be matched against the
+-- `val` type in the filter leaves.
+--
+-- Useful monads @m@ for this are `ErrorResult` and `Maybe`.
+evaluateFilterM :: (Monad m, Applicative m)
+                  => (forall val .
+                        FilterOp field val -> field -> val -> m Bool)
+                  -> Filter field
+                  -> m Bool
+evaluateFilterM opFun fil = case fil of
+  EmptyFilter              -> return True
+  AndFilter flts           -> allM recurse flts
+  OrFilter flts            -> anyM recurse flts
+  NotFilter flt            -> not <$> recurse flt
+  TrueFilter field         -> opFun Truth     field ()
+  EQFilter field val       -> opFun (Comp Eq) field val
+  LTFilter field val       -> opFun (Comp Lt) field val
+  LEFilter field val       -> opFun (Comp Le) field val
+  GTFilter field val       -> opFun (Comp Gt) field val
+  GEFilter field val       -> opFun (Comp Ge) field val
+  RegexpFilter field re    -> opFun Regex     field re
+  ContainsFilter field val -> opFun Contains  field val
+  where
+    recurse = evaluateFilterM opFun
+
+
 -- | Verifies if a given item passes a filter. The runtime context
 -- might be missing, in which case most of the filters will consider
 -- this as passing the filter.
---
--- Note: we use explicit recursion to reduce unneeded memory use;
--- 'any' and 'all' do not play nice with monadic values, resulting in
--- either too much memory use or in too many thunks being created.
-evaluateFilter :: ConfigData -> Maybe b -> a
-               -> Filter (FieldGetter a b, QffMode)
-               -> ErrorResult Bool
-evaluateFilter _ _  _ EmptyFilter = Ok True
-evaluateFilter c mb a (AndFilter flts) = helper flts
-  where helper [] = Ok True
-        helper (f:fs) = do
-          v <- evaluateFilter c mb a f
-          if v
-            then helper fs
-            else Ok False
-evaluateFilter c mb a (OrFilter flts) = helper flts
-  where helper [] = Ok False
-        helper (f:fs) = do
-          v <- evaluateFilter c mb a f
-          if v
-            then Ok True
-            else helper fs
-evaluateFilter c mb a (NotFilter flt)  =
-  not <$> evaluateFilter c mb a flt
-evaluateFilter c mb a (TrueFilter getter)  =
-  wrapGetter c mb a getter $ ignoreMode trueFilter
-evaluateFilter c mb a (EQFilter getter val) =
-  wrapGetter c mb a getter (eqFilter val)
-evaluateFilter c mb a (LTFilter getter val) =
-  wrapGetter c mb a getter $ ignoreMode (binOpFilter (<) val)
-evaluateFilter c mb a (LEFilter getter val) =
-  wrapGetter c mb a getter $ ignoreMode (binOpFilter (<=) val)
-evaluateFilter c mb a (GTFilter getter val) =
-  wrapGetter c mb a getter $ ignoreMode (binOpFilter (>) val)
-evaluateFilter c mb a (GEFilter getter val) =
-  wrapGetter c mb a getter $ ignoreMode (binOpFilter (>=) val)
-evaluateFilter c mb a (RegexpFilter getter re) =
-  wrapGetter c mb a getter $ ignoreMode (regexpFilter re)
-evaluateFilter c mb a (ContainsFilter getter val) =
-  wrapGetter c mb a getter $ ignoreMode (containsFilter val)
+evaluateQueryFilter :: ConfigData -> Maybe b -> a
+                    -> Filter (FieldGetter a b, QffMode)
+                    -> ErrorResult Bool
+evaluateQueryFilter c mb a =
+  -- `Nothing` in the MaybeT means "missing but needed runtime context".
+  -- Turn those cases into True (let the filter pass).
+  fmap (fromMaybe True) . runMaybeT . evaluateFilterM (\op -> case op of
+    Truth    -> \gQff ()  -> wrap gQff trueFilter
+    -- We're special casing comparison for host names.
+    -- All other comparisons behave as usual.
+    Comp Eq  -> \gQff val -> wrap gQff $ eqFilter (snd gQff) val
+    Comp cmp -> \gQff val -> wrap gQff $ binOpFilter (toCompFun cmp) val
+    Regex    -> \gQff re  -> wrap gQff $ regexpFilter re
+    Contains -> \gQff val -> wrap gQff $ containsFilter val
+  )
+  where
+    wrap = wrapGetter c mb a
+
+
+-- | Evaluates a `Filter` on a JSON object.
+evaluateFilterJSON :: Filter JSValue -> ErrorResult Bool
+evaluateFilterJSON =
+  evaluateFilterM $ \op -> case op of
+    Comp cmp -> let compFun = toCompFun cmp
+                in \json fv -> pure $ json `compFun` showFilterValue fv
+    Truth    -> \field () -> trueFilter field
+    Regex    -> flip regexpFilter
+    Contains -> flip containsFilter
+
 
 -- | Runs a getter with potentially missing runtime context.
 tryGetter :: ConfigData -> Maybe b -> a -> FieldGetter a b -> Maybe ResultEntry
