@@ -43,6 +43,7 @@ module Ganeti.Query.Instance
 
 import Control.Applicative
 import Control.Monad (liftM, (>=>))
+import qualified Data.ByteString.UTF8 as UTF8
 import Data.Either
 import Data.List
 import Data.Maybe
@@ -152,7 +153,7 @@ instanceFields =
      FieldConfig (getSecondaryNodeGroupAttribute groupName), QffNormal)
   , (FieldDefinition "snodes.group.uuid" "SecondaryNodesGroupsUUID" QFTOther
      "Node group UUIDs of secondary nodes",
-     FieldConfig (getSecondaryNodeGroupAttribute groupUuid), QffNormal)
+     FieldConfig (getSecondaryNodeGroupAttribute uuidOf), QffNormal)
   ] ++
 
   -- Instance parameter fields, whole
@@ -227,7 +228,7 @@ instanceFields =
     getIndexedOptionalConfField getInstDisksFromObj diskName, QffNormal)
   , (fieldDefinitionCompleter "disk.uuid/%d" "DiskUUID/%d" QFTText
     "UUID of %s disk",
-    getIndexedConfField getInstDisksFromObj diskUuid, QffNormal)
+    getIndexedConfField getInstDisksFromObj uuidOf, QffNormal)
   ] ++
 
   -- Aggregate nic parameter fields
@@ -247,7 +248,7 @@ instanceFields =
      QffNormal)
   , (FieldDefinition "nic.uuids" "NIC_UUIDs" QFTOther
      (nicAggDescPrefix ++ "UUID"),
-     FieldSimple (rsNormal . map nicUuid . instNics), QffNormal)
+     FieldSimple (rsNormal . map uuidOf . instNics), QffNormal)
   , (FieldDefinition "nic.modes" "NIC_modes" QFTOther
      (nicAggDescPrefix ++ "mode"),
      FieldConfig (\cfg -> rsNormal . map
@@ -288,7 +289,7 @@ instanceFields =
      getIndexedOptionalField instNics nicIp, QffNormal)
   , (fieldDefinitionCompleter "nic.uuid/%d" "NicUUID/%d" QFTText
      ("UUID address" ++ nicDescSuffix),
-     getIndexedField instNics nicUuid, QffNormal)
+     getIndexedField instNics uuidOf, QffNormal)
   , (fieldDefinitionCompleter "nic.mac/%d" "NicMAC/%d" QFTText
      ("MAC address" ++ nicDescSuffix),
      getIndexedField instNics nicMac, QffNormal)
@@ -351,6 +352,7 @@ nicAggDescPrefix = "List containing each network interface's "
 -- | Given a network name id, returns the network's name.
 getNetworkName :: ConfigData -> String -> NonEmptyString
 getNetworkName cfg = networkName . (Map.!) (fromContainer $ configNetworks cfg)
+                     . UTF8.fromString
 
 -- | Gets the bridge of a NIC.
 getNicBridge :: FilledNicParams -> Maybe String
@@ -371,7 +373,8 @@ fillNicParamsFromConfig cfg = fillParams (getDefaultNicParams cfg)
 -- | Retrieves the default network interface parameters.
 getDefaultNicParams :: ConfigData -> FilledNicParams
 getDefaultNicParams cfg =
-  (Map.!) (fromContainer . clusterNicparams . configCluster $ cfg) C.ppDefault
+  (Map.!) (fromContainer . clusterNicparams . configCluster $ cfg)
+    $ UTF8.fromString C.ppDefault
 
 -- | Retrieves the real disk size requirements for all the disks of the
 -- instance. This includes the metadata etc. and is different from the values
@@ -411,7 +414,7 @@ getDiskNames cfg =
 -- | Get a list of disk UUIDs for an instance
 getDiskUuids :: ConfigData -> Instance -> ResultEntry
 getDiskUuids cfg =
-  rsErrorNoData . liftA (map diskUuid) . getInstDisksFromObj cfg
+  rsErrorNoData . liftA (map uuidOf) . getInstDisksFromObj cfg
 
 -- | Creates a functions which produces a FieldConfig 'FieldGetter' when fed
 -- an index. Works for fields that may not return a value, expressed through
@@ -582,7 +585,7 @@ getPrimaryNodeGroupName cfg inst =
 -- | Get primary node group uuid
 getPrimaryNodeGroupUuid :: ConfigData -> Instance -> ResultEntry
 getPrimaryNodeGroupUuid cfg inst =
-  rsErrorNoData $ groupUuid <$> getPrimaryNodeGroup cfg inst
+  rsErrorNoData $ uuidOf <$> getPrimaryNodeGroup cfg inst
 
 -- | Get secondary nodes - the configuration objects themselves
 getSecondaryNodes :: ConfigData -> Instance -> ErrorResult [Node]
@@ -634,7 +637,7 @@ beParamGetter field config inst =
 hvParamGetter :: String -- ^ The field we're building the getter for
               -> ConfigData -> Instance -> ResultEntry
 hvParamGetter field cfg inst =
-  rsMaybeUnavail . Map.lookup field . fromContainer $
+  rsMaybeUnavail . Map.lookup (UTF8.fromString field) . fromContainer $
     getFilledInstHvParams (C.toList C.hvcGlobals) cfg inst
 
 -- * Live fields functionality
@@ -736,8 +739,9 @@ liveInstanceStatus cfg (instInfo, foundOnPrimary) inst
         allowDown =
           userShutdownEnabled cfg &&
           (instHypervisor inst /= Just Kvm ||
-           (Map.member C.hvKvmUserShutdown hvparams &&
-            hvparams Map.! C.hvKvmUserShutdown == J.JSBool True))
+           (Map.member (UTF8.fromString C.hvKvmUserShutdown) hvparams &&
+            hvparams Map.! UTF8.fromString C.hvKvmUserShutdown
+              == J.JSBool True))
 
 -- | Determines the status of a dead instance.
 deadInstanceStatus :: ConfigData -> Instance -> InstanceStatus
@@ -852,7 +856,7 @@ extractLiveInfo :: [(Node, ERpcError RpcResultAllInstancesInfo)]
                 -> Instance
                 -> Runtime
 extractLiveInfo nodeResultList nodeConsoleList inst =
-  let uuidConvert     = map (\(x, y) -> (nodeUuid x, y))
+  let uuidConvert     = map (\(x, y) -> (uuidOf x, y))
       uuidResultList  = uuidConvert nodeResultList
       uuidConsoleList = uuidConvert nodeConsoleList
   in case getInstanceInfo uuidResultList inst of
