@@ -50,6 +50,7 @@ import Control.Monad.Error (MonadError)
 import Control.Monad.IO.Class
 import Control.Monad.Trans (lift)
 import Control.Monad.Trans.Maybe
+import qualified Data.ByteString.UTF8 as UTF8
 import qualified Data.Set as Set (toList)
 import Data.IORef
 import Data.List (intersperse)
@@ -342,10 +343,11 @@ handleCall _ status _ (ReplaceFilter mUuid priority predicates action
                                 , frPredicates = predicates
                                 , frAction = action
                                 , frReasonTrail = reason ++ [luxidReason]
-                                , frUuid = uuid
+                                , frUuid = UTF8.fromString uuid
                                 }
           writeConfig cid
-            . (configFiltersL . alterContainerL uuid .~ Just rule)
+            . (configFiltersL . alterContainerL (UTF8.fromString uuid)
+                 .~ Just rule)
             $ lockedCfg
 
     -- Return UUID of added/replaced filter.
@@ -355,14 +357,14 @@ handleCall _ status cfg (DeleteFilter uuid) = runResultT $ do
   -- Check if filter exists.
   _ <- lookupContainer
     (failError $ "Filter rule with UUID " ++ uuid ++ " does not exist")
-    uuid
+    (UTF8.fromString uuid)
     (configFilters cfg)
 
   -- Ask WConfd to change the config for us.
   cid <- liftIO $ makeLuxidClientId status
   withLockedWconfdConfig cid $ \lockedCfg ->
     writeConfig cid
-      . (configFiltersL . alterContainerL uuid .~ Nothing)
+      . (configFiltersL . alterContainerL (UTF8.fromString uuid) .~ Nothing)
       $ lockedCfg
 
   return JSNull
