@@ -61,6 +61,7 @@ module Ganeti.THH ( declareSADT
                   , andRestArguments
                   , withDoc
                   , defaultField
+                  , notSerializeDefaultField
                   , presentInForthcoming
                   , optionalField
                   , optionalNullSerField
@@ -183,6 +184,9 @@ data Field = Field { fieldName        :: String
                      -- ^ a list of extra keys added by 'fieldShow'
                    , fieldDefault     :: Maybe (Q Exp)
                      -- ^ an optional default value of type @t@
+                   , fieldSerializeDefault :: Bool
+                     -- ^ whether not presented default value will be
+                     -- serialized
                    , fieldConstr      :: Maybe String
                    , fieldIsOptional  :: OptionalType
                      -- ^ determines if a field is optional, and if yes,
@@ -200,6 +204,7 @@ simpleField fname ftype =
         , fieldShow        = Nothing
         , fieldExtraKeys   = []
         , fieldDefault     = Nothing
+        , fieldSerializeDefault = True
         , fieldConstr      = Nothing
         , fieldIsOptional  = NotOptional
         , fieldDoc         = ""
@@ -215,6 +220,7 @@ andRestArguments fname =
         , fieldShow        = Nothing
         , fieldExtraKeys   = []
         , fieldDefault     = Nothing
+        , fieldSerializeDefault = True
         , fieldConstr      = Nothing
         , fieldIsOptional  = AndRestArguments
         , fieldDoc         = ""
@@ -234,6 +240,13 @@ renameField constrName field = field { fieldConstr = Just constrName }
 defaultField :: Q Exp -> Field -> Field
 defaultField defval field = field { fieldDefault = Just defval }
 
+-- | A defaultField which will be serialized only if it's value differs from
+-- a default value.
+notSerializeDefaultField :: Q Exp -> Field -> Field
+notSerializeDefaultField defval field =
+  field { fieldDefault = Just defval
+        , fieldSerializeDefault = False }
+
 -- | Mark a field as present in the forthcoming variant.
 presentInForthcoming :: Field -> Field
 presentInForthcoming field = field { fieldPresentInForthcoming = True }
@@ -311,7 +324,7 @@ parseFn :: Field   -- ^ The field definition
         -> Q Exp   -- ^ The resulting function that parses a JSON message
 parseFn field o =
   let fnType = [t| JSON.JSValue -> JSON.Result $(fieldType field) |]
-      expr = maybe [| readJSONWithDesc $(stringE $ fieldName field) False |]
+      expr = maybe [| readJSONWithDesc $(stringE $ fieldName field) |]
                    (`appE` o) (fieldRead field)
   in sigE expr fnType
 
@@ -567,7 +580,7 @@ genReadJSON :: String -> Q Dec
 genReadJSON name = do
   let s = mkName "s"
   body <- [| $(varE (fromRawName name)) =<<
-             readJSONWithDesc $(stringE name) True $(varE s) |]
+             readJSONWithDesc $(stringE name) $(varE s) |]
   return $ FunD 'JSON.readJSON [Clause [VarP s] (NormalB body) []]
 
 -- | Generates a JSON instance for a given type.
@@ -1208,7 +1221,14 @@ saveObjectField fvar field = do
                                    Nothing -> [( $nameE, JSON.JSNull )]
                                    Just v  -> $(formatCode [| v |])
                               |]
-    NotOptional ->            formatCode fvarE
+    NotOptional -> case (fieldDefault field, fieldSerializeDefault field) of
+                     (Just v, False) -> [| if $v /= $fvarE
+                                             then $(formatCode fvarE)
+                                             else [] |]
+                     -- If a default value exists and we shouldn't serialize
+                     -- default fields - serialize only if the value differs
+                     -- from the default one.
+                     _ -> formatCode fvarE
     AndRestArguments -> [| M.toList $(varE fvar) |]
   where nameE = stringE (fieldName field)
         fvarE = varE fvar
@@ -1279,7 +1299,7 @@ objectReadJSON :: String -> Q Dec
 objectReadJSON name = do
   let s = mkName "s"
   body <- [| $(varE . mkName $ "load" ++ name) =<<
-             readJSONWithDesc $(stringE name) False $(varE s) |]
+             readJSONWithDesc $(stringE name) $(varE s) |]
   return $ FunD 'JSON.readJSON [Clause [VarP s] (NormalB body) []]
 
 -- * Inheritable parameter tables implementation
