@@ -1,4 +1,4 @@
-{-# LANGUAGE TemplateHaskell #-}
+{-# LANGUAGE TemplateHaskell, DeriveFunctor #-}
 
 {-| Some common Ganeti types.
 
@@ -173,12 +173,23 @@ module Ganeti.Types
   , hotplugActionToRaw
   , Private(..)
   , showPrivateJSObject
+  , HvParams
+  , OsParams
+  , OsParamsPrivate
+  , TimeStampObject(..)
+  , UuidObject(..)
+  , ForthcomingObject(..)
+  , SerialNoObject(..)
+  , TagsObject(..)
   ) where
 
+import Control.Applicative
 import Control.Monad (liftM)
 import qualified Text.JSON as JSON
 import Text.JSON (JSON, readJSON, showJSON)
 import Data.Ratio (numerator, denominator)
+import qualified Data.Set as Set
+import System.Time (ClockTime)
 
 import qualified Ganeti.ConstantUtils as ConstantUtils
 import Ganeti.JSON
@@ -189,7 +200,7 @@ import Ganeti.Utils
 
 -- | Type that holds a non-negative value.
 newtype NonNegative a = NonNegative { fromNonNegative :: a }
-  deriving (Show, Eq)
+  deriving (Show, Eq, Ord)
 
 -- | Smart constructor for 'NonNegative'.
 mkNonNegative :: (Monad m, Num a, Ord a, Show a) => a -> m (NonNegative a)
@@ -203,7 +214,7 @@ instance (JSON.JSON a, Num a, Ord a, Show a) => JSON.JSON (NonNegative a) where
 
 -- | Type that holds a positive value.
 newtype Positive a = Positive { fromPositive :: a }
-  deriving (Show, Eq)
+  deriving (Show, Eq, Ord)
 
 -- | Smart constructor for 'Positive'.
 mkPositive :: (Monad m, Num a, Ord a, Show a) => a -> m (Positive a)
@@ -217,7 +228,7 @@ instance (JSON.JSON a, Num a, Ord a, Show a) => JSON.JSON (Positive a) where
 
 -- | Type that holds a negative value.
 newtype Negative a = Negative { fromNegative :: a }
-  deriving (Show, Eq)
+  deriving (Show, Eq, Ord)
 
 -- | Smart constructor for 'Negative'.
 mkNegative :: (Monad m, Num a, Ord a, Show a) => a -> m (Negative a)
@@ -231,17 +242,13 @@ instance (JSON.JSON a, Num a, Ord a, Show a) => JSON.JSON (Negative a) where
 
 -- | Type that holds a non-null list.
 newtype NonEmpty a = NonEmpty { fromNonEmpty :: [a] }
-  deriving (Show, Eq)
+  deriving (Show, Eq, Ord)
 
 -- | Smart constructor for 'NonEmpty'.
 mkNonEmpty :: (Monad m) => [a] -> m (NonEmpty a)
 mkNonEmpty [] = fail "Received empty value for non-empty list"
 mkNonEmpty xs = return (NonEmpty xs)
 
-instance (Eq a, Ord a) => Ord (NonEmpty a) where
-  NonEmpty { fromNonEmpty = x1 } `compare` NonEmpty { fromNonEmpty = x2 } =
-    x1 `compare` x2
-
 instance (JSON.JSON a) => JSON.JSON (NonEmpty a) where
   showJSON = JSON.showJSON . fromNonEmpty
   readJSON v = JSON.readJSON v >>= mkNonEmpty
@@ -252,7 +259,7 @@ type NonEmptyString = NonEmpty Char
 type QueryResultCode = Int
 
 newtype IPv4Address = IPv4Address { fromIPv4Address :: String }
-  deriving (Show, Eq)
+  deriving (Show, Eq, Ord)
 
 -- FIXME: this should check that 'address' is a valid ip
 mkIPv4Address :: Monad m => String -> m IPv4Address
@@ -264,7 +271,7 @@ instance JSON.JSON IPv4Address where
   readJSON v = JSON.readJSON v >>= mkIPv4Address
 
 newtype IPv4Network = IPv4Network { fromIPv4Network :: String }
-  deriving (Show, Eq)
+  deriving (Show, Eq, Ord)
 
 -- FIXME: this should check that 'address' is a valid ip
 mkIPv4Network :: Monad m => String -> m IPv4Network
@@ -276,7 +283,7 @@ instance JSON.JSON IPv4Network where
   readJSON v = JSON.readJSON v >>= mkIPv4Network
 
 newtype IPv6Address = IPv6Address { fromIPv6Address :: String }
-  deriving (Show, Eq)
+  deriving (Show, Eq, Ord)
 
 -- FIXME: this should check that 'address' is a valid ip
 mkIPv6Address :: Monad m => String -> m IPv6Address
@@ -288,7 +295,7 @@ instance JSON.JSON IPv6Address where
   readJSON v = JSON.readJSON v >>= mkIPv6Address
 
 newtype IPv6Network = IPv6Network { fromIPv6Network :: String }
-  deriving (Show, Eq)
+  deriving (Show, Eq, Ord)
 
 -- FIXME: this should check that 'address' is a valid ip
 mkIPv6Network :: Monad m => String -> m IPv6Network
@@ -301,7 +308,9 @@ instance JSON.JSON IPv6Network where
 
 -- * Ganeti types
 
--- | Instance disk template type.
+-- | Instance disk template type. The disk template is a name for the
+-- constructor of the disk configuration 'DiskLogicalId' used for
+-- serialization, configuration values, etc.
 $(THH.declareLADT ''String "DiskTemplate"
        [ ("DTDiskless",   "diskless")
        , ("DTFile",       "file")
@@ -422,7 +431,9 @@ $(THH.makeJSONInstance ''CVErrorCode)
 -- | Dynamic device modification, just add/remove version.
 $(THH.declareLADT ''String "DdmSimple"
      [ ("DdmSimpleAdd",    "add")
+     , ("DdmSimpleAttach", "attach")
      , ("DdmSimpleRemove", "remove")
+     , ("DdmSimpleDetach", "detach")
      ])
 $(THH.makeJSONInstance ''DdmSimple)
 
@@ -431,7 +442,9 @@ $(THH.makeJSONInstance ''DdmSimple)
 -- TODO: DDM_SWAP, DDM_MOVE?
 $(THH.declareLADT ''String "DdmFull"
      [ ("DdmFullAdd",    "add")
+     , ("DdmFullAttach", "attach")
      , ("DdmFullRemove", "remove")
+     , ("DdmFullDetach", "detach")
      , ("DdmFullModify", "modify")
      ])
 $(THH.makeJSONInstance ''DdmFull)
@@ -543,10 +556,7 @@ showSUSimple st sk = show (storageTypeToRaw st, sk, []::[String])
 showSULvm :: StorageType -> StorageKey -> SPExclusiveStorage -> String
 showSULvm st sk es = show (storageTypeToRaw st, sk, [es])
 
--- | Mapping from disk templates to storage types
--- FIXME: This is semantically the same as the constant
--- C.diskTemplatesStorageType, remove this when python constants
--- are generated from haskell constants
+-- | Mapping from disk templates to storage types.
 diskTemplateToStorageType :: DiskTemplate -> StorageType
 diskTemplateToStorageType DTExt = StorageExt
 diskTemplateToStorageType DTFile = StorageFile
@@ -695,7 +705,7 @@ type RelativeJobId = Negative Int
 -- | Job ID dependency.
 data JobIdDep = JobDepRelative RelativeJobId
               | JobDepAbsolute JobId
-                deriving (Show, Eq)
+                deriving (Show, Eq, Ord)
 
 instance JSON.JSON JobIdDep where
   showJSON (JobDepRelative i) = showJSON i
@@ -714,7 +724,7 @@ absoluteJobIdDep (JobDepRelative rjid) jid =
 
 -- | Job Dependency type.
 data JobDependency = JobDependency JobIdDep [FinalizedJobStatus]
-                     deriving (Show, Eq)
+                     deriving (Show, Eq, Ord)
 
 instance JSON JobDependency where
   showJSON (JobDependency dep status) = showJSON (dep, status)
@@ -773,7 +783,8 @@ $(THH.declareLADT ''String "ELogType"
   ])
 $(THH.makeJSONInstance ''ELogType)
 
--- | Type of one element of a reason trail.
+-- | Type of one element of a reason trail, of form
+-- @(source, reason, timestamp)@.
 type ReasonElem = (String, String, Integer)
 
 -- | Type representing a reason trail.
@@ -868,14 +879,16 @@ $(THH.makeJSONInstance ''DiskAccessMode)
 -- Python code depends on:
 --   DiskStatusOk < DiskStatusUnknown < DiskStatusFaulty
 $(THH.declareILADT "LocalDiskStatus"
-  [ ("DiskStatusFaulty",  3)
-  , ("DiskStatusOk",      1)
-  , ("DiskStatusUnknown", 2)
+  [ ("DiskStatusOk",      1)
+  , ("DiskStatusSync",    2)
+  , ("DiskStatusUnknown", 3)
+  , ("DiskStatusFaulty",  4)
   ])
 
 localDiskStatusName :: LocalDiskStatus -> String
 localDiskStatusName DiskStatusFaulty = "faulty"
 localDiskStatusName DiskStatusOk = "ok"
+localDiskStatusName DiskStatusSync = "syncing"
 localDiskStatusName DiskStatusUnknown = "unknown"
 
 -- | Replace disks type.
@@ -922,7 +935,7 @@ $(THH.makeJSONInstance ''HotplugTarget)
 -- | A container for values that should be happy to be manipulated yet
 -- refuses to be shown unless explicitly requested.
 newtype Private a = Private { getPrivate :: a }
-  deriving Eq
+  deriving (Eq, Ord, Functor)
 
 instance (Show a, JSON.JSON a) => JSON.JSON (Private a) where
   readJSON = liftM Private . JSON.readJSON
@@ -938,8 +951,9 @@ instance Show a => Show (Private a) where
 instance THH.PyValue a => THH.PyValue (Private a) where
   showValue (Private x) = "Private(" ++ THH.showValue x ++ ")"
 
-instance Functor Private where
-  fmap f (Private x) = Private $ f x
+instance Applicative Private where
+  pure = Private
+  Private f <*> Private x = Private (f x)
 
 instance Monad Private where
   (Private x) >>= f = f x
@@ -949,3 +963,37 @@ showPrivateJSObject :: (JSON.JSON a) =>
                        [(String, a)] -> JSON.JSObject (Private JSON.JSValue)
 showPrivateJSObject value = JSON.toJSObject $ map f value
   where f (k, v) = (k, Private $ JSON.showJSON v)
+
+
+-- | The hypervisor parameter type. This is currently a simple map,
+-- without type checking on key/value pairs.
+type HvParams = Container JSON.JSValue
+
+-- | The OS parameters type. This is, and will remain, a string
+-- container, since the keys are dynamically declared by the OSes, and
+-- the values are always strings.
+type OsParams = Container String
+type OsParamsPrivate = Container (Private String)
+
+
+-- | Class of objects that have timestamps.
+class TimeStampObject a where
+  cTimeOf :: a -> ClockTime
+  mTimeOf :: a -> ClockTime
+
+-- | Class of objects that have an UUID.
+class UuidObject a where
+  uuidOf :: a -> String
+
+-- | Class of objects that can be forthcoming.
+class ForthcomingObject a where
+  isForthcoming :: a -> Bool
+
+-- | Class of object that have a serial number.
+class SerialNoObject a where
+  serialOf :: a -> Int
+
+-- | Class of objects that have tags.
+class TagsObject a where
+  tagsOf :: a -> Set.Set String
+
