@@ -45,6 +45,7 @@ module Ganeti.Utils
   , commaJoin
   , ensureQuoted
   , tryRead
+  , readMaybe
   , formatTable
   , printTable
   , parseUnit
@@ -59,11 +60,14 @@ module Ganeti.Utils
   , logWarningIfBad
   , rStripSpace
   , newUUID
+  , isUUID
   , getCurrentTime
   , getCurrentTimeUSec
   , clockTimeToString
   , clockTimeToCTime
+  , clockTimeToUSec
   , cTimeToClockTime
+  , diffClockTimes
   , chompPrefix
   , warn
   , wrap
@@ -90,12 +94,17 @@ module Ganeti.Utils
   , safeRenameFile
   , FilePermissions(..)
   , ensurePermissions
+  , ordNub
+  , isSubsequenceOf
   ) where
 
+import Control.Applicative
 import Control.Concurrent
 import Control.Exception (try, bracket)
 import Control.Monad
 import Control.Monad.Error
+import qualified Data.Attoparsec.ByteString as A
+import qualified Data.ByteString.UTF8 as UTF8
 import Data.Char (toUpper, isAlphaNum, isDigit, isSpace)
 import qualified Data.Either as E
 import Data.Function (on)
@@ -122,7 +131,8 @@ import System.IO
 import System.Exit
 import System.Posix.Files
 import System.Posix.IO
-import System.Time
+import System.Time (ClockTime(..), getClockTime, TimeDiff(..))
+import qualified System.Time as STime
 
 -- * Debug functions
 
@@ -227,6 +237,15 @@ parseChoices name s _ = fail $ name ++ ": cannot parse string '" ++ s ++ "'"
 tryRead :: (Monad m, Read a) => String -> String -> m a
 tryRead name s = parseChoices name s $ reads s
 
+-- | Parse a string using the 'Read' instance.
+-- Succeeds if there is exactly one valid result.
+--
+-- /Backport from Text.Read introduced in base-4.6.0.0/
+readMaybe :: Read a => String -> Maybe a
+readMaybe s = case reads s of
+  [(a, "")] -> Just a
+  _         -> Nothing
+
 -- | Format a table of strings to maintain consistent length.
 formatTable :: [[String]] -> [Bool] -> [[String]]
 formatTable vals numpos =
@@ -403,6 +422,24 @@ newUUID = do
   contents <- readFile ConstantUtils.randomUuidFile
   return $! rStripSpace $ take 128 contents
 
+-- | Parser that doesn't fail on a valid UUIDs (same as
+-- "Ganeti.Constants.uuidRegex").
+uuidCheckParser :: A.Parser ()
+uuidCheckParser = do
+  -- Not using Attoparsec.Char8 because "all attempts to use characters
+  -- above code point U+00FF will give wrong answers" and we don't
+  -- want such things to be accepted as UUIDs.
+  let lowerHex = A.satisfy (\c -> (48 <= c && c <= 57) ||  -- 0-9
+                                  (97 <= c && c <= 102))   -- a-f
+      hx n = A.count n lowerHex
+      d = A.word8 45  -- '-'
+  void $ hx 8 >> d >> hx 4 >> d >> hx 4 >> d >> hx 4 >> d >> hx 12
+
+-- | Checks if the string is a valid UUID as in "Ganeti.Constants.uuidRegex".
+isUUID :: String -> Bool
+isUUID =
+  isRight . A.parseOnly (uuidCheckParser <* A.endOfInput) . UTF8.fromString
+
 -- | Returns the current time as an 'Integer' representing the number
 -- of seconds from the Unix epoch.
 getCurrentTime :: IO Integer
@@ -413,11 +450,7 @@ getCurrentTime = do
 -- | Returns the current time as an 'Integer' representing the number
 -- of microseconds from the Unix epoch (hence the need for 'Integer').
 getCurrentTimeUSec :: IO Integer
-getCurrentTimeUSec = do
-  TOD ctime pico <- getClockTime
-  -- pico: 10^-12, micro: 10^-6, so we have to shift seconds left and
-  -- picoseconds right
-  return $ ctime * 1000000 + pico `div` 1000000
+getCurrentTimeUSec = liftM clockTimeToUSec getClockTime
 
 -- | Convert a ClockTime into a (seconds-only) timestamp.
 clockTimeToString :: ClockTime -> String
@@ -427,10 +460,28 @@ clockTimeToString (TOD t _) = show t
 clockTimeToCTime :: ClockTime -> EpochTime
 clockTimeToCTime (TOD secs _) = fromInteger secs
 
+-- | Convert a ClockTime the number of microseconds since the epoch.
+clockTimeToUSec :: ClockTime -> Integer
+clockTimeToUSec (TOD ctime pico) =
+  -- pico: 10^-12, micro: 10^-6, so we have to shift seconds left and
+  -- picoseconds right
+  ctime * 1000000 + pico `div` 1000000
+
 -- | Convert a ClockTime into a (seconds-only) 'EpochTime' (AKA @time_t@).
 cTimeToClockTime :: EpochTime -> ClockTime
 cTimeToClockTime (CTime timet) = TOD (toInteger timet) 0
 
+-- | A version of `diffClockTimes` that works around ghc bug #2519.
+diffClockTimes :: ClockTime -> ClockTime -> TimeDiff
+diffClockTimes t1 t2 =
+  let delta = STime.diffClockTimes t1 t2
+      secondInPicoseconds = 1000000000000
+  in if tdPicosec delta < 0
+       then delta { tdSec = tdSec delta - 1
+                  , tdPicosec = tdPicosec delta + secondInPicoseconds
+                  }
+       else delta
+
 {-| Strip a prefix from a string, allowing the last character of the prefix
 (which is assumed to be a separator) to be absent from the string if the string
 terminates there.
@@ -751,3 +802,19 @@ safeRenameFile perms from to = do
         _ <- ensurePermissions dir perms
         renameFile from to
       return $ either (Bad . show) Ok (result :: Either IOError ())
+
+-- | Removes duplicates, preserving order.
+ordNub :: (Ord a) => [a] -> [a]
+ordNub =
+  let go _ [] = []
+      go s (x:xs) = if x `S.member` s
+        then go s xs
+        else x : go (S.insert x s) xs
+  in go S.empty
+
+-- | `isSubsequenceOf a b`: Checks if a is a subsequence of b.
+isSubsequenceOf :: (Eq a) => [a] -> [a] -> Bool
+isSubsequenceOf []    _                    = True
+isSubsequenceOf _     []                   = False
+isSubsequenceOf a@(x:a') (y:b) | x == y    = isSubsequenceOf a' b
+                               | otherwise = isSubsequenceOf a b
