@@ -1,6 +1,6 @@
 {-# LANGUAGE MultiParamTypeClasses, TypeFamilies,
+             GeneralizedNewtypeDeriving,
              TemplateHaskell, CPP, UndecidableInstances #-}
-
 {-| All RPC calls are run within this monad.
 
 It encapsulates:
@@ -53,9 +53,11 @@ module Ganeti.WConfd.Monad
   , WConfdMonad
   , daemonHandle
   , modifyConfigState
+  , modifyConfigStateWithImmediate
   , forceConfigStateDistribution
   , readConfigState
   , modifyConfigDataErr_
+  , modifyConfigWithLock
   , modifyLockWaiting
   , modifyLockWaiting_
   , readLockWaiting
@@ -65,16 +67,10 @@ module Ganeti.WConfd.Monad
   , readTempResState
   ) where
 
--- The following macro is just a temporary solution for 2.12 and 2.13.
--- Since 2.14 cabal creates proper macros for all dependencies.
-#define MIN_VERSION_monad_control(maj,min,rev) \
-  (((maj)<MONAD_CONTROL_MAJOR)|| \
-   (((maj)==MONAD_CONTROL_MAJOR)&&((min)<=MONAD_CONTROL_MINOR))|| \
-   (((maj)==MONAD_CONTROL_MAJOR)&&((min)==MONAD_CONTROL_MINOR)&& \
-    ((rev)<=MONAD_CONTROL_REV)))
-
 import Control.Applicative
 import Control.Arrow ((&&&), second)
+import Control.Concurrent (forkIO, myThreadId)
+import Control.Exception.Lifted (bracket)
 import Control.Monad
 import Control.Monad.Base
 import Control.Monad.Error
@@ -86,16 +82,17 @@ import Data.IORef.Lifted
 import Data.Monoid (Any(..))
 import qualified Data.Set as S
 import Data.Tuple (swap)
-import System.Time (getClockTime)
+import System.Posix.Process (getProcessID)
+import System.Time (getClockTime, ClockTime)
 import qualified Text.JSON as J
 
 import Ganeti.BasicTypes
 import Ganeti.Errors
 import Ganeti.JQueue (notifyJob)
 import Ganeti.Lens
-import Ganeti.Locking.Allocation (LockAllocation)
+import qualified Ganeti.Locking.Allocation as LA
 import Ganeti.Locking.Locks
-import Ganeti.Locking.Waiting (getAllocation)
+import qualified Ganeti.Locking.Waiting as LW
 import Ganeti.Logging
 import Ganeti.Logging.WriterLog
 import Ganeti.Objects (ConfigData)
@@ -153,8 +150,7 @@ mkDaemonHandle :: FilePath
                -> ResultG DaemonHandle
 mkDaemonHandle cpath cstat lstat trstat
                saveWorkerFn distMCsWorkerFn distSSConfWorkerFn
-               saveLockWorkerFn saveTempResWorkerFn
-               livelock = do
+               saveLockWorkerFn saveTempResWorkerFn livelock = do
   ds <- newIORef $ DaemonState cstat lstat trstat
   let readConfigIO = dsConfigState `liftM` readIORef ds :: IO ConfigState
 
@@ -179,23 +175,7 @@ type WConfdMonadIntType = ReaderT DaemonHandle IO
 -- | The internal part of the monad without error handling.
 newtype WConfdMonadInt a = WConfdMonadInt
   { getWConfdMonadInt :: WConfdMonadIntType a }
-
-instance Functor WConfdMonadInt where
-  fmap f = WConfdMonadInt . fmap f . getWConfdMonadInt
-
-instance Applicative WConfdMonadInt where
-  pure = WConfdMonadInt . pure
-  WConfdMonadInt f <*> WConfdMonadInt k = WConfdMonadInt $ f <*> k
-
-instance Monad WConfdMonadInt where
-  return = WConfdMonadInt . return
-  (WConfdMonadInt k) >>= f = WConfdMonadInt $ k >>= getWConfdMonadInt . f
-
-instance MonadIO WConfdMonadInt where
-  liftIO = WConfdMonadInt . liftIO
-
-instance MonadBase IO WConfdMonadInt where
-  liftBase = WConfdMonadInt . liftBase
+  deriving (Functor, Applicative, Monad, MonadIO, MonadBase IO, MonadLog)
 
 instance MonadBaseControl IO WConfdMonadInt where
 #if MIN_VERSION_monad_control(1,0,0)
@@ -212,9 +192,6 @@ instance MonadBaseControl IO WConfdMonadInt where
   restoreM = WConfdMonadInt . restoreM . runStMWConfdMonadInt
 #endif
 
-instance MonadLog WConfdMonadInt where
-  logAt p = WConfdMonadInt . logAt p
-
 -- | Runs the internal part of the WConfdMonad monad on a given daemon
 -- handle.
 runWConfdMonadInt :: WConfdMonadInt a -> DaemonHandle -> IO a
@@ -237,41 +214,62 @@ readConfigState :: WConfdMonad ConfigState
 readConfigState = liftM dsConfigState . readIORef . dhDaemonState
                   =<< daemonHandle
 
+-- | From a result of a configuration change, determine if the
+-- configuration was changed and if full distribution is needed.
+-- If so, also bump the serial number.
+unpackConfigResult :: ClockTime -> ConfigState
+                      -> (a, ConfigState) -> ((a, Bool, Bool), ConfigState)
+unpackConfigResult now cs (r, cs')
+                     | cs /= cs' = ( (r, True, needsFullDist cs cs')
+                                   , over csConfigDataL (bumpSerial now) cs'
+                                   )
+                     | otherwise = ((r, False, False), cs')
+
 -- | Atomically modifies the configuration state in the WConfdMonad
--- with a computation that can possibly fail.
-modifyConfigStateErr
+-- with a computation that can possibly fail; immediately afterwards,
+-- while config write is still going on, do the followup action. Return
+-- only after replication is finished.
+modifyConfigStateErrWithImmediate
   :: (TempResState -> ConfigState -> AtomicModifyMonad (a, ConfigState))
+  -> WConfdMonad ()
   -> WConfdMonad a
-modifyConfigStateErr f = do
+modifyConfigStateErrWithImmediate f immediateFollowup = do
   dh <- daemonHandle
   now <- liftIO getClockTime
 
-  -- If the configuration is modified, we also bump its serial number.
-  -- In order to determine if we need to save, we report if it's modified
-  -- as well as if it needs to be distributed synchronously.
-  let unpackResult cs (r, cs')
-                    | cs /= cs' = ( (r, True, needsFullDist cs cs')
-                                  , over csConfigDataL (bumpSerial now) cs' )
-                    | otherwise = ((r, False, False), cs')
   let modCS ds@(DaemonState { dsTempRes = tr }) =
-        mapMOf2 dsConfigStateL (\cs -> liftM (unpackResult cs) (f tr cs)) ds
+        mapMOf2
+          dsConfigStateL (\cs -> liftM (unpackConfigResult now cs) (f tr cs)) ds
   (r, modified, distSync) <- atomicModifyIORefErrLog (dhDaemonState dh)
                                                      (liftM swap . modCS)
-  when modified $ do
-    if distSync
+  if modified
+    then if distSync
       then do
         logDebug $ "Triggering config write" ++
                    " together with full synchronous distribution"
-        liftBase . triggerAndWait (Any True) . dhSaveConfigWorker $ dh
+        res <- liftBase . triggerWithResult (Any True) $ dhSaveConfigWorker dh
+        immediateFollowup
+        wait res
         logDebug "Config write and distribution finished"
       else do
         -- trigger the config. saving worker and wait for it
         logDebug $ "Triggering config write" ++
                    " and asynchronous distribution"
-        liftBase . triggerAndWait (Any False) . dhSaveConfigWorker $ dh
-    return ()
+        res <- liftBase . triggerWithResult (Any False) $ dhSaveConfigWorker dh
+        immediateFollowup
+        wait res
+        logDebug "Config writer finished with local task"
+    else
+      immediateFollowup
   return r
 
+-- | Atomically modifies the configuration state in the WConfdMonad
+-- with a computation that can possibly fail.
+modifyConfigStateErr
+  :: (TempResState -> ConfigState -> AtomicModifyMonad (a, ConfigState))
+  -> WConfdMonad a
+modifyConfigStateErr = flip modifyConfigStateErrWithImmediate (return ())
+
 -- | Atomically modifies the configuration state in the WConfdMonad
 -- with a computation that can possibly fail.
 modifyConfigStateErr_
@@ -283,6 +281,15 @@ modifyConfigStateErr_ f = modifyConfigStateErr ((liftM ((,) ()) .) . f)
 modifyConfigState :: (ConfigState -> (a, ConfigState)) -> WConfdMonad a
 modifyConfigState f = modifyConfigStateErr ((return .) . const f)
 
+-- | Atomically modifies the configuration state in WConfdMonad; immediately
+-- afterwards (while the config write-out is not necessarily finished) do
+-- another acation.
+modifyConfigStateWithImmediate :: (ConfigState -> (a, ConfigState))
+                                  -> WConfdMonad ()
+                                  -> WConfdMonad a
+modifyConfigStateWithImmediate f =
+  modifyConfigStateErrWithImmediate ((return .) . const f)
+
 -- | Force the distribution of configuration without actually modifying it.
 --
 -- We need a separate call for this operation, because 'modifyConfigState' only
@@ -366,5 +373,56 @@ readLockWaiting = liftM dsLockWaiting
 
 
 -- | Read the underlying lock allocation.
-readLockAllocation :: WConfdMonad (LockAllocation GanetiLocks ClientId)
-readLockAllocation = liftM getAllocation readLockWaiting
+readLockAllocation :: WConfdMonad (LA.LockAllocation GanetiLocks ClientId)
+readLockAllocation = liftM LW.getAllocation readLockWaiting
+
+-- | Modify the configuration while temporarily acquiring
+-- the configuration lock. If the configuration lock is held by
+-- someone else, nothing is changed and Nothing is returned.
+modifyConfigWithLock
+  :: (TempResState -> ConfigState -> AtomicModifyMonad ConfigState)
+     -> State TempResState ()
+     -> WConfdMonad (Maybe ())
+modifyConfigWithLock f tempres = do
+  now <- liftIO getClockTime
+  dh <- lift . WConfdMonadInt $ ask
+  pid <- liftIO getProcessID
+  tid <- liftIO myThreadId
+  let cid = ClientId { ciIdentifier = ClientOther $ "wconfd-" ++ show tid
+                     , ciLockFile = dhLivelock dh
+                     , ciPid = pid
+                     }
+  let modCS ds@(DaemonState { dsTempRes = tr }) =
+        mapMOf2
+          dsConfigStateL
+          (\cs -> liftM (unpackConfigResult now cs . (,) ())  (f tr cs))
+          ds
+  maybeDist <- bracket
+    (atomicModifyWithLens (dhDaemonState dh) dsLockWaitingL
+      $ swap . LW.updateLocks cid [LA.requestExclusive ConfigLock])
+    (\(res, _) -> case res of
+        Ok s | S.null s -> do
+          (_, nfy) <- atomicModifyWithLens (dhDaemonState dh) dsLockWaitingL
+                      $ swap . LW.updateLocks cid [LA.requestRelease ConfigLock]
+          unless (S.null nfy) . liftIO . void . forkIO $ do
+            logDebug . (++) "Locks became available for " . show $ S.toList nfy
+            mapM_ (notifyJob . ciPid) $ S.toList nfy
+            logDebug "Finished notifying processes"
+        _ -> return ())
+    (\(res, _) -> case res of
+        Ok s | S.null s ->do
+          ((), modif, dist) <- atomicModifyIORefErrLog (dhDaemonState dh)
+                                 (liftM swap . modCS)
+          atomicModifyWithLens (dhDaemonState dh) dsTempResL $ runState tempres
+          return $ Just (modif, dist)
+        _ -> return Nothing)
+  flip (maybe $ return Nothing) maybeDist $ \(modified, dist) -> do
+    when modified $ do
+      logDebug . (++) "Triggering config write; distribution "
+        $ if dist then "synchronously" else "asynchronously"
+      liftBase . triggerAndWait (Any dist) $ dhSaveConfigWorker dh
+      logDebug "Config write finished"
+    logDebug "Triggering temporary reservations write"
+    liftBase . triggerAndWait_ . dhSaveTempResWorker $ dh
+    logDebug "Temporary reservations write finished"
+    return $ Just ()
