@@ -51,8 +51,12 @@ import Test.Ganeti.HTools.Instance ( genInstanceSmallerThanNode
 import Test.Ganeti.HTools.Node (genOnlineNode, genNode)
 
 import Ganeti.BasicTypes
+import qualified Ganeti.HTools.AlgorithmParams as Alg
 import qualified Ganeti.HTools.Backend.IAlloc as IAlloc
 import qualified Ganeti.HTools.Cluster as Cluster
+import qualified Ganeti.HTools.Cluster.Evacuate as Evacuate
+import qualified Ganeti.HTools.Cluster.Metrics as Metrics
+import qualified Ganeti.HTools.Cluster.Utils as ClusterUtils
 import qualified Ganeti.HTools.Container as Container
 import qualified Ganeti.HTools.Group as Group
 import qualified Ganeti.HTools.Instance as Instance
@@ -80,7 +84,11 @@ isNodeBig size node = Node.availDisk node > size * Types.unitDsk
 canBalance :: Cluster.Table -> Bool -> Bool -> Bool -> Bool
 canBalance tbl@(Cluster.Table _ _ ini_cv _)  dm im evac =
   maybe False (\(Cluster.Table _ _ fin_cv _) -> ini_cv - fin_cv > 1e-12)
-  $ Cluster.tryBalance tbl dm im evac False 0 0
+  $ Cluster.tryBalance (Alg.defaultOptions { Alg.algMinGain = 0.0
+                                           , Alg.algMinGainLimit = 0.0
+                                           , Alg.algDiskMoves = dm
+                                           , Alg.algInstanceMoves = im
+                                           , Alg.algEvacMode = evac}) tbl
 
 -- | Assigns a new fresh instance to a cluster; this is not
 -- allocation, so no resource checks are done.
@@ -123,7 +131,7 @@ prop_Score_Zero node =
      (Node.tSpindles node > 0) && (Node.tCpu node > 0)) ==>
   let fn = Node.buildPeers node Container.empty
       nlst = replicate count fn
-      score = Cluster.compCVNodes nlst
+      score = Metrics.compCVNodes nlst
   -- we can't say == 0 here as the floating point errors accumulate;
   -- this should be much lower than the default score in CLI.hs
   in score <= 1e-12
@@ -148,8 +156,9 @@ prop_Alloc_sane inst =
   forAll genOnlineNode $ \node ->
   let (nl, il, inst') = makeSmallEmptyCluster node count inst
       reqnodes = Instance.requiredNodes $ Instance.diskTemplate inst
+      opts = Alg.defaultOptions
   in case Cluster.genAllocNodes defGroupList nl reqnodes True >>=
-     Cluster.tryAlloc nl il inst' of
+     Cluster.tryAlloc opts nl il inst' of
        Bad msg -> failTest msg
        Ok as ->
          case Cluster.asSolution as of
@@ -160,7 +169,7 @@ prop_Alloc_sane inst =
              in counterexample "Cluster can be balanced after allocation"
                   (not (canBalance tbl True True False)) .&&.
                 counterexample "Solution score differs from actual node list"
-                  (abs (Cluster.compCV xnl - cv) < 1e-12)
+                  (abs (Metrics.compCV xnl - cv) < 1e-12)
 
 -- | Checks that on a 2-5 node cluster, we can allocate a random
 -- instance spec via tiered allocation (whatever the original instance
@@ -176,8 +185,9 @@ prop_CanTieredAlloc =
       il = Container.empty
       rqnodes = Instance.requiredNodes $ Instance.diskTemplate inst
       allocnodes = Cluster.genAllocNodes defGroupList nl rqnodes True
+      opts = Alg.defaultOptions
   in case allocnodes >>= \allocnodes' ->
-    Cluster.tieredAlloc nl il (Just 5) inst allocnodes' [] [] of
+    Cluster.tieredAlloc opts nl il (Just 5) inst allocnodes' [] [] of
        Bad msg -> failTest $ "Failed to tiered alloc: " ++ msg
        Ok (_, nl', il', ixes, cstats) ->
          let (ai_alloc, ai_pool, ai_unav) =
@@ -203,8 +213,9 @@ genClusterAlloc :: Int -> Node.Node -> Instance.Instance
 genClusterAlloc count node inst =
   let nl = makeSmallCluster node count
       reqnodes = Instance.requiredNodes $ Instance.diskTemplate inst
+      opts = Alg.defaultOptions
   in case Cluster.genAllocNodes defGroupList nl reqnodes True >>=
-     Cluster.tryAlloc nl Container.empty inst of
+     Cluster.tryAlloc opts nl Container.empty inst of
        Bad msg -> Bad $ "Can't allocate: " ++ msg
        Ok as ->
          case Cluster.asSolution as of
@@ -223,7 +234,7 @@ prop_AllocRelocate =
   case genClusterAlloc count node inst of
     Bad msg -> failTest msg
     Ok (nl, il, inst') ->
-      case IAlloc.processRelocate defGroupList nl il
+      case IAlloc.processRelocate Alg.defaultOptions defGroupList nl il
              (Instance.idx inst) 1
              [(if Instance.diskTemplate inst' == Types.DTDrbd8
                  then Instance.sNode
@@ -234,15 +245,15 @@ prop_AllocRelocate =
 -- | Helper property checker for the result of a nodeEvac or
 -- changeGroup operation.
 check_EvacMode :: Group.Group -> Instance.Instance
-               -> Result (Node.List, Instance.List, Cluster.EvacSolution)
+               -> Result (Node.List, Instance.List, Evacuate.EvacSolution)
                -> Property
 check_EvacMode grp inst result =
   case result of
     Bad msg -> failTest $ "Couldn't evacuate/change group:" ++ msg
     Ok (_, _, es) ->
-      let moved = Cluster.esMoved es
-          failed = Cluster.esFailed es
-          opcodes = not . null $ Cluster.esOpCodes es
+      let moved = Evacuate.esMoved es
+          failed = Evacuate.esFailed es
+          opcodes = not . null $ Evacuate.esOpCodes es
       in conjoin
            [ failmsg ("'failed' not empty: " ++ show failed) (null failed)
            , failmsg "'opcodes' is null" opcodes
@@ -267,7 +278,8 @@ prop_AllocEvacuate =
     Bad msg -> failTest msg
     Ok (nl, il, inst') ->
       conjoin . map (\mode -> check_EvacMode defGroup inst' $
-                              Cluster.tryNodeEvac defGroupList nl il mode
+                              Evacuate.tryNodeEvac Alg.defaultOptions
+                                defGroupList nl il mode
                                 [Instance.idx inst']) .
                               evacModeOptions .
                               Instance.mirrorType $ inst'
@@ -292,8 +304,9 @@ prop_AllocChangeGroup =
           nl4 = Container.fromList . map (\n -> (Node.idx n, n)) $ nl3
           gl' = Container.add (Group.idx grp2) grp2 defGroupList
           nl' = IntMap.union nl nl4
+          opts = Alg.defaultOptions
       in check_EvacMode grp2 inst' $
-         Cluster.tryChangeGroup gl' nl' il [] [Instance.idx inst']
+         Cluster.tryChangeGroup opts gl' nl' il [] [Instance.idx inst']
 
 -- | Check that allocating multiple instances on a cluster, then
 -- adding an empty node, results in a valid rebalance.
@@ -308,13 +321,14 @@ prop_AllocBalance =
       il = Container.empty
       allocnodes = Cluster.genAllocNodes defGroupList nl' 2 True
       i_templ = createInstance Types.unitMem Types.unitDsk Types.unitCpu
+      opts = Alg.defaultOptions
   in case allocnodes >>= \allocnodes' ->
-    Cluster.iterateAlloc nl' il (Just 5) i_templ allocnodes' [] [] of
+    Cluster.iterateAlloc opts nl' il (Just 5) i_templ allocnodes' [] [] of
        Bad msg -> failTest $ "Failed to allocate: " ++ msg
        Ok (_, _, _, [], _) -> failTest "Failed to allocate: no instances"
        Ok (_, xnl, il', _, _) ->
          let ynl = Container.add (Node.idx hnode) hnode xnl
-             cv = Cluster.compCV ynl
+             cv = Metrics.compCV ynl
              tbl = Cluster.Table ynl il' cv []
          in counterexample "Failed to rebalance" $
             canBalance tbl True True False
@@ -345,7 +359,7 @@ prop_SplitCluster node inst =
   let nl = makeSmallCluster node 2
       (nl', il') = foldl (\(ns, is) _ -> assignInstance ns is inst 0 1)
                    (nl, Container.empty) [1..icnt]
-      gni = Cluster.splitCluster nl' il'
+      gni = ClusterUtils.splitCluster nl' il'
   in sum (map (Container.size . snd . snd) gni) == icnt &&
      all (\(guuid, (nl'', _)) -> all ((== guuid) . Node.group)
                                  (Container.elems nl'')) gni
@@ -356,7 +370,7 @@ prop_SplitCluster node inst =
 canAllocOn :: Node.List -> Int -> Instance.Instance -> Maybe String
 canAllocOn nl reqnodes inst =
   case Cluster.genAllocNodes defGroupList nl reqnodes True >>=
-       Cluster.tryAlloc nl Container.empty inst of
+       Cluster.tryAlloc Alg.defaultOptions nl Container.empty inst of
        Bad msg -> Just $ "Can't allocate: " ++ msg
        Ok as ->
          case Cluster.asSolution as of
