@@ -80,9 +80,10 @@ genInstanceWithin min_mem min_dsk min_cpu min_spin
   spindles <- case max_spin of
     Nothing -> genMaybe $ choose (min_spin, maxSpindles)
     Just ls -> liftM Just $ choose (min_spin, ls)
+  forthcoming <- arbitrary
   let disk = Instance.Disk dsk spindles
   return $ Instance.create
-    name mem dsk [disk] vcpus run_st [] True pn sn dt 1 []
+    name mem dsk [disk] vcpus run_st [] True pn sn dt 1 [] forthcoming
 
 -- | Generate an instance with maximum disk/mem/cpu values.
 genInstanceSmallerThan :: Int -> Int -> Int -> Maybe Int
@@ -96,7 +97,7 @@ genInstanceSmallerThanNode node =
                          (Node.availDisk node `div` 2)
                          (Node.availCpu node `div` 2)
                          (if Node.exclStorage node
-                          then Just $ Node.fSpindles node `div` 2
+                          then Just $ Node.fSpindlesForth node `div` 2
                           else Nothing)
 
 -- | Generates an instance possibly bigger than a node.
