@@ -49,6 +49,7 @@ import Test.Ganeti.Confd.Utils
 import Test.Ganeti.Confd.Types
 import Test.Ganeti.Daemon
 import Test.Ganeti.Errors
+import Test.Ganeti.HTools.Backend.MonD
 import Test.Ganeti.HTools.Backend.Simu
 import Test.Ganeti.HTools.Backend.Text
 import Test.Ganeti.HTools.CLI
@@ -64,6 +65,7 @@ import Test.Ganeti.Hypervisor.Xen.XmParser
 import Test.Ganeti.JSON
 import Test.Ganeti.Jobs
 import Test.Ganeti.JQueue
+import Test.Ganeti.JQScheduler
 import Test.Ganeti.Kvmd
 import Test.Ganeti.Locking.Allocation
 import Test.Ganeti.Locking.Locks
@@ -81,6 +83,7 @@ import Test.Ganeti.Query.Network
 import Test.Ganeti.Query.Query
 import Test.Ganeti.Rpc
 import Test.Ganeti.Runtime
+import Test.Ganeti.SlotMap
 import Test.Ganeti.Ssconf
 import Test.Ganeti.Storage.Diskstats.Parser
 import Test.Ganeti.Storage.Drbd.Parser
@@ -121,6 +124,7 @@ allTests =
   , testBlock_Drbd_Parser
   , testBlock_Drbd_Types
   , testErrors
+  , testHTools_Backend_MonD
   , testHTools_Backend_Simu
   , testHTools_Backend_Text
   , testHTools_CLI
@@ -136,6 +140,7 @@ allTests =
   , testJSON
   , testJobs
   , testJQueue
+  , testJQScheduler
   , testKvmd
   , testLocking_Allocation
   , testLocking_Locks
@@ -153,6 +158,7 @@ allTests =
   , testQuery_Query
   , testRpc
   , testRuntime
+  , testSlotMap
   , testSsconf
   , testStorage_Lvm_LVParser
   , testTHH
