@@ -31,6 +31,7 @@
 """Script for unittesting the cmdlib module"""
 
 
+import mock
 import unittest
 import itertools
 import copy
@@ -39,6 +40,7 @@ from ganeti import constants
 from ganeti import mcpu
 from ganeti import cmdlib
 from ganeti.cmdlib import cluster
+from ganeti.cmdlib.cluster import verify
 from ganeti.cmdlib import instance_storage
 from ganeti.cmdlib import instance_utils
 from ganeti.cmdlib import common
@@ -443,42 +445,42 @@ class TestComputeIPolicySpecViolation(unittest.TestCase):
   def test(self):
     compute_fn = _ValidateComputeMinMaxSpec
     ret = common.ComputeIPolicySpecViolation(self._MICRO_IPOL, 1024, 1, 1, 1,
-                                             [1024], 1, constants.DT_PLAIN,
+                                             [1024], 1, [constants.DT_PLAIN],
                                              _compute_fn=compute_fn)
     self.assertEqual(ret, [])
 
   def testDiskFull(self):
     compute_fn = _NoDiskComputeMinMaxSpec
     ret = common.ComputeIPolicySpecViolation(self._MICRO_IPOL, 1024, 1, 1, 1,
-                                             [1024], 1, constants.DT_PLAIN,
+                                             [1024], 1, [constants.DT_PLAIN],
                                              _compute_fn=compute_fn)
     self.assertEqual(ret, [constants.ISPEC_DISK_COUNT])
 
   def testDiskLess(self):
     compute_fn = _NoDiskComputeMinMaxSpec
-    ret = common.ComputeIPolicySpecViolation(self._MICRO_IPOL, 1024, 1, 1, 1,
-                                             [1024], 1, constants.DT_DISKLESS,
+    ret = common.ComputeIPolicySpecViolation(self._MICRO_IPOL, 1024, 1, 0, 1,
+                                             [], 1, [],
                                              _compute_fn=compute_fn)
     self.assertEqual(ret, [])
 
   def testWrongTemplates(self):
     compute_fn = _ValidateComputeMinMaxSpec
     ret = common.ComputeIPolicySpecViolation(self._MICRO_IPOL, 1024, 1, 1, 1,
-                                             [1024], 1, constants.DT_DRBD8,
+                                             [1024], 1, [constants.DT_DRBD8],
                                              _compute_fn=compute_fn)
     self.assertEqual(len(ret), 1)
     self.assertTrue("Disk template" in ret[0])
 
   def testInvalidArguments(self):
     self.assertRaises(AssertionError, common.ComputeIPolicySpecViolation,
-                      self._MICRO_IPOL, 1024, 1, 1, 1, [], 1,
+                      self._MICRO_IPOL, 1024, 1, 1, 1, constants.DT_DISKLESS, 1,
                       constants.DT_PLAIN,)
 
   def testInvalidSpec(self):
     spec = _SpecWrapper([None, False, "foo", None, "bar", None])
     compute_fn = spec.ComputeMinMaxSpec
     ret = common.ComputeIPolicySpecViolation(self._MICRO_IPOL, 1024, 1, 1, 1,
-                                             [1024], 1, constants.DT_PLAIN,
+                                             [1024], 1, [constants.DT_PLAIN],
                                              _compute_fn=compute_fn)
     self.assertEqual(ret, ["foo", "bar"])
     self.assertFalse(spec.spec)
@@ -537,7 +539,7 @@ class TestComputeIPolicySpecViolation(unittest.TestCase):
       ret = common.ComputeIPolicySpecViolation(ipolicy, mem_size, cpu_count,
                                                disk_count, nic_count,
                                                disk_sizes, spindle_use,
-                                               disk_template)
+                                               [disk_template]*disk_count)
       self.assertEqual(len(ret), violations)
 
     AssertComputeViolation(ipolicy1, 0)
@@ -561,42 +563,57 @@ class TestComputeIPolicyDiskSizesViolation(unittest.TestCase):
   # Minimal policy accepted by _ComputeIPolicyDiskSizesViolation()
   _MICRO_IPOL = {
     constants.IPOLICY_DTS: [constants.DT_PLAIN, constants.DT_DISKLESS],
-    constants.ISPECS_MINMAX: [NotImplemented],
+    constants.ISPECS_MINMAX: [None],
     }
 
+  def MakeDisks(self, *dev_types):
+    return [mock.Mock(dev_type=d) for d in dev_types]
+
   def test(self):
     compute_fn = _ValidateComputeMinMaxSpec
-    ret = common.ComputeIPolicyDiskSizesViolation(self._MICRO_IPOL, [1024],
-                                                  constants.DT_PLAIN,
-                                                  _compute_fn=compute_fn)
+    ret = common.ComputeIPolicyDiskSizesViolation(
+      self._MICRO_IPOL, [1024], self.MakeDisks(constants.DT_PLAIN),
+      _compute_fn=compute_fn)
     self.assertEqual(ret, [])
 
   def testDiskFull(self):
     compute_fn = _NoDiskComputeMinMaxSpec
-    ret = common.ComputeIPolicyDiskSizesViolation(self._MICRO_IPOL, [1024],
-                                                  constants.DT_PLAIN,
-                                                  _compute_fn=compute_fn)
+    ret = common.ComputeIPolicyDiskSizesViolation(
+      self._MICRO_IPOL, [1024], self.MakeDisks(constants.DT_PLAIN),
+      _compute_fn=compute_fn)
     self.assertEqual(ret, [constants.ISPEC_DISK_COUNT])
 
+  def testDisksMixed(self):
+    compute_fn = _ValidateComputeMinMaxSpec
+    ipol = copy.deepcopy(self._MICRO_IPOL)
+    ipol[constants.IPOLICY_DTS].append(constants.DT_DRBD8)
+    ret = common.ComputeIPolicyDiskSizesViolation(
+      ipol, [1024, 1024],
+      self.MakeDisks(constants.DT_DRBD8, constants.DT_PLAIN),
+      _compute_fn=compute_fn)
+    self.assertEqual(ret, [])
+
+
   def testDiskLess(self):
     compute_fn = _NoDiskComputeMinMaxSpec
     ret = common.ComputeIPolicyDiskSizesViolation(self._MICRO_IPOL, [],
-                                                  constants.DT_DISKLESS,
+                                                  [],
                                                   _compute_fn=compute_fn)
     self.assertEqual(ret, [])
 
   def testWrongTemplates(self):
     compute_fn = _ValidateComputeMinMaxSpec
-    ret = common.ComputeIPolicyDiskSizesViolation(self._MICRO_IPOL, [1024],
-                                                  constants.DT_DRBD8,
-                                                  _compute_fn=compute_fn)
+
+    ret = common.ComputeIPolicyDiskSizesViolation(
+      self._MICRO_IPOL, [1024], self.MakeDisks(constants.DT_DRBD8),
+      _compute_fn=compute_fn)
     self.assertEqual(len(ret), 1)
     self.assertTrue("Disk template" in ret[0])
 
-  def _AssertComputeViolation(self, ipolicy, disk_sizes, disk_template,
+  def _AssertComputeViolation(self, ipolicy, disk_sizes, dev_types,
                               violations):
-    ret = common.ComputeIPolicyDiskSizesViolation(ipolicy, disk_sizes,
-                                                  disk_template)
+    ret = common.ComputeIPolicyDiskSizesViolation(
+      ipolicy, disk_sizes, self.MakeDisks(*dev_types))
     self.assertEqual(len(ret), violations)
 
   def testWithIPolicy(self):
@@ -624,11 +641,13 @@ class TestComputeIPolicyDiskSizesViolation(unittest.TestCase):
       constants.IPOLICY_DTS: [disk_template],
       }
 
-    self._AssertComputeViolation(ipolicy, [512], disk_template, 0)
-    self._AssertComputeViolation(ipolicy, [], disk_template, 1)
-    self._AssertComputeViolation(ipolicy, [512, 512], disk_template, 1)
-    self._AssertComputeViolation(ipolicy, [511], disk_template, 1)
-    self._AssertComputeViolation(ipolicy, [513], disk_template, 1)
+    self._AssertComputeViolation(ipolicy, [512], [disk_template], 0)
+    self._AssertComputeViolation(ipolicy, [], [disk_template], 1)
+    self._AssertComputeViolation(ipolicy, [], [], 1)
+    self._AssertComputeViolation(ipolicy, [512, 512],
+                                 [disk_template, disk_template], 1)
+    self._AssertComputeViolation(ipolicy, [511], [disk_template], 1)
+    self._AssertComputeViolation(ipolicy, [513], [disk_template], 1)
 
 
 class _StubComputeIPolicySpecViolation:
@@ -675,40 +694,63 @@ class _FakeConfigForComputeIPolicyInstanceViolation:
     return ("pnode_uuid", )
 
   def GetInstanceDisks(self, _):
-    return [objects.Disk(size=512, spindles=13, uuid="disk_uuid")]
+    return [objects.Disk(size=512, spindles=13, uuid="disk_uuid",
+                         dev_type=constants.DT_PLAIN)]
 
 
 class TestComputeIPolicyInstanceViolation(unittest.TestCase):
-  def test(self):
-    beparams = {
+  def setUp(self):
+    self.beparams = {
       constants.BE_MAXMEM: 2048,
       constants.BE_VCPUS: 2,
       constants.BE_SPINDLE_USE: 4,
       }
-    cfg = _FakeConfigForComputeIPolicyInstanceViolation(beparams, False)
-    instance = objects.Instance(beparams=beparams, disks=["disk_uuid"],
+    self.cfg = _FakeConfigForComputeIPolicyInstanceViolation(
+        self.beparams, False)
+    self.cfg_exclusive = _FakeConfigForComputeIPolicyInstanceViolation(
+        self.beparams, True)
+    self.stub = mock.MagicMock()
+    self.stub.return_value = []
+
+  def testPlain(self):
+    instance = objects.Instance(beparams=self.beparams, disks=["disk_uuid"],
                                 nics=[], primary_node="pnode_uuid",
                                 disk_template=constants.DT_PLAIN)
-    stub = _StubComputeIPolicySpecViolation(2048, 2, 1, 0, [512], 4,
-                                            constants.DT_PLAIN)
-    ret = common.ComputeIPolicyInstanceViolation(NotImplemented, instance,
-                                                 cfg, _compute_fn=stub)
+    ret = common.ComputeIPolicyInstanceViolation(
+        NotImplemented, instance, self.cfg, _compute_fn=self.stub)
     self.assertEqual(ret, [])
-    instance2 = objects.Instance(beparams={}, disks=["disk_uuid"],
+    self.stub.assert_called_with(NotImplemented, 2048, 2, 1, 0, [512], 4,
+                                 [constants.DT_PLAIN])
+
+  def testNoBeparams(self):
+    instance = objects.Instance(beparams={}, disks=["disk_uuid"],
                                  nics=[], primary_node="pnode_uuid",
                                  disk_template=constants.DT_PLAIN)
-    ret = common.ComputeIPolicyInstanceViolation(NotImplemented, instance2,
-                                                 cfg, _compute_fn=stub)
+    ret = common.ComputeIPolicyInstanceViolation(
+        NotImplemented, instance, self.cfg, _compute_fn=self.stub)
     self.assertEqual(ret, [])
-    cfg_es = _FakeConfigForComputeIPolicyInstanceViolation(beparams, True)
-    stub_es = _StubComputeIPolicySpecViolation(2048, 2, 1, 0, [512], 13,
-                                               constants.DT_PLAIN)
-    ret = common.ComputeIPolicyInstanceViolation(NotImplemented, instance,
-                                                 cfg_es, _compute_fn=stub_es)
+    self.stub.assert_called_with(NotImplemented, 2048, 2, 1, 0, [512], 4,
+                                 [constants.DT_PLAIN])
+
+  def testExclusiveStorage(self):
+    instance = objects.Instance(beparams=self.beparams, disks=["disk_uuid"],
+                                nics=[], primary_node="pnode_uuid",
+                                disk_template=constants.DT_PLAIN)
+    ret = common.ComputeIPolicyInstanceViolation(
+        NotImplemented, instance, self.cfg_exclusive, _compute_fn=self.stub)
     self.assertEqual(ret, [])
-    ret = common.ComputeIPolicyInstanceViolation(NotImplemented, instance2,
-                                                 cfg_es, _compute_fn=stub_es)
+    self.stub.assert_called_with(NotImplemented, 2048, 2, 1, 0, [512], 13,
+                                 [constants.DT_PLAIN])
+
+  def testExclusiveStorageNoBeparams(self):
+    instance = objects.Instance(beparams={}, disks=["disk_uuid"],
+                                 nics=[], primary_node="pnode_uuid",
+                                 disk_template=constants.DT_PLAIN)
+    ret = common.ComputeIPolicyInstanceViolation(
+        NotImplemented, instance, self.cfg_exclusive, _compute_fn=self.stub)
     self.assertEqual(ret, [])
+    self.stub.assert_called_with(NotImplemented, 2048, 2, 1, 0, [512], 13,
+                                 [constants.DT_PLAIN])
 
 
 class _CallRecorder:
@@ -787,9 +829,9 @@ class _OpTestVerifyErrors(opcodes.OpCode):
     ]
 
 
-class _LuTestVerifyErrors(cluster._VerifyErrors):
+class _LuTestVerifyErrors(verify._VerifyErrors):
   def __init__(self, **kwargs):
-    cluster._VerifyErrors.__init__(self)
+    super(_LuTestVerifyErrors, self).__init__()
     self.op = _OpTestVerifyErrors(**kwargs)
     self.op.Validate(True)
     self.msglist = []
