@@ -76,134 +76,6 @@ class TestParseTimespec(unittest.TestCase):
       self.failUnlessRaises(OpPrereqError, cli.ParseTimespec, value)
 
 
-class TestSplitKeyVal(unittest.TestCase):
-  """Testing case for cli._SplitKeyVal"""
-  DATA = "a=b,c,no_d,-e"
-  RESULT = {"a": "b", "c": True, "d": False, "e": None}
-  RESULT_NOPREFIX = {"a": "b", "c": {}, "no_d": {}, "-e": {}}
-
-  def testSplitKeyVal(self):
-    """Test splitting"""
-    self.failUnlessEqual(cli._SplitKeyVal("option", self.DATA, True),
-                         self.RESULT)
-
-  def testDuplicateParam(self):
-    """Test duplicate parameters"""
-    for data in ("a=1,a=2", "a,no_a"):
-      self.failUnlessRaises(ParameterError, cli._SplitKeyVal,
-                            "option", data, True)
-
-  def testEmptyData(self):
-    """Test how we handle splitting an empty string"""
-    self.failUnlessEqual(cli._SplitKeyVal("option", "", True), {})
-
-
-class TestIdentKeyVal(unittest.TestCase):
-  """Testing case for cli.check_ident_key_val"""
-
-  def testIdentKeyVal(self):
-    """Test identkeyval"""
-    def cikv(value):
-      return cli.check_ident_key_val("option", "opt", value)
-
-    self.assertEqual(cikv("foo:bar"), ("foo", {"bar": True}))
-    self.assertEqual(cikv("foo:bar=baz"), ("foo", {"bar": "baz"}))
-    self.assertEqual(cikv("bar:b=c,c=a"), ("bar", {"b": "c", "c": "a"}))
-    self.assertEqual(cikv("no_bar"), ("bar", False))
-    self.assertRaises(ParameterError, cikv, "no_bar:foo")
-    self.assertRaises(ParameterError, cikv, "no_bar:foo=baz")
-    self.assertRaises(ParameterError, cikv, "bar:foo=baz,foo=baz")
-    self.assertEqual(cikv("-foo"), ("foo", None))
-    self.assertRaises(ParameterError, cikv, "-foo:a=c")
-
-    # Check negative numbers
-    self.assertEqual(cikv("-1:remove"), ("-1", {
-      "remove": True,
-      }))
-    self.assertEqual(cikv("-29447:add,size=4G"), ("-29447", {
-      "add": True,
-      "size": "4G",
-      }))
-    for i in ["-:", "-"]:
-      self.assertEqual(cikv(i), ("", None))
-
-  @staticmethod
-  def _csikv(value):
-    return cli._SplitIdentKeyVal("opt", value, False)
-
-  def testIdentKeyValNoPrefix(self):
-    """Test identkeyval without prefixes"""
-    test_cases = [
-      ("foo:bar", None),
-      ("foo:no_bar", None),
-      ("foo:bar=baz,bar=baz", None),
-      ("foo",
-       ("foo", {})),
-      ("foo:bar=baz",
-       ("foo", {"bar": "baz"})),
-      ("no_foo:-1=baz,no_op=3",
-       ("no_foo", {"-1": "baz", "no_op": "3"})),
-      ]
-    for (arg, res) in test_cases:
-      if res is None:
-        self.assertRaises(ParameterError, self._csikv, arg)
-      else:
-        self.assertEqual(self._csikv(arg), res)
-
-
-class TestMultilistIdentKeyVal(unittest.TestCase):
-  """Test for cli.check_multilist_ident_key_val()"""
-
-  @staticmethod
-  def _cmikv(value):
-    return cli.check_multilist_ident_key_val("option", "opt", value)
-
-  def testListIdentKeyVal(self):
-    test_cases = [
-      ("",
-       None),
-      ("foo", [
-        {"foo": {}}
-        ]),
-      ("foo:bar=baz", [
-        {"foo": {"bar": "baz"}}
-        ]),
-      ("foo:bar=baz/foo:bat=bad",
-       None),
-      ("foo:abc=42/bar:def=11", [
-        {"foo": {"abc": "42"},
-         "bar": {"def": "11"}}
-        ]),
-      ("foo:abc=42/bar:def=11,ghi=07", [
-        {"foo": {"abc": "42"},
-         "bar": {"def": "11", "ghi": "07"}}
-        ]),
-      ("foo:abc=42/bar:def=11//",
-       None),
-      ("foo:abc=42/bar:def=11,ghi=07//foobar", [
-        {"foo": {"abc": "42"},
-         "bar": {"def": "11", "ghi": "07"}},
-        {"foobar": {}}
-        ]),
-      ("foo:abc=42/bar:def=11,ghi=07//foobar:xyz=88", [
-        {"foo": {"abc": "42"},
-         "bar": {"def": "11", "ghi": "07"}},
-        {"foobar": {"xyz": "88"}}
-        ]),
-      ("foo:abc=42/bar:def=11,ghi=07//foobar:xyz=88/foo:uvw=314", [
-        {"foo": {"abc": "42"},
-         "bar": {"def": "11", "ghi": "07"}},
-        {"foobar": {"xyz": "88"},
-         "foo": {"uvw": "314"}}
-        ]),
-      ]
-    for (arg, res) in test_cases:
-      if res is None:
-        self.assertRaises(ParameterError, self._cmikv, arg)
-      else:
-        self.assertEqual(res, self._cmikv(arg))
-
-
 class TestToStream(unittest.TestCase):
   """Test the ToStream functions"""
 
@@ -802,14 +674,6 @@ class TestParseFields(unittest.TestCase):
                      ["def", "ault", "name", "foo"])
 
 
-class TestConstants(unittest.TestCase):
-  def testPriority(self):
-    self.assertEqual(set(cli._PRIONAME_TO_VALUE.values()),
-                     set(constants.OP_PRIO_SUBMIT_VALID))
-    self.assertEqual(list(value for _, value in cli._PRIORITY_NAMES),
-                     sorted(constants.OP_PRIO_SUBMIT_VALID, reverse=True))
-
-
 class TestParseNicOption(unittest.TestCase):
   def test(self):
     self.assertEqual(cli.ParseNicOption([("0", { "link": "eth0", })]),
