@@ -50,8 +50,11 @@ from ganeti.hypervisor import hv_kvm
 import ganeti.hypervisor.hv_kvm.netdev as netdev
 import ganeti.hypervisor.hv_kvm.monitor as monitor
 
+import mock
 import testutils
 
+from testutils.config_mock import ConfigMock
+
 
 class QmpStub(threading.Thread):
   """Stub for a QMP endpoint for a KVM instance
@@ -244,7 +247,7 @@ class TestQmp(testutils.GanetiTestCase):
 
 
 class TestConsole(unittest.TestCase):
-  def _Test(self, instance, node, group, hvparams):
+  def MakeConsole(self, instance, node, group, hvparams):
     cons = hv_kvm.KVMHypervisor.GetInstanceConsole(instance, node, group,
                                                    hvparams, {})
     self.assertEqual(cons.Validate(), None)
@@ -261,7 +264,7 @@ class TestConsole(unittest.TestCase):
       constants.HV_VNC_BIND_ADDRESS: None,
       constants.HV_KVM_SPICE_BIND: None,
       }
-    cons = self._Test(instance, node, group, hvparams)
+    cons = self.MakeConsole(instance, node, group, hvparams)
     self.assertEqual(cons.kind, constants.CONS_SSH)
     self.assertEqual(cons.host, node.name)
     self.assertEqual(cons.command[0], pathutils.KVM_CONSOLE_WRAPPER)
@@ -279,7 +282,7 @@ class TestConsole(unittest.TestCase):
       constants.HV_VNC_BIND_ADDRESS: "192.0.2.1",
       constants.HV_KVM_SPICE_BIND: None,
       }
-    cons = self._Test(instance, node, group, hvparams)
+    cons = self.MakeConsole(instance, node, group, hvparams)
     self.assertEqual(cons.kind, constants.CONS_VNC)
     self.assertEqual(cons.host, "192.0.2.1")
     self.assertEqual(cons.port, constants.VNC_BASE_PORT + 10)
@@ -297,7 +300,7 @@ class TestConsole(unittest.TestCase):
       constants.HV_VNC_BIND_ADDRESS: None,
       constants.HV_KVM_SPICE_BIND: "192.0.2.1",
       }
-    cons = self._Test(instance, node, group, hvparams)
+    cons = self.MakeConsole(instance, node, group, hvparams)
     self.assertEqual(cons.kind, constants.CONS_SPICE)
     self.assertEqual(cons.host, "192.0.2.1")
     self.assertEqual(cons.port, 11000)
@@ -314,78 +317,94 @@ class TestConsole(unittest.TestCase):
       constants.HV_VNC_BIND_ADDRESS: None,
       constants.HV_KVM_SPICE_BIND: None,
       }
-    cons = self._Test(instance, node, group, hvparams)
+    cons = self.MakeConsole(instance, node, group, hvparams)
     self.assertEqual(cons.kind, constants.CONS_MESSAGE)
 
 
 class TestVersionChecking(testutils.GanetiTestCase):
-  def testParseVersion(self):
-    parse = hv_kvm.KVMHypervisor._ParseKVMVersion
-    help_112 = testutils.ReadTestData("kvm_1.1.2_help.txt")
-    help_10 = testutils.ReadTestData("kvm_1.0_help.txt")
-    help_01590 = testutils.ReadTestData("kvm_0.15.90_help.txt")
-    help_0125 = testutils.ReadTestData("kvm_0.12.5_help.txt")
-    help_091 = testutils.ReadTestData("kvm_0.9.1_help.txt")
-    self.assertEqual(parse(help_112), ("1.1.2", 1, 1, 2))
-    self.assertEqual(parse(help_10), ("1.0", 1, 0, 0))
-    self.assertEqual(parse(help_01590), ("0.15.90", 0, 15, 90))
-    self.assertEqual(parse(help_0125), ("0.12.5", 0, 12, 5))
-    self.assertEqual(parse(help_091), ("0.9.1", 0, 9, 1))
+  @staticmethod
+  def ParseTestData(name):
+    help = testutils.ReadTestData(name)
+    return hv_kvm.KVMHypervisor._ParseKVMVersion(help)
+
+  def testParseVersion112(self):
+    self.assertEqual(
+        self.ParseTestData("kvm_1.1.2_help.txt"), ("1.1.2", 1, 1, 2))
+
+  def testParseVersion10(self):
+    self.assertEqual(self.ParseTestData("kvm_1.0_help.txt"), ("1.0", 1, 0, 0))
+
+  def testParseVersion01590(self):
+    self.assertEqual(
+        self.ParseTestData("kvm_0.15.90_help.txt"), ("0.15.90", 0, 15, 90))
+
+  def testParseVersion0125(self):
+    self.assertEqual(
+        self.ParseTestData("kvm_0.12.5_help.txt"), ("0.12.5", 0, 12, 5))
+
+  def testParseVersion091(self):
+    self.assertEqual(
+        self.ParseTestData("kvm_0.9.1_help.txt"), ("0.9.1", 0, 9, 1))
 
 
 class TestSpiceParameterList(unittest.TestCase):
-  def test(self):
-    defaults = constants.HVC_DEFAULTS[constants.HT_KVM]
+  def setUp(self):
+    self.defaults = constants.HVC_DEFAULTS[constants.HT_KVM]
 
-    params = \
-      compat.UniqueFrozenset(getattr(constants, name)
-                             for name in dir(constants)
-                             if name.startswith("HV_KVM_SPICE_"))
+  def testAudioCompressionDefaultOn(self):
+    self.assertTrue(self.defaults[constants.HV_KVM_SPICE_AUDIO_COMPR])
 
-    # Parameters whose default value evaluates to True and don't need to be set
-    defaults_true = frozenset(filter(defaults.__getitem__, params))
+  def testVdAgentDefaultOn(self):
+    self.assertTrue(self.defaults[constants.HV_KVM_SPICE_USE_VDAGENT])
 
-    self.assertEqual(defaults_true, frozenset([
-      constants.HV_KVM_SPICE_AUDIO_COMPR,
-      constants.HV_KVM_SPICE_USE_VDAGENT,
-      constants.HV_KVM_SPICE_TLS_CIPHERS,
-      ]))
+  def testTlsCiphersDefaultOn(self):
+    self.assertTrue(self.defaults[constants.HV_KVM_SPICE_TLS_CIPHERS])
 
-    # HV_KVM_SPICE_BIND decides whether the other parameters must be set if
-    # their default evaluates to False
-    assert constants.HV_KVM_SPICE_BIND in params
-    assert constants.HV_KVM_SPICE_BIND not in defaults_true
+  def testBindDefaultOff(self):
+    self.assertFalse(self.defaults[constants.HV_KVM_SPICE_BIND])
 
-    # Exclude some parameters
-    params -= defaults_true | frozenset([
-      constants.HV_KVM_SPICE_BIND,
-      ])
-
-    self.assertEqual(hv_kvm._SPICE_ADDITIONAL_PARAMS, params)
+  def testAdditionalParams(self):
+    params = compat.UniqueFrozenset(
+        getattr(constants, name)
+        for name in dir(constants)
+        if name.startswith("HV_KVM_SPICE_"))
+    fixed = set([
+        constants.HV_KVM_SPICE_BIND, constants.HV_KVM_SPICE_TLS_CIPHERS,
+        constants.HV_KVM_SPICE_USE_VDAGENT, constants.HV_KVM_SPICE_AUDIO_COMPR])
+    self.assertEqual(hv_kvm._SPICE_ADDITIONAL_PARAMS, params - fixed)
 
 
 class TestHelpRegexps(testutils.GanetiTestCase):
-  def testBootRe(self):
-    """Check _BOOT_RE
+  """Check _BOOT_RE
 
-    It has too match -drive.*boot=on|off except if there is another dash-option
-    at the beginning of the line.
+  It has to match -drive.*boot=on|off except if there is another dash-option
+  at the beginning of the line.
 
-    """
+  """
+
+  @staticmethod
+  def SearchTestData(name):
     boot_re = hv_kvm.KVMHypervisor._BOOT_RE
-    help_112 = testutils.ReadTestData("kvm_1.1.2_help.txt")
-    help_10 = testutils.ReadTestData("kvm_1.0_help.txt")
-    help_01590 = testutils.ReadTestData("kvm_0.15.90_help.txt")
-    help_0125 = testutils.ReadTestData("kvm_0.12.5_help.txt")
-    help_091 = testutils.ReadTestData("kvm_0.9.1_help.txt")
-    help_091_fake = testutils.ReadTestData("kvm_0.9.1_help_boot_test.txt")
+    help = testutils.ReadTestData(name)
+    return boot_re.search(help)
 
-    self.assertTrue(boot_re.search(help_091))
-    self.assertTrue(boot_re.search(help_0125))
-    self.assertFalse(boot_re.search(help_091_fake))
-    self.assertFalse(boot_re.search(help_112))
-    self.assertFalse(boot_re.search(help_10))
-    self.assertFalse(boot_re.search(help_01590))
+  def testBootRe112(self):
+    self.assertFalse(self.SearchTestData("kvm_1.1.2_help.txt"))
+
+  def testBootRe10(self):
+    self.assertFalse(self.SearchTestData("kvm_1.0_help.txt"))
+
+  def testBootRe01590(self):
+    self.assertFalse(self.SearchTestData("kvm_0.15.90_help.txt"))
+
+  def testBootRe0125(self):
+    self.assertTrue(self.SearchTestData("kvm_0.12.5_help.txt"))
+
+  def testBootRe091(self):
+    self.assertTrue(self.SearchTestData("kvm_0.9.1_help.txt"))
+
+  def testBootRe091_fake(self):
+    self.assertFalse(self.SearchTestData("kvm_0.9.1_help_boot_test.txt"))
 
 
 class TestGetTunFeatures(unittest.TestCase):
@@ -485,5 +504,73 @@ class TestGetRuntimeInfo(unittest.TestCase):
     self.assertTrue(devinfo.pci==5)
 
 
+class PostfixMatcher(object):
+  def __init__(self, string):
+    self.string = string
+
+  def __eq__(self, other):
+    return other.endswith(self.string)
+
+  def __repr__(self):
+    return "<Postfix %s>" % self.string
+
+class TestKvmRuntime(testutils.GanetiTestCase):
+  """The _ExecuteKvmRuntime is at the core of all KVM operations."""
+
+  def setUp(self):
+    super(TestKvmRuntime, self).setUp()
+    kvm_class = 'ganeti.hypervisor.hv_kvm.KVMHypervisor'
+    self.MockOut('qmp', mock.patch('ganeti.hypervisor.hv_kvm.QmpConnection'))
+    self.MockOut('run_cmd', mock.patch('ganeti.utils.RunCmd'))
+    self.MockOut('ensure_dirs', mock.patch('ganeti.utils.EnsureDirs'))
+    self.MockOut('write_file', mock.patch('ganeti.utils.WriteFile'))
+    self.MockOut(mock.patch(kvm_class + '.ValidateParameters'))
+    self.MockOut(mock.patch('ganeti.hypervisor.hv_kvm.OpenTap',
+                            return_value=('test_nic', [], [])))
+    self.MockOut(mock.patch(kvm_class + '._ConfigureNIC'))
+    self.MockOut('pid_alive', mock.patch(kvm_class + '._InstancePidAlive',
+                                         return_value=('file', -1, False)))
+    self.MockOut(mock.patch(kvm_class + '._ExecuteCpuAffinity'))
+    self.MockOut(mock.patch(kvm_class + '._CallMonitorCommand'))
+
+    self.cfg = ConfigMock()
+    params = constants.HVC_DEFAULTS[constants.HT_KVM].copy()
+    beparams = constants.BEC_DEFAULTS.copy()
+    self.instance = self.cfg.AddNewInstance(name='name.example.com',
+                                            hypervisor='kvm',
+                                            hvparams=params,
+                                            beparams=beparams)
+
+  def testDirectoriesCreated(self):
+    hypervisor = hv_kvm.KVMHypervisor()
+    self.mocks['ensure_dirs'].assert_called_with([
+        (PostfixMatcher('/run/ganeti/kvm-hypervisor'), 0775),
+        (PostfixMatcher('/run/ganeti/kvm-hypervisor/pid'), 0775),
+        (PostfixMatcher('/run/ganeti/kvm-hypervisor/uid'), 0775),
+        (PostfixMatcher('/run/ganeti/kvm-hypervisor/ctrl'), 0775),
+        (PostfixMatcher('/run/ganeti/kvm-hypervisor/conf'), 0775),
+        (PostfixMatcher('/run/ganeti/kvm-hypervisor/nic'), 0775),
+        (PostfixMatcher('/run/ganeti/kvm-hypervisor/chroot'), 0775),
+        (PostfixMatcher('/run/ganeti/kvm-hypervisor/chroot-quarantine'), 0775),
+        (PostfixMatcher('/run/ganeti/kvm-hypervisor/keymap'), 0775)])
+
+  def testStartInstance(self):
+    hypervisor = hv_kvm.KVMHypervisor()
+    def RunCmd(cmd, **kwargs):
+      if '--help' in cmd:
+        return mock.Mock(
+            failed=False, output=testutils.ReadTestData("kvm_1.1.2_help.txt"))
+      if '-S' in cmd:
+        self.mocks['pid_alive'].return_value = ('file', -1, True)
+        return mock.Mock(failed=False)
+      elif '-M' in cmd:
+        return mock.Mock(failed=False, output='')
+      elif '-device' in cmd:
+        return mock.Mock(failed=False, output='name "virtio-blk-pci"')
+      else:
+        raise errors.ProgrammerError('Unexpected command: %s' % cmd)
+    self.mocks['run_cmd'].side_effect = RunCmd
+    hypervisor.StartInstance(self.instance, [], False)
+
 if __name__ == "__main__":
   testutils.GanetiTestProgram()
