@@ -182,6 +182,20 @@ class TestInstanceStateParsing(unittest.TestCase):
       "--p--d",
       "------",
       "--p---",
+      "r--ss-",
+      "r-pss-",
+      "rb-ss-",
+      "rbpss-",
+      "-b-ss-",
+      "-bpss-",
+      "---ss-",
+      "r--sr-",
+      "r-psr-",
+      "rb-sr-",
+      "rbpsr-",
+      "-b-sr-",
+      "-bpsr-",
+      "---sr-",
     ]
     for state in states:
       self.assertEqual(hv_xen._XenToHypervisorInstanceState(state),
