@@ -55,7 +55,6 @@ REQ_BGL_WHITELIST = compat.UniqueFrozenset([
   opcodes.OpClusterDestroy,
   opcodes.OpClusterPostInit,
   opcodes.OpClusterRename,
-  opcodes.OpClusterRenewCrypto,
   opcodes.OpNodeAdd,
   opcodes.OpNodeRemove,
   opcodes.OpTestAllocator,
@@ -177,115 +176,5 @@ class TestProcessResult(unittest.TestCase):
     self.assertEqual(op2.debug_level, 3)
 
 
-class _FakeLuWithLocks:
-  def __init__(self, needed_locks, share_locks):
-    self.needed_locks = needed_locks
-    self.share_locks = share_locks
-    self.locks = []
-
-  def owned_locks(self, *_):
-    return self.locks
-
-
-class TestVerifyLocks(unittest.TestCase):
-  def testNoLocks(self):
-    lu = _FakeLuWithLocks({}, {})
-    mcpu._VerifyLocks(lu,
-                      _mode_whitelist=NotImplemented,
-                      _nal_whitelist=NotImplemented)
-
-  def testNotAllSameMode(self):
-    for level in [locking.LEVEL_NODE, locking.LEVEL_NODE_RES]:
-      lu = _FakeLuWithLocks({
-        level: ["foo"],
-        }, {
-        level: 0,
-        locking.LEVEL_NODE_ALLOC: 0,
-        })
-      mcpu._VerifyLocks(lu, _mode_whitelist=[], _nal_whitelist=[])
-
-  def testDifferentMode(self):
-    for level in [locking.LEVEL_NODE, locking.LEVEL_NODE_RES]:
-      lu = _FakeLuWithLocks({
-        level: ["foo"],
-        }, {
-        level: 0,
-        locking.LEVEL_NODE_ALLOC: 1,
-        })
-      try:
-        mcpu._VerifyLocks(lu, _mode_whitelist=[], _nal_whitelist=[])
-      except AssertionError, err:
-        self.assertTrue("using the same mode as nodes" in str(err))
-      else:
-        self.fail("Exception not raised")
-
-      # Once more with the whitelist
-      mcpu._VerifyLocks(lu, _mode_whitelist=[_FakeLuWithLocks],
-                        _nal_whitelist=[])
-
-  def testSameMode(self):
-    for level in [locking.LEVEL_NODE, locking.LEVEL_NODE_RES]:
-      lu = _FakeLuWithLocks({
-        level: ["foo"],
-        locking.LEVEL_NODE_ALLOC: locking.ALL_SET,
-        }, {
-        level: 1,
-        locking.LEVEL_NODE_ALLOC: 1,
-        })
-
-      try:
-        mcpu._VerifyLocks(lu, _mode_whitelist=[_FakeLuWithLocks],
-                          _nal_whitelist=[])
-      except AssertionError, err:
-        self.assertTrue("whitelisted to use different modes" in str(err))
-      else:
-        self.fail("Exception not raised")
-
-      # Once more without the whitelist
-      mcpu._VerifyLocks(lu, _mode_whitelist=[], _nal_whitelist=[])
-
-  def testAllWithoutAllocLock(self):
-    for level in [locking.LEVEL_NODE, locking.LEVEL_NODE_RES]:
-      lu = _FakeLuWithLocks({
-        level: locking.ALL_SET,
-        }, {
-        level: 0,
-        locking.LEVEL_NODE_ALLOC: 0,
-        })
-
-      try:
-        mcpu._VerifyLocks(lu, _mode_whitelist=[], _nal_whitelist=[])
-      except AssertionError, err:
-        self.assertTrue("allocation lock must be used if" in str(err))
-      else:
-        self.fail("Exception not raised")
-
-      # Once more with the whitelist
-      mcpu._VerifyLocks(lu, _mode_whitelist=[],
-                        _nal_whitelist=[_FakeLuWithLocks])
-
-  def testAllWithAllocLock(self):
-    for level in [locking.LEVEL_NODE, locking.LEVEL_NODE_RES]:
-      lu = _FakeLuWithLocks({
-        level: locking.ALL_SET,
-        locking.LEVEL_NODE_ALLOC: locking.ALL_SET,
-        }, {
-        level: 0,
-        locking.LEVEL_NODE_ALLOC: 0,
-        })
-      lu.locks = [locking.NAL]
-
-      try:
-        mcpu._VerifyLocks(lu, _mode_whitelist=[],
-                          _nal_whitelist=[_FakeLuWithLocks])
-      except AssertionError, err:
-        self.assertTrue("whitelisted for not acquiring" in str(err))
-      else:
-        self.fail("Exception not raised")
-
-      # Once more without the whitelist
-      mcpu._VerifyLocks(lu, _mode_whitelist=[], _nal_whitelist=[])
-
-
 if __name__ == "__main__":
   testutils.GanetiTestProgram()
