@@ -33,6 +33,7 @@
 import re
 import unittest
 import random
+import uuid as uuid_module
 
 from ganeti import constants
 from ganeti import utils
@@ -869,8 +870,8 @@ class TestInstanceQuery(unittest.TestCase):
     instbyname = dict((inst.name, inst) for inst in instances)
 
     disk_usage = dict((inst.uuid,
-                       gmi.ComputeDiskSize(inst.disk_template,
-                                           [{"size": disk.size}
+                       gmi.ComputeDiskSize([{"size": disk.size,
+                                             "dev_type": disk.dev_type}
                                            for disk in inst.disks]))
                       for inst in instances)
 
@@ -1306,10 +1307,10 @@ class TestQueryFilter(unittest.TestCase):
         namevalempty = ""
         genval = lambda i: "x%s" % i
         randvals = ["x17361", "x22015", "x13193", "x15215"]
-        if what == constants.QR_EXPORT:
-          namefield = "export"
-        else:
-          namefield = "name"
+        namefield = {
+          constants.QR_EXPORT: "export",
+          constants.QR_FILTER: "uuid",
+        }.get(what, "name")
 
       assert namefield in fielddefs
 
@@ -1385,15 +1386,11 @@ class TestQueryFilter(unittest.TestCase):
     levels_max = query._FilterCompilerHelper._LEVELS_MAX
 
     for (what, fielddefs) in query.ALL_FIELDS.items():
-      if what == constants.QR_JOB:
-        namefield = "id"
-        nameval = 123
-      elif what == constants.QR_EXPORT:
-        namefield = "export"
-        nameval = "value"
-      else:
-        namefield = "name"
-        nameval = "value"
+      namefield, nameval = {
+        constants.QR_JOB: ("id", 123),
+        constants.QR_EXPORT: ("export", "value"),
+        constants.QR_FILTER: ("uuid", str(uuid_module.uuid4())),
+      }.get(what, ("name", "value"))
 
       checks = [
         [], ["="], ["=", "foo"], ["unknownop"], ["!"],
