@@ -725,6 +725,16 @@ class _FakeConfigForRpcRunner:
 
   def __init__(self, cluster=NotImplemented):
     self._cluster = cluster
+    self._disks = [
+      objects.Disk(dev_type=constants.DT_PLAIN, size=4096,
+                   logical_id=("vg", "disk6120"),
+                   uuid="disk_uuid_1"),
+      objects.Disk(dev_type=constants.DT_PLAIN, size=1024,
+                   logical_id=("vg", "disk8508"),
+                   uuid="disk_uuid_2"),
+      ]
+    for disk in self._disks:
+      disk.UpgradeConfig()
 
   def GetNodeInfo(self, name):
     return objects.Node(name=name)
@@ -738,6 +748,12 @@ class _FakeConfigForRpcRunner:
   def GetInstanceDiskParams(self, _):
     return constants.DISK_DT_DEFAULTS
 
+  def GetInstanceSecondaryNodes(self, _):
+    return []
+
+  def GetInstanceDisks(self, _):
+    return self._disks
+
 
 class TestRpcRunner(unittest.TestCase):
   def testUploadFile(self):
@@ -819,12 +835,8 @@ class TestRpcRunner(unittest.TestCase):
           }),
         ],
       disk_template=constants.DT_PLAIN,
-      disks=[
-        objects.Disk(dev_type=constants.DT_PLAIN, size=4096,
-                     logical_id=("vg", "disk6120")),
-        objects.Disk(dev_type=constants.DT_PLAIN, size=1024,
-                     logical_id=("vg", "disk8508")),
-        ])
+      disks=["disk_uuid_1", "disk_uuid_2"]
+      )
     inst.UpgradeConfig()
 
     cfg = _FakeConfigForRpcRunner(cluster=cluster)
@@ -893,21 +905,30 @@ class TestRpcRunner(unittest.TestCase):
     self.assertEqual(result["hvparams"][constants.HT_KVM], {
       constants.HV_BOOT_ORDER: "xyz",
       })
-    self.assertEqual(result["disks"], [{
+    del result["disks_info"][0]["ctime"]
+    del result["disks_info"][0]["mtime"]
+    del result["disks_info"][1]["ctime"]
+    del result["disks_info"][1]["mtime"]
+    self.assertEqual(result["disks_info"], [{
       "dev_type": constants.DT_PLAIN,
       "dynamic_params": {},
       "size": 4096,
       "logical_id": ("vg", "disk6120"),
       "params": constants.DISK_DT_DEFAULTS[inst.disk_template],
+      "serial_no": 1,
+      "uuid": "disk_uuid_1",
       }, {
       "dev_type": constants.DT_PLAIN,
       "dynamic_params": {},
       "size": 1024,
       "logical_id": ("vg", "disk8508"),
       "params": constants.DISK_DT_DEFAULTS[inst.disk_template],
+      "serial_no": 1,
+      "uuid": "disk_uuid_2",
       }])
 
-    self.assertTrue(compat.all(disk.params == {} for disk in inst.disks),
+    inst_disks = cfg.GetInstanceDisks(inst.uuid)
+    self.assertTrue(compat.all(disk.params == {} for disk in inst_disks),
                     msg="Configuration objects were modified")
 
 
