@@ -850,74 +850,6 @@ class TestPidFileFunctions(unittest.TestCase):
     shutil.rmtree(self.dir)
 
 
-class TestSshKeys(testutils.GanetiTestCase):
-  """Test case for the AddAuthorizedKey function"""
-
-  KEY_A = "ssh-dss AAAAB3NzaC1w5256closdj32mZaQU root@key-a"
-  KEY_B = ('command="/usr/bin/fooserver -t --verbose",from="198.51.100.4" '
-           "ssh-dss AAAAB3NzaC1w520smc01ms0jfJs22 root@key-b")
-
-  def setUp(self):
-    testutils.GanetiTestCase.setUp(self)
-    self.tmpname = self._CreateTempFile()
-    handle = open(self.tmpname, "w")
-    try:
-      handle.write("%s\n" % TestSshKeys.KEY_A)
-      handle.write("%s\n" % TestSshKeys.KEY_B)
-    finally:
-      handle.close()
-
-  def testAddingNewKey(self):
-    utils.AddAuthorizedKey(self.tmpname,
-                           "ssh-dss AAAAB3NzaC1kc3MAAACB root@test")
-
-    self.assertFileContent(self.tmpname,
-      "ssh-dss AAAAB3NzaC1w5256closdj32mZaQU root@key-a\n"
-      'command="/usr/bin/fooserver -t --verbose",from="198.51.100.4"'
-      " ssh-dss AAAAB3NzaC1w520smc01ms0jfJs22 root@key-b\n"
-      "ssh-dss AAAAB3NzaC1kc3MAAACB root@test\n")
-
-  def testAddingAlmostButNotCompletelyTheSameKey(self):
-    utils.AddAuthorizedKey(self.tmpname,
-        "ssh-dss AAAAB3NzaC1w5256closdj32mZaQU root@test")
-
-    # Only significant fields are compared, therefore the key won't be
-    # updated/added
-    self.assertFileContent(self.tmpname,
-      "ssh-dss AAAAB3NzaC1w5256closdj32mZaQU root@key-a\n"
-      'command="/usr/bin/fooserver -t --verbose",from="198.51.100.4"'
-      " ssh-dss AAAAB3NzaC1w520smc01ms0jfJs22 root@key-b\n")
-
-  def testAddingExistingKeyWithSomeMoreSpaces(self):
-    utils.AddAuthorizedKey(self.tmpname,
-      "ssh-dss  AAAAB3NzaC1w5256closdj32mZaQU   root@key-a")
-    utils.AddAuthorizedKey(self.tmpname,
-      "ssh-dss AAAAB3NzaC1w520smc01ms0jfJs22")
-
-    self.assertFileContent(self.tmpname,
-      "ssh-dss AAAAB3NzaC1w5256closdj32mZaQU root@key-a\n"
-      'command="/usr/bin/fooserver -t --verbose",from="198.51.100.4"'
-      " ssh-dss AAAAB3NzaC1w520smc01ms0jfJs22 root@key-b\n"
-      "ssh-dss AAAAB3NzaC1w520smc01ms0jfJs22\n")
-
-  def testRemovingExistingKeyWithSomeMoreSpaces(self):
-    utils.RemoveAuthorizedKey(self.tmpname,
-        "ssh-dss  AAAAB3NzaC1w5256closdj32mZaQU   root@key-a")
-
-    self.assertFileContent(self.tmpname,
-      'command="/usr/bin/fooserver -t --verbose",from="198.51.100.4"'
-      " ssh-dss AAAAB3NzaC1w520smc01ms0jfJs22 root@key-b\n")
-
-  def testRemovingNonExistingKey(self):
-    utils.RemoveAuthorizedKey(self.tmpname,
-        "ssh-dss  AAAAB3Nsdfj230xxjxJjsjwjsjdjU   root@test")
-
-    self.assertFileContent(self.tmpname,
-      "ssh-dss AAAAB3NzaC1w5256closdj32mZaQU root@key-a\n"
-      'command="/usr/bin/fooserver -t --verbose",from="198.51.100.4"'
-      " ssh-dss AAAAB3NzaC1w520smc01ms0jfJs22 root@key-b\n")
-
-
 class TestNewUUID(unittest.TestCase):
   """Test case for NewUUID"""
 
